/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.repository;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.openepics.names.repository.model.Discipline;
import org.openepics.names.repository.model.NameStructure;
import org.openepics.names.repository.model.Persistable;
import org.openepics.names.repository.model.Structure;
import org.openepics.names.rest.beans.FieldStructure;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.util.RepositoryUtil;
import org.springframework.stereotype.Repository;

/**
 * Handle discipline information in JPA.
 *
 * @author Lars Johansson
 */
@Repository
public class DisciplineRepository {

    @PersistenceContext
    private EntityManager em;

    /**
     * Count disciplines.
     *
     * @param statuses statuses
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @return count of disciplines
     */
    public Long countDisciplines(Status[] statuses, Boolean deleted,
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who) {
        return countDisciplines(statuses, deleted,
                uuid, name, mnemonic, mnemonicEquivalence, mnemonicPath, description, who,
                Boolean.FALSE);
    }

    /**
     * Count disciplines.
     *
     * @param statuses statuses
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @param includeHistory include history
     * @return count of disciplines
     */
    public Long countDisciplines(Status[] statuses, Boolean deleted,
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who,
            Boolean includeHistory) {

        // note
        //     use of function for mnemonic path
        // where
        //     statuses
        //     deleted
        //     queryFields, queryValues

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Long> cq = cb.createQuery(Long.class);
        Root<Discipline> from = cq.from(Discipline.class);

        cq.where(cb.and(preparePredicatesDisciplines(cb, from, cq.subquery(Long.class), statuses, deleted,
                uuid, name, mnemonic, mnemonicEquivalence, mnemonicPath, description, who,
                includeHistory).toArray(new Predicate[0])));
        cq.select(cb.count(from));

        return em.createQuery(cq).getSingleResult();
    }

    /**
     * Find disciplines.
     *
     * @param status status
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @return list of disciplines
     */
    public List<Discipline> readDisciplines(Status status, Boolean deleted,
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who) {

        return readDisciplines(status != null ? new Status[] {status} : null, deleted,
                uuid, name, mnemonic, mnemonicEquivalence, mnemonicPath, description, who,
                Boolean.FALSE, null, null, null, null);
    }

    /**
     * Find disciplines.
     *
     * @param statuses statuses
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param orderBy order by
     * @param isAsc is ascending
     * @param offset offset
     * @param limit limit
     * @return list of disciplines
     */
    public List<Discipline> readDisciplines(Status[] statuses, Boolean deleted,
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who,
            FieldStructure orderBy, Boolean isAsc, Integer offset, Integer limit) {
        return readDisciplines(statuses, deleted,
                uuid, name, mnemonic, mnemonicEquivalence, mnemonicPath, description, who,
                Boolean.FALSE, orderBy, isAsc, offset, limit);
    }

    /**
     * Find disciplines.
     *
     * @param statuses statuses
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @param includeHistory include history
     * @param orderBy order by
     * @param isAsc is ascending
     * @param offset offset
     * @param limit limit
     * @return list of disciplines
     */
    public List<Discipline> readDisciplines(Status[] statuses, Boolean deleted,
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who,
            Boolean includeHistory, FieldStructure orderBy, Boolean isAsc, Integer offset, Integer limit) {

        // note
        //     use of function for mnemonic path
        // where
        //     statuses
        //     latest, deleted
        //     queryFields, queryValues
        // order
        //     orderBy, isAsc
        // paging
        //     offset, limit

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Discipline> cq = cb.createQuery(Discipline.class);
        Root<Discipline> from = cq.from(Discipline.class);

        cq.where(cb.and(preparePredicatesDisciplines(cb, from, cq.subquery(Long.class), statuses, deleted,
                uuid, name, mnemonic, mnemonicEquivalence, mnemonicPath, description, who,
                includeHistory).toArray(new Predicate[0])));
        cq.select(from);

        if (orderBy != null) {
            if (BooleanUtils.toBoolean(isAsc)) {
                if (FieldStructure.NAME.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_NAME)));
                } else if (FieldStructure.MNEMONIC.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.MNEMONICEQUIVALENCE.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_MNEMONIC_EQUIVALENCE)));
                } else if (FieldStructure.MNEMONICPATH.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldStructure.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_NAME)));
                }
            } else {
                if (FieldStructure.NAME.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_NAME)));
                } else if (FieldStructure.MNEMONIC.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.MNEMONICEQUIVALENCE.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_MNEMONIC_EQUIVALENCE)));
                } else if (FieldStructure.MNEMONICPATH.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldStructure.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_NAME)));
                }
            }
        }

        TypedQuery<Discipline> query = em.createQuery(cq);
        if (offset != null && limit != null) {
            query.setFirstResult(offset * limit);
            query.setMaxResults(limit);
        }

        return query.getResultList();
    }

    /**
     * Prepare predicates for disciplines.
     *
     * @param cb criteria builder
     * @param from criteria query root
     * @param sub sub query to be used if history not included
     * @param statuses statuses
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @param includeHistory include history
     * @return list of predicates
     */
    private List<Predicate> preparePredicatesDisciplines(CriteriaBuilder cb, Root<Discipline> from, Subquery<Long> sub, Status[] statuses, Boolean deleted,
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who,
            Boolean includeHistory) {

        List<Predicate> predicates = new ArrayList<>();

        if (!Boolean.TRUE.equals(includeHistory)) {
            // purpose of Naming to show valid entries
            //     therefore
            //         exclude some values unless history requested
            //         make sure to not exclude present and future values
            //
            // not only
            //     exclude APPROVED and not latest
            //          select * from Structure s where
            //              not(s.status = 'APPROVED' and s.latest = false)
            // not only
            //     exclude earlier than APPROVED and latest (considered history)
            //     include PENDING and not latest           (considered future)
            //          select * from Structure s where
            //                 (not s.id < (select id from structure s2 where s2.uuid = s.uuid and s2.latest = true))
            //              or (s.status = 'PENDING' and s.latest = false)
            // instead
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)
            //         select * from Structure s where
            //        	      (not s.id < (select id from system s2 where s2.uuid = s.uuid and s2.latest = true))
            //             or (not  exists(select id from system s2 where s2.uuid = s.uuid and s2.latest = true))

            Root<Discipline> fromSub = sub.from(Discipline.class);
            sub.where(cb.and(
                    cb.equal(fromSub.get(NameStructure.FIELD_UUID),   from.get(NameStructure.FIELD_UUID)),
                    cb.equal(fromSub.get(NameStructure.FIELD_LATEST), Boolean.TRUE)
                    ));
            sub.select(fromSub.get(Persistable.FIELD_ID));

            Predicate predicateExcludeInclude =
                    cb.or(
                            cb.not(cb.lessThan(from.get(Persistable.FIELD_ID).as(Long.class), sub)),
                            cb.not(cb.exists(sub)));
            predicates.add(predicateExcludeInclude);
        }

        if (statuses != null) {
            List<Predicate> predicatesStatus = new ArrayList<>();
            for (Status status : statuses) {
                predicatesStatus.add(cb.equal(from.get(NameStructure.FIELD_STATUS), status));
            }
            predicates.add(cb.or(predicatesStatus.toArray(new Predicate[0])));
        }
        if (deleted != null) {
            predicates.add(cb.equal(from.get(NameStructure.FIELD_DELETED), deleted));
        }

        // prepare pattern
        //     jpa query characters % and _
        //     remove excess % characters

        if (!StringUtils.isEmpty(uuid)) {
            predicates.add(cb.and(cb.equal(from.get(NameStructure.FIELD_UUID), RepositoryUtil.preparePattern(uuid))));
        }
        if (!StringUtils.isEmpty(name)) {
            predicates.add(cb.and(cb.like(from.get(Structure.FIELD_NAME), RepositoryUtil.preparePattern(name))));
        }
        if (!StringUtils.isEmpty(mnemonic)) {
            predicates.add(cb.and(cb.like(from.get(Structure.FIELD_MNEMONIC), RepositoryUtil.preparePattern(mnemonic))));
        }
        if (!StringUtils.isEmpty(mnemonicEquivalence)) {
            predicates.add(cb.and(cb.like(from.get(Structure.FIELD_MNEMONIC_EQUIVALENCE), RepositoryUtil.preparePattern(mnemonicEquivalence))));
        }
        if (!StringUtils.isEmpty(mnemonicPath)) {
            predicates.add(cb.and(cb.like(from.get(Structure.FIELD_MNEMONIC), RepositoryUtil.preparePattern(mnemonicPath))));
        }
        if (!StringUtils.isEmpty(description)) {
            predicates.add(cb.and(cb.like(from.get(NameStructure.FIELD_DESCRIPTION), RepositoryUtil.preparePattern(description))));
        }
        if (!StringUtils.isEmpty(who)) {
            predicates.add(cb.and(cb.like(from.get(NameStructure.FIELD_REQUESTED_BY), RepositoryUtil.preparePattern(who))));
        }

        return predicates;
    }

    /**
     * Count disciplines history.
     *
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @return count of disciplines
     */
    public Long countDisciplinesHistory(
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who) {

        // note
        //     deleted - null
        //     includeHistory - true
        // where
        //     queryFields, queryValues

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Long> cq = cb.createQuery(Long.class);
        Root<Discipline> from = cq.from(Discipline.class);

        Subquery<String> sub = cq.subquery(String.class);
        Root<Discipline> fromSub = sub.from(Discipline.class);
        sub.where(cb.and(preparePredicatesDisciplines(cb, fromSub, null, null, null,
                uuid, name, mnemonic, mnemonicEquivalence, mnemonicPath, description, who,
                Boolean.TRUE).toArray(new Predicate[0])));
        sub.select(fromSub.get(NameStructure.FIELD_UUID));

        cq.where(cb.and(cb.in(from.get(NameStructure.FIELD_UUID)).value(sub)));
        cq.select(cb.count(from));

        return em.createQuery(cq).getSingleResult();
    }

    /**
     * Find disciplines history.
     *
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @param orderBy order by
     * @param isAsc is ascending
     * @param offset offset
     * @param limit limit
     * @return list of disciplines
     */
    public List<Discipline> readDisciplinesHistory(
            String uuid, String name, String mnemonic, String mnemonicEquivalence, String mnemonicPath, String description, String who,
            FieldStructure orderBy, Boolean isAsc, Integer offset, Integer limit) {

        // note
        //     deleted - null
        //     includeHistory - true
        // where
        //     queryFields, queryValues
        // order
        //     orderBy, isAsc
        // paging
        //     offset, limit

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Discipline> cq = cb.createQuery(Discipline.class);
        Root<Discipline> from = cq.from(Discipline.class);

        Subquery<String> sub = cq.subquery(String.class);
        Root<Discipline> fromSub = sub.from(Discipline.class);
        sub.where(cb.and(preparePredicatesDisciplines(cb, fromSub, null, null, null,
                uuid, name, mnemonic, mnemonicEquivalence, mnemonicPath, description, who,
                Boolean.TRUE).toArray(new Predicate[0])));
        sub.select(fromSub.get(NameStructure.FIELD_UUID));

        cq.where(cb.and(cb.in(from.get(NameStructure.FIELD_UUID)).value(sub)));
        cq.select(from);

        if (orderBy != null) {
            if (BooleanUtils.toBoolean(isAsc)) {
                if (FieldStructure.NAME.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_NAME)));
                } else if (FieldStructure.MNEMONIC.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.MNEMONICEQUIVALENCE.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_MNEMONIC_EQUIVALENCE)));
                } else if (FieldStructure.MNEMONICPATH.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldStructure.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_UUID)), cb.asc(from.get(Persistable.FIELD_ID)));
                }
            } else {
                if (FieldStructure.NAME.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_NAME)));
                } else if (FieldStructure.MNEMONIC.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.MNEMONICEQUIVALENCE.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_MNEMONIC_EQUIVALENCE)));
                } else if (FieldStructure.MNEMONICPATH.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Structure.FIELD_MNEMONIC)));
                } else if (FieldStructure.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldStructure.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_UUID)), cb.desc(from.get(Persistable.FIELD_ID)));
                }
            }
        }

        TypedQuery<Discipline> query = em.createQuery(cq);
        if (offset != null && limit != null) {
            query.setFirstResult(offset * limit);
            query.setMaxResults(limit);
        }

        return query.getResultList();
    }

    /**
     * Persist discipline into persistence context.
     *
     * @param discipline discipline
     */
    public void createDiscipline(Discipline discipline) {
        em.persist(discipline);
    }

    /**
     * Merge discipline into persistence context.
     *
     * @param discipline discipline
     */
    public void updateDiscipline(Discipline discipline) {
        em.merge(discipline);
    }

}
