/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.repository;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.openepics.names.repository.model.Name;
import org.openepics.names.repository.model.NameStructure;
import org.openepics.names.repository.model.Persistable;
import org.openepics.names.rest.beans.FieldName;
import org.openepics.names.util.RepositoryUtil;
import org.springframework.stereotype.Repository;

/**
 * Handle name information in JPA.
 *
 * @author Lars Johansson
 */
@Repository
public class NameRepository {

    @PersistenceContext
    private EntityManager em;

    /**
     * Count names.
     *
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @return count of names
     */
    public Long countNames(Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who) {
        return countNames(deleted,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                Boolean.FALSE);
    }

    /**
     * Count names.
     *
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @param includeHistory include history
     * @return count of names
     */
    public Long countNames(Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            Boolean includeHistory) {

        // note
        //     use of function for mnemonic path
        // where
        //     latest, deleted
        //     queryFields, queryValues

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Long> cq = cb.createQuery(Long.class);
        Root<Name> from = cq.from(Name.class);

        cq.where(cb.and(preparePredicatesNames(cb, from, deleted,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                includeHistory).toArray(new Predicate[0])));
        cq.select(cb.count(from));

        return em.createQuery(cq).getSingleResult();
    }

    /**
     * Find names.
     *
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @return list of names
     */
    public List<Name> readNames(Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who) {

        return readNames(deleted,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                Boolean.FALSE, null, null, null, null);
    }

    /**
     * Find names.
     *
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @param orderBy order by
     * @param isAsc is ascending
     * @param offset offset
     * @param limit limit
     * @return list of names
     */
    public List<Name> readNames(Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            FieldName orderBy, Boolean isAsc, Integer offset, Integer limit) {
        return readNames(deleted,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                Boolean.FALSE, orderBy, isAsc, offset, limit);
    }

    /**
     * Find names.
     *
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @param includeHistory include history
     * @param orderBy order by
     * @param isAsc is ascending
     * @param offset offset
     * @param limit limit
     * @return list of names
     */
    public List<Name> readNames(Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            Boolean includeHistory, FieldName orderBy, Boolean isAsc, Integer offset, Integer limit) {

        // note
        //     use of function for mnemonic path
        // where
        //     latest, deleted
        //     queryFields, queryValues
        // order
        //     orderBy, isAsc
        // paging
        //     offset, limit

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Name> cq = cb.createQuery(Name.class);
        Root<Name> from = cq.from(Name.class);

        cq.where(cb.and(preparePredicatesNames(cb, from, deleted,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                includeHistory).toArray(new Predicate[0])));
        cq.select(from);

        if (orderBy != null) {
            if (BooleanUtils.toBoolean(isAsc)) {
                if (FieldName.NAME.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Name.FIELD_CONVENTION_NAME)));
                } else if (FieldName.NAMEEQUIVALENCE.equals(orderBy)) {
                        cq.orderBy(cb.asc(from.get(Name.FIELD_CONVENTION_NAME_EQUIVALENCE)));
                } else if (FieldName.SYSTEMSTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.asc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_SYSTEM_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DEVICESTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.asc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_DEVICE_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.INDEX.equals(orderBy)) {
                    cq.orderBy(cb.asc(cb.function(NameStructure.FUNCTION_GET_INSTANCE_INDEX, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldName.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.asc(from.get(Name.FIELD_CONVENTION_NAME)));
                }
            } else {
                if (FieldName.NAME.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Name.FIELD_CONVENTION_NAME)));
                } else if (FieldName.NAMEEQUIVALENCE.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Name.FIELD_CONVENTION_NAME_EQUIVALENCE)));
                } else if (FieldName.SYSTEMSTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.desc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_SYSTEM_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DEVICESTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.desc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_DEVICE_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.INDEX.equals(orderBy)) {
                    cq.orderBy(cb.desc(cb.function(NameStructure.FUNCTION_GET_INSTANCE_INDEX, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldName.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.desc(from.get(Name.FIELD_CONVENTION_NAME)));
                }
            }
        }

        TypedQuery<Name> query = em.createQuery(cq);
        if (offset != null && limit != null) {
            query.setFirstResult(offset * limit);
            query.setMaxResults(limit);
        }

        return query.getResultList();
    }

    /**
     * Prepare predicates for names.
     *
     * @param cb criteria builder
     * @param from criteria query root
     * @param deleted deleted
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @param includeHistory include history
     * @return list of predicates
     */
    private List<Predicate> preparePredicatesNames(CriteriaBuilder cb, Root<Name> from, Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            Boolean includeHistory) {

        List<Predicate> predicates = new ArrayList<>();

        if (!Boolean.TRUE.equals(includeHistory)) {
            // purpose of Naming to show valid entries
            //     therefore
            //         exclude some values unless history requested
            //         make sure to not exclude present and future values
            //
            // exclude not latest
            //     select * from Name n where
            //         not(n.latest = false)

            Predicate predicateNotLatest = cb.equal(from.get(NameStructure.FIELD_LATEST), Boolean.FALSE);
            Predicate predicateExclude   = cb.not(cb.and(predicateNotLatest));
            predicates.add(predicateExclude);
        }

        if (deleted != null) {
            predicates.add(cb.equal(from.get(NameStructure.FIELD_DELETED), deleted));
        }

        // prepare pattern
        //     jpa query characters % and _
        //     remove excess % characters

        if (!StringUtils.isEmpty(uuid)) {
            predicates.add(cb.and(cb.equal(from.get(NameStructure.FIELD_UUID), RepositoryUtil.preparePattern(uuid))));
        }
        if (!StringUtils.isEmpty(name)) {
            predicates.add(cb.and(cb.like(from.get(Name.FIELD_CONVENTION_NAME), RepositoryUtil.preparePattern(name))));
        }
        if (!StringUtils.isEmpty(nameEquivalence)) {
            predicates.add(cb.and(cb.like(from.get(Name.FIELD_CONVENTION_NAME_EQUIVALENCE), RepositoryUtil.preparePattern(nameEquivalence))));
        }
        if (!StringUtils.isEmpty(systemStructure)) {
            predicates.add(cb.and(cb.like(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_SYSTEM_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME)), RepositoryUtil.preparePattern(systemStructure))));
        }
        if (!StringUtils.isEmpty(deviceStructure)) {
            predicates.add(cb.and(cb.like(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_DEVICE_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME)), RepositoryUtil.preparePattern(deviceStructure))));
        }
        if (!StringUtils.isEmpty(index)) {
            predicates.add(cb.and(cb.like(cb.function(NameStructure.FUNCTION_GET_INSTANCE_INDEX, String.class, from.get(Name.FIELD_CONVENTION_NAME)), RepositoryUtil.preparePattern(index))));
        }
        if (!StringUtils.isEmpty(description)) {
            predicates.add(cb.and(cb.like(from.get(NameStructure.FIELD_DESCRIPTION), RepositoryUtil.preparePattern(description))));
        }
        if (!StringUtils.isEmpty(who)) {
            predicates.add(cb.and(cb.like(from.get(NameStructure.FIELD_REQUESTED_BY), RepositoryUtil.preparePattern(who))));
        }

        return predicates;
    }

    /**
     * Count names history.
     *
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @return count of names
     */
    public Long countNamesHistory(
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who) {

        // note
        //     use of function for mnemonic path
        //     deleted - null
        //     includeHistory - true
        // where
        //     queryFields, queryValues

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Long> cq = cb.createQuery(Long.class);
        Root<Name> from = cq.from(Name.class);

        Subquery<String> sub = cq.subquery(String.class);
        Root<Name> fromSub = sub.from(Name.class);
        sub.where(cb.and(preparePredicatesNames(cb, fromSub, null,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                Boolean.TRUE).toArray(new Predicate[0])));
        sub.select(fromSub.get(NameStructure.FIELD_UUID));

        cq.where(cb.and(cb.in(from.get(NameStructure.FIELD_UUID)).value(sub)));
        cq.select(cb.count(from));

        return em.createQuery(cq).getSingleResult();
    }

    /**
     * Find names history.
     *
     * @param uuid uuid
     * @param name name
     * @param nameEquivalence name equivalence
     * @param systemStructure system structure mnemonic
     * @param deviceStructure device structure mnemonic
     * @param index index
     * @param description description
     * @param who who
     * @param orderBy order by
     * @param isAsc is ascending
     * @param offset offset
     * @param limit limit
     * @return list of names
     */
    public List<Name> readNamesHistory(
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            FieldName orderBy, Boolean isAsc, Integer offset, Integer limit) {

        // note
        //     use of function for mnemonic path
        //     deleted - null
        //     includeHistory - true
        // where
        //     queryFields, queryValues
        // order
        //     orderBy, isAsc
        // paging
        //     offset, limit

        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<Name> cq = cb.createQuery(Name.class);
        Root<Name> from = cq.from(Name.class);

        Subquery<String> sub = cq.subquery(String.class);
        Root<Name> fromSub = sub.from(Name.class);
        sub.where(cb.and(preparePredicatesNames(cb, fromSub, null,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                Boolean.TRUE).toArray(new Predicate[0])));
        sub.select(fromSub.get(NameStructure.FIELD_UUID));

        cq.where(cb.and(cb.in(from.get(NameStructure.FIELD_UUID)).value(sub)));
        cq.select(from);

        if (orderBy != null) {
            if (BooleanUtils.toBoolean(isAsc)) {
                if (FieldName.NAME.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Name.FIELD_CONVENTION_NAME)));
                } else if (FieldName.NAMEEQUIVALENCE.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(Name.FIELD_CONVENTION_NAME_EQUIVALENCE)));
                } else if (FieldName.SYSTEMSTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.asc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_SYSTEM_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DEVICESTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.asc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_DEVICE_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.INDEX.equals(orderBy)) {
                    cq.orderBy(cb.asc(cb.function(NameStructure.FUNCTION_GET_INSTANCE_INDEX, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldName.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.asc(from.get(NameStructure.FIELD_UUID)), cb.asc(from.get(Persistable.FIELD_ID)));
                }
            } else {
                if (FieldName.NAME.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Name.FIELD_CONVENTION_NAME)));
                } else if (FieldName.NAMEEQUIVALENCE.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(Name.FIELD_CONVENTION_NAME_EQUIVALENCE)));
                } else if (FieldName.SYSTEMSTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.desc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_SYSTEM_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DEVICESTRUCTURE.equals(orderBy)) {
                    cq.orderBy(cb.desc(cb.function(NameStructure.FUNCTION_GET_MNEMONIC_PATH_DEVICE_STRUCTURE, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.INDEX.equals(orderBy)) {
                    cq.orderBy(cb.desc(cb.function(NameStructure.FUNCTION_GET_INSTANCE_INDEX, String.class, from.get(Name.FIELD_CONVENTION_NAME))));
                } else if (FieldName.DESCRIPTION.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_DESCRIPTION)));
                } else if (FieldName.WHEN.equals(orderBy)) {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_REQUESTED)));
                } else {
                    cq.orderBy(cb.desc(from.get(NameStructure.FIELD_UUID)), cb.desc(from.get(Persistable.FIELD_ID)));
                }
            }
        }

        TypedQuery<Name> query = em.createQuery(cq);
        if (offset != null && limit != null) {
            query.setFirstResult(offset * limit);
            query.setMaxResults(limit);
        }

        return query.getResultList();
    }

    /**
     * Count legacy names
     *
     * @param name name
     * @return count of legacy names
     */
    public Long countNamesLegacy(String name) {
        // a name is considered legacy if it is active but refers to a parent that is deleted
        // query
        //     logical but non-trivial
        //     relies on lifecycle and parent-child attributes
        //     look for one or more deleted parents anywhere in system structure or device structure hierarchies
        StringBuilder sql = new StringBuilder();
        sql.append("select count(n) from Name n where n.latest = true and n.deleted = false ");
        sql.append("and (");
        sql.append("       (n.systemGroupUuid in (select sg.uuid from SystemGroup sg where sg.status = 'APPROVED' and sg.latest = true and sg.deleted = true))");
        sql.append("    or (n.systemUuid      in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true and sy.deleted = true))");
        sql.append("    or (n.systemUuid      in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true                         and sy.parentUuid in (select sg.uuid from SystemGroup sg where sg.status = 'APPROVED' and sg.latest = true and sg.deleted = true)))");
        sql.append("    or (n.subsystemUuid   in (select su.uuid from Subsystem   su where su.status = 'APPROVED' and su.latest = true and su.deleted = true))");
        sql.append("    or (n.subsystemUuid   in (select su.uuid from Subsystem   su where su.status = 'APPROVED' and su.latest = true                         and su.parentUuid in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true and sy.deleted = true)))");
        sql.append("    or (n.subsystemUuid   in (select su.uuid from Subsystem   su where su.status = 'APPROVED' and su.latest = true                         and su.parentUuid in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true                          and sy.parentUuid in (select sg.uuid from SystemGroup sg where sg.status = 'APPROVED' and sg.latest = true and sg.deleted = true))))");
        sql.append("    or (n.deviceTypeUuid  in (select dt.uuid from DeviceType  dt where dt.status = 'APPROVED' and dt.latest = true and dt.deleted = true))");
        sql.append("    or (n.deviceTypeUuid  in (select dt.uuid from DeviceType  dt where dt.status = 'APPROVED' and dt.latest = true                         and dt.parentUuid in (select dg.uuid from DeviceGroup dg where dg.status = 'APPROVED' and dg.latest = true and dg.deleted = true)))");
        sql.append("    or (n.deviceTypeUuid  in (select dt.uuid from DeviceType  dt where dt.status = 'APPROVED' and dt.latest = true                         and dt.parentUuid in (select dg.uuid from DeviceGroup dg where dg.status = 'APPROVED' and dg.latest = true                          and dg.parentUuid in (select di.uuid from Discipline  di where di.status = 'APPROVED' and di.latest = true and di.deleted = true))))");
        sql.append(")");

        boolean hasWhere = StringUtils.isNotEmpty(name);
        Query query = null;
        if (hasWhere) {
            sql.append(" and n.conventionName like :pName");
            query = em.createQuery(sql.toString(), Long.class).setParameter("pName", name);
        } else {
            query = em.createQuery(sql.toString(), Long.class);
        }

        return (Long) query.getSingleResult();
    }

    /**
     * Find legacy names.
     *
     * @param name name
     * @param orderBy order by
     * @param isAsc is ascending
     * @param offset offset
     * @param limit limit
     * @return list of names
     * @return
     */
    @SuppressWarnings("unchecked")
    public List<Name> readNamesLegacy(String name,
            FieldName orderBy, Boolean isAsc, Integer offset, Integer limit) {

        // a name is considered legacy if it is active but refers to a parent that is deleted
        // query
        //     logical but non-trivial
        //     relies on lifecycle and parent-child attributes
        //     look for one or more deleted parents anywhere in system structure or device structure hierarchies
        StringBuilder sql = new StringBuilder();
        sql.append("select n from Name n where n.latest = true and n.deleted = false ");
        sql.append("and (");
        sql.append("       (n.systemGroupUuid in (select sg.uuid from SystemGroup sg where sg.status = 'APPROVED' and sg.latest = true and sg.deleted = true))");
        sql.append("    or (n.systemUuid      in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true and sy.deleted = true))");
        sql.append("    or (n.systemUuid      in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true                         and sy.parentUuid in (select sg.uuid from SystemGroup sg where sg.status = 'APPROVED' and sg.latest = true and sg.deleted = true)))");
        sql.append("    or (n.subsystemUuid   in (select su.uuid from Subsystem   su where su.status = 'APPROVED' and su.latest = true and su.deleted = true))");
        sql.append("    or (n.subsystemUuid   in (select su.uuid from Subsystem   su where su.status = 'APPROVED' and su.latest = true                         and su.parentUuid in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true and sy.deleted = true)))");
        sql.append("    or (n.subsystemUuid   in (select su.uuid from Subsystem   su where su.status = 'APPROVED' and su.latest = true                         and su.parentUuid in (select sy.uuid from System      sy where sy.status = 'APPROVED' and sy.latest = true                          and sy.parentUuid in (select sg.uuid from SystemGroup sg where sg.status = 'APPROVED' and sg.latest = true and sg.deleted = true))))");
        sql.append("    or (n.deviceTypeUuid  in (select dt.uuid from DeviceType  dt where dt.status = 'APPROVED' and dt.latest = true and dt.deleted = true))");
        sql.append("    or (n.deviceTypeUuid  in (select dt.uuid from DeviceType  dt where dt.status = 'APPROVED' and dt.latest = true                         and dt.parentUuid in (select dg.uuid from DeviceGroup dg where dg.status = 'APPROVED' and dg.latest = true and dg.deleted = true)))");
        sql.append("    or (n.deviceTypeUuid  in (select dt.uuid from DeviceType  dt where dt.status = 'APPROVED' and dt.latest = true                         and dt.parentUuid in (select dg.uuid from DeviceGroup dg where dg.status = 'APPROVED' and dg.latest = true                          and dg.parentUuid in (select di.uuid from Discipline  di where di.status = 'APPROVED' and di.latest = true and di.deleted = true))))");
        sql.append(")");

        StringBuilder sqlOrderBy = new StringBuilder();
        if (orderBy != null) {
            sqlOrderBy.append(" order by ");

            if (FieldName.NAMEEQUIVALENCE.equals(orderBy)) {
                sqlOrderBy.append("n.");
                sqlOrderBy.append(Name.FIELD_CONVENTION_NAME_EQUIVALENCE);
            } else if (FieldName.SYSTEMSTRUCTURE.equals(orderBy)) {
                sqlOrderBy.append(NameStructure.FUNCTION_GET_MNEMONIC_PATH_SYSTEM_STRUCTURE);
                sqlOrderBy.append("(n.conventionName)");
            } else if (FieldName.DEVICESTRUCTURE.equals(orderBy)) {
                sqlOrderBy.append(NameStructure.FUNCTION_GET_MNEMONIC_PATH_DEVICE_STRUCTURE);
                sqlOrderBy.append("(n.conventionName)");
            } else if (FieldName.INDEX.equals(orderBy)) {
                sqlOrderBy.append(NameStructure.FUNCTION_GET_INSTANCE_INDEX);
                sqlOrderBy.append("(n.conventionName)");
            } else if (FieldName.DESCRIPTION.equals(orderBy)) {
                sqlOrderBy.append("n.");
                sqlOrderBy.append(Name.FIELD_DESCRIPTION);
            } else if (FieldName.WHEN.equals(orderBy)) {
                sqlOrderBy.append("n.");
                sqlOrderBy.append(Name.FIELD_REQUESTED);
            } else {
                sqlOrderBy.append("n.");
                sqlOrderBy.append(Name.FIELD_CONVENTION_NAME);
            }

            if (BooleanUtils.toBoolean(isAsc)) {
                sqlOrderBy.append(" asc");
            } else {
                sqlOrderBy.append(" desc");
            }
        }

        boolean hasWhere = StringUtils.isNotEmpty(name);
        boolean hasOrderBy = orderBy != null;
        Query query = null;
        if (hasWhere) {
            sql.append(" and n.conventionName like :pName");
            if (hasOrderBy) {
                sql.append(sqlOrderBy.toString());
            }
            query = em.createQuery(sql.toString(), Name.class).setParameter("pName", name);
        } else {
            if (hasOrderBy) {
                sql.append(sqlOrderBy.toString());
            }
            query = em.createQuery(sql.toString(), Name.class);
        }

        if (offset != null && limit != null) {
            query.setFirstResult(offset * limit);
            query.setMaxResults(limit);
        }
        return query.getResultList();
    }

    /**
     * Persist name into persistence context.
     *
     * @param name name
     */
    public void createName(Name name) {
        em.persist(name);
    }

    /**
     * Merge name into persistence context.
     *
     * @param name name
     */
    public void updateName(Name name) {
        em.merge(name);
    }

}
