/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.repository.model;

import java.io.Serializable;
import java.util.Date;
import java.util.Objects;
import java.util.UUID;

import javax.persistence.Column;
import javax.persistence.MappedSuperclass;

import org.openepics.names.rest.beans.Status;

/**
 * A superclass implementing properties required by JPA. It should be extended by
 * System structure and Device structure classes that need to be persisted to the database.
 *
 * @author Lars Johansson
 */
@MappedSuperclass
public class Structure extends NameStructure implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = -5980090194197159918L;

    public static final String FIELD_NAME                 = "name";
    public static final String FIELD_MNEMONIC             = "mnemonic";
    public static final String FIELD_MNEMONIC_EQUIVALENCE = "mnemonicEquivalence";

    private String name;
    private String mnemonic;
    @Column(name = "mnemonic_equivalence")
    private String mnemonicEquivalence;

    /**
     * Constructor for Structure.
     */
    public Structure() {
    }

    /**
     * Constructor for Structure.
     *
     * @param uuid uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param description description
     * @param status status
     * @param latest latest
     * @param deleted deleted
     * @param requested requested
     * @param requestedBy requested by
     * @param requestedComment requested comment
     */
    public Structure(UUID uuid, String name, String mnemonic, String mnemonicEquivalence,
            String description, Status status, Boolean latest, Boolean deleted,
            Date requested, String requestedBy, String requestedComment) {
        setUuid(uuid);
        setName(name);
        setMnemonic(mnemonic);
        setMnemonicEquivalence(mnemonicEquivalence);
        setDescription(description);
        setStatus(status);
        setLatest(latest);
        setDeleted(deleted);
        setRequested(requested);
        setRequestedBy(requestedBy);
        setRequestedComment(requestedComment);
    }

    public String getName() {
        return name;
    }
    public void setName(String name) {
        this.name = name;
    }
    public String getMnemonic() {
        return mnemonic;
    }
    public void setMnemonic(String mnemonic) {
        this.mnemonic = mnemonic;
    }
    public String getMnemonicEquivalence() {
        return mnemonicEquivalence;
    }
    public void setMnemonicEquivalence(String mnemonicEquivalence) {
        this.mnemonicEquivalence = mnemonicEquivalence;
    }

    /**
     * Utility method to help set attributes for Structure class, which in practice is either of its sub classes.
     *
     * @param status status
     * @param processed processed
     * @param processedBy processed by
     * @param processedComment processed comment
     */
    public void setAttributesStatusProcessed(Status status, Date processed, String processedBy, String processedComment) {
        setStatus(status);
        setProcessed(processed);
        setProcessedBy(processedBy);
        setProcessedComment(processedComment);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;

        return equals((Structure) obj);
    }

    public boolean equals(Structure other) {
        if (other == null)
            return false;

        if (!super.equals(other)) {
            return false;
        }

        if (getName() == null) {
            if (other.getName() != null)
                return false;
        } else if (!getName().equals(other.getName()))
            return false;
        if (getMnemonic() == null) {
            if (other.getMnemonic() != null)
                return false;
        } else if (!getMnemonic().equals(other.getMnemonic()))
            return false;
        if (getMnemonicEquivalence() == null) {
            if (other.getMnemonicEquivalence() != null)
                return false;
        } else if (!getMnemonicEquivalence().equals(other.getMnemonicEquivalence()))
            return false;

        return true;
    }

    @Override
    public boolean equalsId(Object other) {
        if (other instanceof Structure object) {
            return object.getId().equals(getId());
        }
        return false;
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId(), getUuid());
    }

}
