/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.rest.api.v1;

import java.util.List;

import org.openepics.names.rest.beans.FieldStructure;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommandConfirm;
import org.openepics.names.rest.beans.element.StructureElementCommandCreate;
import org.openepics.names.rest.beans.element.StructureElementCommandUpdate;
import org.openepics.names.rest.beans.response.Response;
import org.openepics.names.rest.beans.response.ResponseBoolean;
import org.openepics.names.rest.beans.response.ResponseBooleanList;
import org.openepics.names.rest.beans.response.ResponsePageStructureElements;
import org.openepics.names.util.ExcelUtil;
import org.springframework.core.io.Resource;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;

/**
 * This part of REST API provides structures data for Naming application.
 *
 * @author Lars Johansson
 */
@Tag(name        = "Structures",
     description = "handle structures data for Naming application")
@RequestMapping("/api/v1/structures")
public interface IStructures {

    /*
       StructureElementCommandCreate  -       type, parent, name, mnemonic, description, comment
       StructureElementCommandUpdate  - uuid, type, parent, name, mnemonic, description, comment
       StructureElementCommandConfirm - uuid, type,                                      comment
       ----------------------------------------------------------------------------------------------------
       converted into StructureElementCommand
       ----------------------------------------------------------------------------------------------------
       StructureElementCommand - subset of StructureElement attributes
       StructureElement
                          uuid            (UUID)
                          type            (Type)
                          parent          (UUID)
                          name            (String)
                          mnemonic        (String)
                          description     (String)
                          comment         (String)
                          -------------------------
                          mnemonicPath    (String)
                          level           (Integer)
                          status          (Status)
                          latest          (Boolean)
                          deleted         (Boolean)
                          when            (Date)
                          who             (String)

       Methods
           create    POST   /structures                                       - createStructures
           create    POST   /structures/upload                                - createStructures               upload Excel file
           ----------------------------------------------------------------------------------------------------
           read      GET    /structures/{type}                                - readStructures
           read      GET    /structures/{type}/download                       - readStructuresDownload         download Excel file
           read      GET    /structures/children/{uuid}                       - readStructuresChildren
           read      GET    /structures/mnemonic/{mnemonic}                   - readStructuresMnemonic
           read      GET    /structures/mnemonicPath/{mnemonicPath}           - readStructuresMnemonicPath
           read      GET    /structures/history                               - readStructuresHistory
           read      GET    /structures/history/{uuid}                        - readStructuresHistory
           ----------------------------------------------------------------------------------------------------
           read      GET    /structures/equivalence/{mnemonic}                - equivalenceMnemonic
           read      GET    /structures/exists/{type}/{mnemonicPath}          - existsStructure
           read      GET    /structures/isValidToCreate/{type}/{mnemonicPath} - isValidToCreateStructure
           ----------------------------------------------------------------------------------------------------
           read      GET    /structures/validateCreate                        - validateStructuresCreate       hidden from Swagger UI
           read      GET    /structures/validateUpdate                        - validateStructuresUpdate       hidden from Swagger UI
           read      GET    /structures/validateDelete                        - validateStructuresDelete       hidden from Swagger UI
           read      GET    /structures/validateApprove                       - validateStructuresApprove      hidden from Swagger UI
           read      GET    /structures/validateCancel                        - validateStructuresCancel       hidden from Swagger UI
           read      GET    /structures/validateReject                        - validateStructuresReject       hidden from Swagger UI
           ----------------------------------------------------------------------------------------------------
           update    PUT    /structures                                       - updateStructures
           update    PUT    /structures/upload                                - updateStructures               upload Excel file
           ----------------------------------------------------------------------------------------------------
           delete    DELETE /structures                                       - deleteStructures
           delete    DELETE /structures/upload                                - deleteStructures               upload Excel file
           ----------------------------------------------------------------------------------------------------
           update    PATCH  /structures/approve                               - approveStructures
           update    PATCH  /structures/approve/upload                        - approveStructures              upload Excel file
           update    PATCH  /structures/cancel                                - cancelStructures
           update    PATCH  /structures/cancel/upload                         - cancelStructures               upload Excel file
           update    PATCH  /structures/reject                                - rejectStructures
           update    PATCH  /structures/reject/upload                         - rejectStructures               upload Excel file

       Note
           /upload and /download endpoints handle Excel files
           authentication/authorization
               3 levels - no, user, administrator
                   no            - read
                   user          - create, update, delete
                   administrator - approve, cancel, reject
           mnemonic path for structure includes all levels of mnemonics
     */

    public static final String DEFAULT_PAGE            = "0";
    public static final String DEFAULT_PAGE_SIZE       = "100";
    public static final String DEFAULT_SORT_FIELD_WHEN = "WHEN";
    public static final String DEFAULT_SORT_ORDER_ASC  = "true";

    /**
     * Create (propose) structures by list of structure element commands for create.
     * Return list of structure elements for created structures (proposals).
     *
     * @param structureElementCommands list of structure element commands for create
     * @return list of structure elements for created structures (proposals)
     */
    @Operation(
            summary     = "Create (propose) structures by array of structure element commands for create",
            description = """
                          Create (propose) structures by array of structure element commands for create.
                          Return array of structure elements for created structures (proposals).

                          Required attributes:
                          - type
                          - parent (System, Subsystem, DeviceGroup, DeviceType)(uuid)
                          - name
                          - mnemonic (System, Subsystem, Discipline, DeviceType, may be set for SystemGroup, not allowed for DeviceGroup)
                          - description
                          - comment

                          Uuid is created by Naming.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "201",
                    description  = "OK. Return array of structure elements for created structures (proposals).",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = StructureElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PostMapping(
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<List<StructureElement>> createStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandCreate.class,
                                            requiredProperties = {"type","name","description","comment"}))))
            @RequestBody List<StructureElementCommandCreate> structureElementCommands);

    /**
     * Create (propose) structures by upload Excel file.
     * Return Excel file with structure elements for created structures (proposals).
     *
     * @param file file
     * @return Excel file
     */
    @Operation(
            summary     = "Create (propose) structures by upload Excel file",
            description = """
                          Create (propose) structures by upload Excel file.
                          Return Excel file with structure elements for created structures (proposals).

                          Expected columns as in Excel template for structures.

                          Required attributes:
                          - type
                          - parent (System, Subsystem, DeviceGroup, DeviceType)(uuid)
                          - name
                          - mnemonic (System, Subsystem, Discipline, DeviceType, may be set for SystemGroup, not allowed for DeviceGroup)
                          - description
                          - comment

                          Uuid is created by Naming.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "201",
                    description  = "OK. Return Excel file with structure elements for created structures (proposals).",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PostMapping(
            value = "/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> createStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with structure element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Find valid structures (search).
     * Return paged list of structure elements.
     *
     * @param type type of structure
     * @param statuses statuses for structures
     * @param deleted if deleted-only structures are to be included
     * (false for non-deleted-only structures, true for deleted-only structures, not used for both cases)
     * @param uuid uuid
     * @param parent parent uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of structure elements
     */
    @Operation(
            summary     = "Find valid structures (search)",
            description = """
                          Find valid structures (search).
                          Return paged array of structure elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of structure elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageStructureElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/{type}",
            produces = {"application/json"})
    public ResponsePageStructureElements readStructures(
            @Parameter(in = ParameterIn.PATH,  description = "search by type of structure") @PathVariable("type") Type type,
            @Parameter(in = ParameterIn.QUERY, description = "search by statuses") @RequestParam(required = false) Status[] statuses,
            @Parameter(in = ParameterIn.QUERY, description = "if deleted structures are to be included or not, omitted for both deleted and not deleted structures") @RequestParam(required = false) Boolean deleted,
            @Parameter(in = ParameterIn.QUERY, description = "find by uuid") @RequestParam(required = false) String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "search by parent uuid") @RequestParam(required = false) String parent,
            @Parameter(in = ParameterIn.QUERY, description = "search by name") @RequestParam(required = false) String name,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic") @RequestParam(required = false) String mnemonic,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic equivalence") @RequestParam(required = false) String mnemonicEquivalence,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic path") @RequestParam(required = false) String mnemonicPath,
            @Parameter(in = ParameterIn.QUERY, description = "search by description") @RequestParam(required = false) String description,
            @Parameter(in = ParameterIn.QUERY, description = "search by who") @RequestParam(required = false) String who,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldStructure orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid structures (search).
     * Return Excel file with paged list of structure elements.
     *
     * @param type type of structure
     * @param statuses statuses for structures
     * @param deleted if deleted-only structures are to be included
     * (false for non-deleted-only structures, true for deleted-only structures, not used for both cases)
     * @param uuid uuid
     * @param parent parent uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of structure elements
     */
    @Operation(
            summary     = "Find valid structures (search) and download Excel file",
            description = """
                          Find valid structures (search).
                          Return Excel file with paged list of structure elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with paged list of structure elements.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/{type}/download",
            produces = {"application/vnd.ms-excel"})
    public ResponseEntity<Resource> readStructuresDownload(
            @Parameter(in = ParameterIn.PATH,  description = "search by type of structure") @PathVariable("type") Type type,
            @Parameter(in = ParameterIn.QUERY, description = "search by statuses") @RequestParam(required = false) Status[] statuses,
            @Parameter(in = ParameterIn.QUERY, description = "if deleted structures are to be included or not, omitted for both deleted and not deleted structures") @RequestParam(required = false) Boolean deleted,
            @Parameter(in = ParameterIn.QUERY, description = "find by uuid") @RequestParam(required = false) String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "search by parent uuid") @RequestParam(required = false) String parent,
            @Parameter(in = ParameterIn.QUERY, description = "search by name") @RequestParam(required = false) String name,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic") @RequestParam(required = false) String mnemonic,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic equivalence") @RequestParam(required = false) String mnemonicEquivalence,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic path") @RequestParam(required = false) String mnemonicPath,
            @Parameter(in = ParameterIn.QUERY, description = "search by description") @RequestParam(required = false) String description,
            @Parameter(in = ParameterIn.QUERY, description = "search by who") @RequestParam(required = false) String who,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldStructure orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid children structures by parent uuid (exact match).
     * Return paged list of structure elements.
     *
     * @param uuid uuid
     * @param type type of structure
     * @param statuses statuses for structures
     * @param deleted if deleted-only structures are to be included
     * (false for non-deleted-only structures, true for deleted-only structures, not used for both cases)
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param mnemonicPath mnemonic path
     * @param description description
     * @param who who
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of structure elements
     */
    @Operation(
            summary     = "Find valid children structures by type and parent uuid (exact match)",
            description = """
                          Find valid children structures by type and parent uuid (exact match).
                          Return paged array of structure elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of structure elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageStructureElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/children/{uuid}",
            produces = {"application/json"})
    public ResponsePageStructureElements readStructuresChildren(
            @Parameter(in = ParameterIn.PATH,  description = "find by uuid") @PathVariable("uuid") String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "search by type of structure") @RequestParam(required = false) Type type,
            @Parameter(in = ParameterIn.QUERY, description = "search by statuses") @RequestParam(required = false) Status[] statuses,
            @Parameter(in = ParameterIn.QUERY, description = "if deleted structures are to be included or not, omitted for both deleted and not deleted structures") @RequestParam(required = false) Boolean deleted,
            @Parameter(in = ParameterIn.QUERY, description = "search by name") @RequestParam(required = false) String name,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic") @RequestParam(required = false) String mnemonic,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic equivalence") @RequestParam(required = false) String mnemonicEquivalence,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic path") @RequestParam(required = false) String mnemonicPath,
            @Parameter(in = ParameterIn.QUERY, description = "search by description") @RequestParam(required = false) String description,
            @Parameter(in = ParameterIn.QUERY, description = "search by who") @RequestParam(required = false) String who,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldStructure orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid structures by mnemonic (search).
     * Return paged list of structure elements.
     *
     * @param mnemonic mnemonic
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of structure elements
     */
    @Operation(
            summary     = "Find valid structures by mnemonic (search)",
            description = """
                          Find valid structures by mnemonic (search).
                          Return paged array of structure elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of structure elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageStructureElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/mnemonic/{mnemonic}",
            produces = {"application/json"})
    public ResponsePageStructureElements readStructuresMnemonic(
            @Parameter(in = ParameterIn.PATH,  description = "search by mnemonic") @PathVariable("mnemonic") String mnemonic,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldStructure orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find valid structures by mnemonic path (search).
     * Return paged list of structure elements.
     *
     * @param mnemonicPath mnemonic path
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of structure elements
     */
    @Operation(
            summary     = "Find valid structures by mnemonic path (search)",
            description = """
                          Find valid structures by mnemonic path (search).
                          Return paged array of structure elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of structure elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageStructureElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/mnemonicPath/{mnemonicPath}",
            produces = {"application/json"})
    public ResponsePageStructureElements readStructuresMnemonicpath(
            @Parameter(in = ParameterIn.PATH,  description = "search by mnemonic path") @PathVariable("mnemonicPath") String mnemonicPath,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldStructure orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find history for structure (search).
     * History consists of lines of uuid.
     * The line of uuid is not broken in retrieving history. If combination of parameters is found in structure entries, the entire lines of uuid are returned.
     * Return paged list of structure elements.
     *
     * @param type type of structure
     * @param uuid uuid
     * @param parent parent uuid
     * @param name name
     * @param mnemonic mnemonic
     * @param mnemonicEquivalence mnemonic equivalence
     * @param description description
     * @param who who
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of structure elements
     */
    @Operation(
            summary     = "Find history for structure (search)",
            description = """
                          Find history for structure (search).
                          History consists of lines of uuid. The line of uuid is not broken in retrieving history. If combination of parameters is found in structure entries, the entire lines of uuid are returned.
                          Return paged array of structure elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of structure elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageStructureElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/history",
            produces = {"application/json"})
    public ResponsePageStructureElements readStructuresHistory(
            @Parameter(in = ParameterIn.QUERY, description = "search by type of structure") @RequestParam(required = false) Type type,
            @Parameter(in = ParameterIn.QUERY, description = "find by uuid") @RequestParam(required = false) String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "search by parent uuid") @RequestParam(required = false) String parent,
            @Parameter(in = ParameterIn.QUERY, description = "search by name") @RequestParam(required = false) String name,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic") @RequestParam(required = false) String mnemonic,
            @Parameter(in = ParameterIn.QUERY, description = "search by mnemonic equivalence") @RequestParam(required = false) String mnemonicEquivalence,
            @Parameter(in = ParameterIn.QUERY, description = "search by description") @RequestParam(required = false) String description,
            @Parameter(in = ParameterIn.QUERY, description = "search by who") @RequestParam(required = false) String who,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false) FieldStructure orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    /**
     * Find history for structure by uuid (exact match).
     * History consists of line of uuid.
     * The line of uuid is not broken in retrieving history. If the uuid is found in a structure entry, the entire line of uuid is returned.
     * Return paged list of structure elements.
     *
     * @param uuid uuid
     * @param type type of structure to search in
     * @param orderBy order by field
     * @param isAsc sort order, ascending or descending
     * @param page page starting from 0, offset
     * @param pageSize page size, limit
     * @return paged list of structure elements
     */
    @Operation(
            summary     = "Find history for structure by uuid (exact match)",
            description = """
                          Find history for structure by uuid (exact match).
                          History consists of line of uuid. The line of uuid is not broken in retrieving history. If the uuid is found in a structure entry, the entire line of uuid is returned.
                          Return paged array of structure elements.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return paged array of structure elements.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponsePageStructureElements.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/history/{uuid}",
            produces = {"application/json"})
    public ResponsePageStructureElements readStructuresHistory(
            @Parameter(in = ParameterIn.PATH,  description = "find by uuid") @PathVariable("uuid") String uuid,
            @Parameter(in = ParameterIn.QUERY, description = "search by type of structure") @RequestParam(required = false) Type type,
            @Parameter(in = ParameterIn.QUERY, description = "order by field") @RequestParam(required = false, defaultValue = DEFAULT_SORT_FIELD_WHEN) FieldStructure orderBy,
            @Parameter(in = ParameterIn.QUERY, description = "sort order, ascending or descending") @RequestParam(required = false, defaultValue = DEFAULT_SORT_ORDER_ASC) Boolean isAsc,
            @Parameter(in = ParameterIn.QUERY, description = "page starting from 0, offset") @RequestParam(required = false, defaultValue = DEFAULT_PAGE) Integer page,
            @Parameter(in = ParameterIn.QUERY, description = "page size, limit") @RequestParam(required = false, defaultValue = DEFAULT_PAGE_SIZE) Integer pageSize);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Return mnemonic equivalence for mnemonic.
     *
     * @param mnemonic mnemonic to get mnemonic equivalence for
     * @return mnemonic equivalence
     */
    @Operation(
            summary     = "Return mnemonic equivalence for mnemonic",
            description = "Return mnemonic equivalence for mnemonic."
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return mnemonic equivalence.",
                    content = @Content(
                            mediaType = "text/plain",
                            schema = @Schema(implementation = String.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))

    })
    @GetMapping(
            value = "/equivalence/{mnemonic}")
    public String equivalenceMnemonic(
            @Parameter(in = ParameterIn.PATH, description = "mnemonic to get mnemonic equivalence for") @PathVariable("mnemonic") String mnemonic);

    /**
     * Return if mnemonic path exists in structure (exact match).
     *
     * <p>
     * Returned object has three fields (response, message, details).
     * <ul>
     * <li>response: boolean (true/false)</li>
     * <li>message: reason, if method fails</li>
     * <li>details: details, if method fails</li>
     * </ul>
     * </p>
     *
     * <p>
     * Note that method will always return false for DeviceGroup. Because DeviceGroup does not have mnemonic,
     * its mnemonic path is the mnemonic of its parent (Discipline).  However a query for mnemonic path
     * for DeviceGroup is ambiguous as it lacks mnemonic. Purpose of method is to check is a particular entry
     * exists in hierarchy of structures. Purpose of method is not to check if multiple entries match mnemonic path.
     * Therefor method returns false for DeviceGroup.
     * </p>
     *
     * @param type type of structure to search in
     * @param mnemonicPath mnemonic path to find structure for
     * @return if mnemonic path exists in structure
     */
    @Operation(
            summary     = "Return if mnemonic path exists in structure (exact match)",
            description = """
                          Return if mnemonic path exists in structure (exact match).
                          Response is true if mnemonic path exists, false otherwise.

                          Note that method will always return false for DeviceGroup. Because DeviceGroup does not have mnemonic,
                          its mnemonic path is the mnemonic of its parent (Discipline).  However a query for mnemonic path
                          for DeviceGroup is ambiguous as it lacks mnemonic. Purpose of method is to check is a particular entry
                          exists in hierarchy of structures. Purpose of method is not to check if multiple entries match mnemonic path.
                          Therefor method returns false for DeviceGroup.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if mnemonic path exists, false otherwise.
                                   Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBoolean.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/exists/{type}/{mnemonicPath}")
    public ResponseEntity<ResponseBoolean> existsStructure(
            @Parameter(in = ParameterIn.PATH,  description = "type of structure to search in") @PathVariable("type") Type type,
            @Parameter(in = ParameterIn.PATH,  description = "mnemonic path to find structure for") @PathVariable("mnemonicPath") String mnemonicPath);

    /**
     * Return if mnemonic path is valid to create in structure (exact match).
     * Method answers question 'would it be ok to create given mnemonic path in structure?'.
     *
     * <p>
     * Note that method can not fully answer question posed above. One reason is that system group
     * may have empty mnemonic. Another reason is that device groups has no mnemonic and is between
     * discipline and device type. Thus it can not be found which discipline (uuid-wise) that
     * a device type belongs.
     * </p>
     *
     * @param type type of structure to search in
     * @param mnemonicPath mnemonic path to find structure for
     * @return if mnemonic path is valid to create in structure
     */
    @Operation(
            summary     = "Return if mnemonic path is valid to create in structure",
            description = """
                          Return if mnemonic path is valid to create in structure.
                          Method answers question 'would it be ok to create given mnemonic path in structure?'.

                          Note that method can not fully answer question posed above. One reason is that system group
                          may have empty mnemonic. Another reason is that device groups has no mnemonic and is
                          between discipline and device type. Thus it can not be found which discipline (uuid-wise)
                          that a device type belongs.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if structure is valid to create, false otherwise.
                                   Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBoolean.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/isValidToCreate/{type}/{mnemonicPath}",
            produces = {"application/json"})
    public ResponseEntity<ResponseBoolean> isValidToCreateStructure(
            @Parameter(in = ParameterIn.PATH,  description = "type of structure to search in") @PathVariable("type") Type type,
            @Parameter(in = ParameterIn.PATH,  description = "mnemonic path to find structure for") @PathVariable("mnemonicPath") String mnemonicPath);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Return if structures are valid to create (propose) by list of structure element commands for create.
     * If structures are valid to create, successful create of structures can be expected.
     *
     * <p>
     * Returned object has five fields (message, details, field, value, list).
     * <ul>
     * <li>message:   reason, if method fails</li>
     * <li>details:   details, if method fails</li>
     * <li>field:     field, if method fails</li>
     * <li>value:     boolean (true/false), overall result of method</li>
     * <li>list:      list of response objects (with fields reason, details, field, value), one for each input element</li>
     * </ul>
     * </p>
     *
     * @param structureElementCommands list of structure element commands for create
     * @return if list of structures is valid to create (propose)
     */
    @Hidden
    @Operation(
            summary     = "Return if structures are valid to create (propose) by list of structure element commands for create",
            description = """
                          Return if structures are valid to create (propose) by list of structure element commands for create.
                          If structures are valid to create, successful create of structures can be expected.

                          Required attributes:
                          - type
                          - parent (System, Subsystem, DeviceGroup, DeviceType)(uuid)
                          - name
                          - mnemonic (System, Subsystem, Discipline, DeviceType, may be set for SystemGroup, not allowed for DeviceGroup)
                          - description
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all structure element commands validated ok, false otherwise.
                                   Responses contain array with result for each structure element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateCreate",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateStructuresCreate(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandCreate.class,
                                            requiredProperties = {"type","name","description","comment"}))))
            @RequestBody List<StructureElementCommandCreate> structureElementCommands);

    /**
     * Return if structures are valid to update (propose) by list of structure element commands for update.
     * If structures are valid to update, successful update of structures can be expected.
     *
     * <p>
     * Returned object has five fields (message, details, field, value, list).
     * <ul>
     * <li>message:   reason, if method fails</li>
     * <li>details:   details, if method fails</li>
     * <li>field:     field, if method fails</li>
     * <li>value:     boolean (true/false), overall result of method</li>
     * <li>list:      list of response objects (with fields reason, details, field, value), one for each input element</li>
     * </ul>
     * </p>
     *
     * @param structureElementCommands list of structure element commands for update
     * @return if list of structures is valid to update (propose)
     */
    @Hidden
    @Operation(
            summary     = "Return if structures are valid to update (propose) by list of structure element commands for update",
            description = """
                          Return if structures are valid to update (propose) by list of structure element commands for update.
                          If structures are valid to update, successful update of structures can be expected.

                          Required attributes:
                          - uuid
                          - type
                          - parent (System, Subsystem, DeviceGroup, DeviceType)(uuid)
                          - name
                          - mnemonic (System, Subsystem, Discipline, DeviceType, may be set for SystemGroup, not allowed for DeviceGroup)
                          - description
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all structure element commands validated ok, false otherwise.
                                   Responses contain array with result for each structure element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateUpdate",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateStructuresUpdate(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandUpdate.class,
                                            requiredProperties = {"uuid","type","name","description","comment"}))))
            @RequestBody List<StructureElementCommandUpdate> structureElementCommands);

    /**
     * Return if structures are valid to delete (propose) by list of structure element commands for confirm.
     * If structures are valid to delete, successful delete of structures can be expected.
     *
     * <p>
     * Returned object has five fields (message, details, field, value, list).
     * <ul>
     * <li>message:   reason, if method fails</li>
     * <li>details:   details, if method fails</li>
     * <li>field:     field, if method fails</li>
     * <li>value:     boolean (true/false), overall result of method</li>
     * <li>list:      list of response objects (with fields reason, details, field, value), one for each input element</li>
     * </ul>
     * </p>
     *
     * @param structureElementCommands list of structure element commands for confirm
     * @return if list of structures is valid to delete (propose)
     */
    @Hidden
    @Operation(
            summary     = "Return if structures are valid to delete (propose) by list of structure element commands for confirm",
            description = """
                          Return if structures are valid to delete (propose) by list of structure element commands for confirm.
                          If structures are valid to delete, successful delete of structures can be expected.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all structure element commands validated ok, false otherwise.
                                   Responses contain array with result for each structure element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateDelete",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateStructuresDelete(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    /**
     * Return if structures are valid to approve by list of structure element commands for confirm.
     * If structures are valid to approve, successful approve of structures can be expected.
     *
     * <p>
     * Returned object has five fields (message, details, field, value, list).
     * <ul>
     * <li>message:   reason, if method fails</li>
     * <li>details:   details, if method fails</li>
     * <li>field:     field, if method fails</li>
     * <li>value:     boolean (true/false), overall result of method</li>
     * <li>list:      list of response objects (with fields reason, details, field, value), one for each input element</li>
     * </ul>
     * </p>
     *
     * @param structureElementCommands list of structure element commands for confirm
     * @return if list of structures is valid to approve
     */
    @Hidden
    @Operation(
            summary     = "Return if structures are valid to approve by list of structure element commands for confirm",
            description = """
                          Return if structures are valid to approve by list of structure element commands for confirm.
                          If structures are valid to approve, successful approve of structures can be expected.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all structure element commands validated ok, false otherwise.
                                   Responses contain array with result for each structure element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateApprove",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateStructuresApprove(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    /**
     * Return if structures are valid to cancel by list of structure element commands for confirm.
     * If structures are valid to cancel, successful cancel of structures can be expected.
     *
     * <p>
     * Returned object has five fields (message, details, field, value, list).
     * <ul>
     * <li>message:   reason, if method fails</li>
     * <li>details:   details, if method fails</li>
     * <li>field:     field, if method fails</li>
     * <li>value:     boolean (true/false), overall result of method</li>
     * <li>list:      list of response objects (with fields reason, details, field, value), one for each input element</li>
     * </ul>
     * </p>
     *
     * @param structureElementCommands list of structure element commands for confirm
     * @return if list of structures is valid to cancel
     */
    @Hidden
    @Operation(
            summary     = "Return if structures are valid to cancel by list of structure element commands for confirm",
            description = """
                          Return if structures are valid to cancel by list of structure element commands for confirm.
                          If structures are valid to cancel, successful cancel of structures can be expected.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all structure element commands validated ok, false otherwise.
                                   Responses contain array with result for each structure element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateCancel",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateStructuresCancel(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    /**
     * Return if structures are valid to reject by list of structure element commands for confirm.
     * If structures are valid to reject, successful reject of structures can be expected.
     *
     * <p>
     * Returned object has five fields (message, details, field, value, list).
     * <ul>
     * <li>message:   reason, if method fails</li>
     * <li>details:   details, if method fails</li>
     * <li>field:     field, if method fails</li>
     * <li>value:     boolean (true/false), overall result of method</li>
     * <li>list:      list of response objects (with fields reason, details, field, value), one for each input element</li>
     * </ul>
     * </p>
     *
     * @param structureElementCommands list of structure element commands for confirm
     * @return if list of structures is valid to reject
     */
    @Hidden
    @Operation(
            summary     = "Return if structures are valid to reject by list of structure element commands for confirm",
            description = """
                          Return if structures are valid to reject by list of structure element commands for confirm.
                          If structures are valid to reject, successful reject of structures can be expected.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = """
                                   OK. Response is true if all structure element commands validated ok, false otherwise.
                                   Responses contain array with result for each structure element command. Message and details are available if no response is available.
                                   """,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = ResponseBooleanList.class))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @GetMapping(
            value = "/validateReject",
            produces = {"application/json"},
            consumes = {"application/json"})
    public ResponseEntity<ResponseBooleanList> validateStructuresReject(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Update (propose) structures by list of structure element commands for update.
     * Return list of structure elements for updated structures (proposals).
     *
     * @param structureElementCommands list of structure element commands for update
     * @return list of structure elements for updated structures (proposals)
     */
    @Operation(
            summary     = "Update (propose) structures by array of structure element commands for update",
            description = """
                          Update (propose) structures by array of structure element commands for update.
                          Return array of structure elements for updated structures (proposals).

                          Required attributes:
                          - uuid
                          - type
                          - parent (System, Subsystem, DeviceGroup, DeviceType)(uuid)
                          - name
                          - mnemonic (System, Subsystem, Discipline, DeviceType, may be set for SystemGroup, not allowed for DeviceGroup)
                          - description
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return array of structure elements for updated structures (proposals).",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = StructureElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PutMapping(
            produces = {"application/json"},
            consumes = {"application/json"})
    public List<StructureElement> updateStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandUpdate.class,
                                            requiredProperties = {"uuid","type","name","description","comment"}))))
            @RequestBody List<StructureElementCommandUpdate> structureElementCommands);

    /**
     * Update (propose) structures by upload Excel file.
     * Return Excel file with structure elements for updated structures (proposals).
     *
     * @param file file
     * @return Excel file
     */
    @Operation(
            summary     = "Update (propose) structures by upload Excel file",
            description = """
                          Update (propose) structures by upload Excel file.
                          Return Excel file with structure elements for updated structures (proposals).

                          Expected columns as in Excel template for structures.

                          Required attributes:
                          - uuid
                          - type
                          - parent (System, Subsystem, DeviceGroup, DeviceType)(uuid)
                          - name
                          - mnemonic (System, Subsystem, Discipline, DeviceType, may be set for SystemGroup, not allowed for DeviceGroup)
                          - description
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with structure elements for updated structures (proposals).",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "409",
                    description  = "Conflict. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PutMapping(
            value = "/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> updateStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with structure element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

    // ----------------------------------------------------------------------------------------------------

    /**
     * Delete (propose) structures by list of structure element commands for confirm.
     * Return list of structure elements for deleted structures (proposals).
     *
     * @param structureElementCommands list of structure element commands for confirm
     * @return list of structure elements for deleted structures (proposals)
     */
    @Operation(
            summary     = "Delete (propose) structures by array of structure element commands for confirm",
            description = """
                          Delete (propose) structures by array of structure element commands for confirm.
                          Return array of structure elements for deleted structures (proposals).

                          Required attributes:
                          - uuid
                          - type
                          - comment

                          Other attributes are not considered.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return array of structure elements for deleted structures (proposals).",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = StructureElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @DeleteMapping(
            produces = {"application/json"},
            consumes = {"application/json"})
    public List<StructureElement> deleteStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    /**
     * Delete (propose) structures by upload Excel file.
     * Return Excel file with structure elements for deleted structures (proposals).
     *
     * @param file file
     * @return Excel file
     */
    @Operation(
            summary     = "Delete (propose) structures by upload Excel file",
            description = """
                          Delete (propose) structures by upload Excel file.
                          Return Excel file with structure elements for deleted structures (proposals).

                          Expected columns as in Excel template for structures.

                          Required attributes:
                          - uuid
                          - type
                          - comment

                          Other attributes are not considered.
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with structure elements for deleted structures (proposals).",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @DeleteMapping(
            value = "/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> deleteStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with structure element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);
    // ----------------------------------------------------------------------------------------------------

    /**
     * Approve structures (proposals) by list of structure element commands for confirm.
     * Return list of structure elements for approved structures.
     *
     * <p>
     * Name is automatically created name when creation of system structure is approved.
     * </p>
     *
     * @param structureElementCommands list of structure element commands for confirm
     * @return list of structure elements for approved structures
     */
    @Operation(
            summary     = "Approve structures (proposals) by array of structure element commands for confirm",
            description = """
                          Approve structures (proposals) by array of structure element commands for confirm.
                          Return array of structure elements for approved structures.

                          Name is automatically created name when creation of system structure is approved.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return array of structure elements for approved structures.",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = StructureElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PatchMapping(
            value = "/approve",
            produces = {"application/json"},
            consumes = {"application/json"})
    public List<StructureElement> approveStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    /**
     * Approve structures (proposals) by upload Excel file.
     * Return Excel file with structure elements for approved structures.
     *
     * <p>
     * Name is automatically created name when creation of system structure is approved.
     * </p>
     *
     * @param structureElements list of structure elements
     * @return Excel file
     */
    @Operation(
            summary     = "Approve structures (proposals) by upload Excel file",
            description = """
                          Approve structures (proposals) by upload Excel file.
                          Return Excel file with structure elements for for approved structures.

                          Name is automatically created name when creation of system structure is approved.

                          Expected columns as in Excel template for structures.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with structure elements for approved structures.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PatchMapping(
            value = "/approve/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> approveStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with structure element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

    /**
     * Cancel structures (proposals) by list of structure element commands for confirm.
     * Return list of cancelled structure elements.
     *
     * @param structureElementCommands list of structure elements commands for confirm
     * @return list of structure elements for cancelled structures
     */
    @Operation(
            summary     = "Cancel structures (proposals) by array of structure element commands for confirm",
            description = """
                          Cancel structures (proposals) by array of structure element commands for confirm.
                          Return array of structure elements for cancelled structures.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
)
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return array of structure elements for cancelled structures.",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = StructureElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PatchMapping(
            value = "/cancel",
            produces = {"application/json"},
            consumes = {"application/json"})
    public List<StructureElement> cancelStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    /**
     * Cancel structures (proposals) by upload Excel file.
     * Return Excel file with cancelled structure elements.
     *
     * @param structureElements list of structure elements
     * @return Excel file
     */
    @Operation(
            summary     = "Cancel structures (proposals) by upload Excel file",
            description = """
                          Cancel structures (proposals) by upload Excel file.
                          Return Excel file with structure elements for cancelled structures.

                          Expected columns as in Excel template for structures.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
)
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with structure elements for cancelled structures.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description  = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PatchMapping(
            value = "/cancel/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> cancelStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with structure element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

    /**
     * Reject structures (proposals) by list of structure element commands for confirm.
     * Return list of rejected structure elements.
     *
     * @param structureElementCommands list of structure element commands for confirm
     * @return list of structure elements for rejected structures
     */
    @Operation(
            summary     = "Reject structures (proposals) by array of structure element commands for confirm",
            description = """
                          Reject structures (proposals) by array of structure element commands for confirm.
                          Return array of structure elements for rejected structures.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return array of structure elements for rejected structures.",
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(schema = @Schema(implementation = StructureElement.class)))),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PatchMapping(
            value = "/reject",
            produces = {"application/json"},
            consumes = {"application/json"})
    public List<StructureElement> rejectStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "array of structure element commands",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            array = @ArraySchema(
                                    schema = @Schema(
                                            implementation = StructureElementCommandConfirm.class,
                                            requiredProperties = {"uuid","type","comment"}))))
            @RequestBody List<StructureElementCommandConfirm> structureElementCommands);

    /**
     * Reject structures (proposals) by upload Excel file.
     * Return Excel file with rejected structure elements.
     *
     * @param structureElements list of structure elements
     * @return Excel file
     */
    @Operation(
            summary     = "Reject structures (proposals) by upload Excel file",
            description = """
                          Reject structures (proposals) by upload Excel file.
                          Return Excel file with structure elements for rejected structures.

                          Expected columns as in Excel template for structures.

                          Required attributes:
                          - uuid
                          - type
                          - comment
                          """
    )
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description  = "OK. Return Excel file with structure elements for rejected structures.",
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_EXCEL)),
            @ApiResponse(
                    responseCode = "400",
                    description  = "Bad request. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "404",
                    description  = "Not Found. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "422",
                    description  = "Unprocessable entity. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class))),
            @ApiResponse(
                    responseCode = "500",
                    description = "Internal server error. Reason and information such as message, details, field are available.",
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = Response.class)))
    })
    @PatchMapping(
            value = "/reject/upload",
            produces = {"application/vnd.ms-excel"},
            consumes = {"multipart/form-data"})
    public ResponseEntity<Resource> rejectStructures(
            @Parameter(
                    in = ParameterIn.DEFAULT,
                    description = "Excel file with structure element commands",
                    required = true,
                    content = @Content(mediaType = ExcelUtil.MIME_TYPE_OPENXML_SPREADSHEET))
            @RequestParam("file") MultipartFile file);

}
