/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.rest.controller;

import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.openepics.names.exception.ServiceException;
import org.openepics.names.rest.api.v1.INames;
import org.openepics.names.rest.beans.FieldName;
import org.openepics.names.rest.beans.element.NameElement;
import org.openepics.names.rest.beans.element.NameElementCommand;
import org.openepics.names.rest.beans.element.NameElementCommandConfirm;
import org.openepics.names.rest.beans.element.NameElementCommandCreate;
import org.openepics.names.rest.beans.element.NameElementCommandUpdate;
import org.openepics.names.rest.beans.response.Response;
import org.openepics.names.rest.beans.response.ResponseBoolean;
import org.openepics.names.rest.beans.response.ResponseBooleanList;
import org.openepics.names.rest.beans.response.ResponsePageNameElements;
import org.openepics.names.service.LogService;
import org.openepics.names.service.NamesService;
import org.openepics.names.util.ExcelUtil;
import org.openepics.names.util.NameCommand;
import org.openepics.names.util.NameElementUtil;
import org.openepics.names.util.TextUtil;
import org.openepics.names.util.ValidateNameElementUtil;
import org.openepics.names.util.ValidateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.google.common.collect.Lists;

/**
 * This part of REST API provides name data for Naming application.
 *
 * @author Lars Johansson
 */
@RestController
@EnableAutoConfiguration
public class NamesController implements INames {

    // TODO validate authority
    //          either
    //              none
    //              naming user
    //              naming admin (& user)

    // note
    //     global exception handler available
    //     controller layer to call validation prior to invoking service layer
    //         business logic may be validation
    //         different kinds of validation

    private static final Logger LOGGER = Logger.getLogger(NamesController.class.getName());

    private static final String ATTACHMENT_FILENAME_NAME_ELEMENT_XLSX = "attachment; filename=NameElement.xlsx";

    private LogService logService;
    private NamesService namesService;

    @Autowired
    public NamesController(
            LogService logUtil,
            NamesService namesService) {
        this.logService = logUtil;
        this.namesService = namesService;
    }

    @Override
    public ResponseEntity<List<NameElement>> createNames(List<NameElementCommandCreate> nameElementCommands) {
        // validate authority - user & admin
        // convert
        // validate
        // create names

        try {
            List<NameElementCommand> commands = NameElementUtil.convertCommandCreate2Command(nameElementCommands);
            namesService.validateNamesCreate(commands);
            return new ResponseEntity<>(namesService.createNames(commands, TextUtil.TEST_WHO), Response.getHeaderJson(), HttpStatus.CREATED);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponseEntity<Resource> createNames(MultipartFile file) {
        // validate authority - user & admin
        // convert from input
        // ( convert )
        // validate
        // create names
        // convert to output

        try {
             if (ExcelUtil.hasExcelFormat(file)) {
                 List<NameElementCommand> commands = ExcelUtil.excelToNameElementCommands(file.getInputStream(), NameCommand.CREATE);
                 namesService.validateNamesCreate(commands);
                 List<NameElement> nameElements = namesService.createNames(commands, TextUtil.TEST_WHO);
                 InputStreamResource isr = new InputStreamResource(ExcelUtil.nameElementsToExcel(nameElements));
                 return ResponseEntity.ok()
                         .header(HttpHeaders.CONTENT_DISPOSITION, ATTACHMENT_FILENAME_NAME_ELEMENT_XLSX)
                         .contentType(MediaType.parseMediaType(ExcelUtil.MIME_TYPE_EXCEL))
                         .body(isr);
             } else {
                 throw new ServiceException(TextUtil.FILE_COULD_NOT_BE_PARSED, null, null, null);
             }
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (IOException e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw new ServiceException(TextUtil.FILE_COULD_NOT_BE_PARSED, e.getMessage(), null, e);
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    // ----------------------------------------------------------------------------------------------------

    @Override
    public ResponsePageNameElements readNames(Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // validate
        // read names

        try {
            ValidateNameElementUtil.validateNamesInputRead(
                    deleted,
                    uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                    null,
                    orderBy, isAsc,
                    page, pageSize);
            return namesService.readNames(deleted,
                    uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                    orderBy, isAsc, page, pageSize);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponseEntity<Resource> readNamesDownload(Boolean deleted,
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // validate
        // read names
        // convert to output

        ResponsePageNameElements nameElements = readNames(deleted,
                uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                orderBy, isAsc, page, pageSize);
        InputStreamResource isr = new InputStreamResource(ExcelUtil.nameElementsToExcel(nameElements));
        return ResponseEntity.ok()
                .header(HttpHeaders.CONTENT_DISPOSITION, ATTACHMENT_FILENAME_NAME_ELEMENT_XLSX)
                .contentType(MediaType.parseMediaType(ExcelUtil.MIME_TYPE_EXCEL))
                .body(isr);
    }

    @Override
    public ResponsePageNameElements readNames(String name,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputName(name);
            return namesService.readNames(name, orderBy, isAsc, page, pageSize);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponsePageNameElements readNamesSystemStructure(String mnemonicPath,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputMnemonicPath(mnemonicPath);
            return namesService.readNamesSystemStructure(mnemonicPath,
                    orderBy, isAsc, page, pageSize);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponsePageNameElements readNamesDeviceStructure(String mnemonicPath,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputMnemonicPath(mnemonicPath);
            return namesService.readNamesDeviceStructure(mnemonicPath,
                    orderBy, isAsc, page, pageSize);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponsePageNameElements readNamesHistory(
            String uuid, String name, String nameEquivalence, String systemStructure, String deviceStructure, String index, String description, String who,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputAvailable(
                    !ValidateUtil.areAllNull(uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who));
            ValidateNameElementUtil.validateNamesInputRead(
                    null,
                    uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                    null,
                    orderBy, isAsc,
                    page, pageSize);
            return namesService.readNamesHistory(
                    uuid, name, nameEquivalence, systemStructure, deviceStructure, index, description, who,
                    orderBy, isAsc, page, pageSize);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponsePageNameElements readNamesHistory(String uuid,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputUuid(uuid);
            return namesService.readNamesHistory(uuid,
                    orderBy, isAsc, page, pageSize);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponsePageNameElements readNamesLegacy(String name,
            FieldName orderBy, Boolean isAsc, Integer page, Integer pageSize) {
        // not validate
        // read names
        try {
            return namesService.readNamesLegacy(name,
                    orderBy, isAsc, page, pageSize);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    // ----------------------------------------------------------------------------------------------------

    @Override
    public String equivalenceName(String name) {
        // validate
        // read

        try {
            ValidateUtil.validateInputName(name);
            return namesService.equivalenceName(name);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponseEntity<ResponseBoolean> existsName(String name) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputName(name);
            return new ResponseEntity<>(new ResponseBoolean(namesService.existsName(name)), Response.getHeaderJson(), HttpStatus.OK);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            return new ResponseEntity<>(new ResponseBoolean(Boolean.FALSE, e.getMessage(), e.getDetails(), e.getField()), Response.getHeaderJson(), HttpStatus.OK);
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            return new ResponseEntity<>(new ResponseBoolean(Boolean.FALSE, TextUtil.OPERATION_COULD_NOT_BE_PERFORMED), Response.getHeaderJson(), HttpStatus.OK);
        }
    }

    @Override
    public ResponseEntity<ResponseBoolean> isLegacyName(String name) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputName(name);
            return new ResponseEntity<>(new ResponseBoolean(namesService.isLegacyName(name)), Response.getHeaderJson(), HttpStatus.OK);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            return new ResponseEntity<>(new ResponseBoolean(Boolean.FALSE, e.getMessage(), e.getDetails(), e.getField()), Response.getHeaderJson(), HttpStatus.OK);
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            return new ResponseEntity<>(new ResponseBoolean(Boolean.FALSE, TextUtil.OPERATION_COULD_NOT_BE_PERFORMED), Response.getHeaderJson(), HttpStatus.OK);
        }
    }

    @Override
    public ResponseEntity<ResponseBoolean> isValidToCreateName(String name) {
        // validate
        // read names

        try {
            ValidateUtil.validateInputName(name);
            return new ResponseEntity<>(new ResponseBoolean(namesService.isValidToCreateName(name)), Response.getHeaderJson(), HttpStatus.OK);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            return new ResponseEntity<>(new ResponseBoolean(Boolean.FALSE, e.getMessage(), e.getDetails(), e.getField()), Response.getHeaderJson(), HttpStatus.OK);
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            return new ResponseEntity<>(new ResponseBoolean(Boolean.FALSE, TextUtil.OPERATION_COULD_NOT_BE_PERFORMED), Response.getHeaderJson(), HttpStatus.OK);
        }
    }

    // ----------------------------------------------------------------------------------------------------

    @Override
    public ResponseEntity<ResponseBooleanList> validateNamesCreate(List<NameElementCommandCreate> nameElementCommands) {
        // convert
        // validate

        List<NameElementCommand> commands = NameElementUtil.convertCommandCreate2Command(nameElementCommands);
        boolean response = true;
        String reason = "";
        List<ResponseBoolean> responses = Lists.newArrayList();
        for (NameElementCommand command : commands) {
            try {
                namesService.validateNamesCreate(command);
                responses.add(new ResponseBoolean(Boolean.TRUE));
            } catch (ServiceException e) {
                logService.logServiceException(LOGGER, Level.WARNING, e);
                logService.logStackTraceElements(LOGGER, Level.WARNING, e);
                if (response) {
                    response = false;
                    reason = TextUtil.ONE_OR_MORE_ELEMENTS_ARE_NOT_CORRECT;
                }
                responses.add(new ResponseBoolean(Boolean.FALSE, e.getMessage(), e.getDetails(), e.getField()));
            } catch (Exception e) {
                logService.logStackTraceElements(LOGGER, Level.WARNING, e);
                if (response) {
                    response = false;
                    reason = TextUtil.ONE_OR_MORE_ELEMENTS_ARE_NOT_CORRECT;
                }
                responses.add(new ResponseBoolean(Boolean.FALSE, TextUtil.OPERATION_COULD_NOT_BE_PERFORMED));
            }
        }
        return new ResponseEntity<>(new ResponseBooleanList(responses, Boolean.valueOf(response), reason), Response.getHeaderJson(), HttpStatus.OK);
    }

    @Override
    public ResponseEntity<ResponseBooleanList> validateNamesUpdate(List<NameElementCommandUpdate> nameElementCommands) {
        // convert
        // validate

        List<NameElementCommand> commands = NameElementUtil.convertCommandUpdate2Command(nameElementCommands);
        boolean response = true;
        String reason = "";
        List<ResponseBoolean> responses = Lists.newArrayList();
        for (NameElementCommand command : commands) {
            try {
                namesService.validateNamesUpdate(command);
                responses.add(new ResponseBoolean(Boolean.TRUE));
            } catch (ServiceException e) {
                logService.logServiceException(LOGGER, Level.WARNING, e);
                logService.logStackTraceElements(LOGGER, Level.WARNING, e);
                if (response) {
                    response = false;
                    reason = TextUtil.ONE_OR_MORE_ELEMENTS_ARE_NOT_CORRECT;
                }
                responses.add(new ResponseBoolean(Boolean.FALSE, e.getMessage(), e.getDetails(), e.getField()));
            } catch (Exception e) {
                logService.logStackTraceElements(LOGGER, Level.WARNING, e);
                if (response) {
                    response = false;
                    reason = TextUtil.ONE_OR_MORE_ELEMENTS_ARE_NOT_CORRECT;
                }
                responses.add(new ResponseBoolean(Boolean.FALSE, TextUtil.OPERATION_COULD_NOT_BE_PERFORMED));
            }
        }
        return new ResponseEntity<>(new ResponseBooleanList(responses, Boolean.valueOf(response), reason), Response.getHeaderJson(), HttpStatus.OK);
    }

    @Override
    public ResponseEntity<ResponseBooleanList> validateNamesDelete(List<NameElementCommandConfirm> nameElementCommands) {
        // convert
        // validate

        List<NameElementCommand> commands = NameElementUtil.convertCommandConfirm2Command(nameElementCommands);
        boolean response = true;
        String reason = "";
        List<ResponseBoolean> responses = Lists.newArrayList();
        for (NameElementCommand command : commands) {
            try {
                namesService.validateNamesDelete(command);
                responses.add(new ResponseBoolean(Boolean.TRUE));
            } catch (ServiceException e) {
                logService.logServiceException(LOGGER, Level.WARNING, e);
                logService.logStackTraceElements(LOGGER, Level.WARNING, e);
                if (response) {
                    response = false;
                    reason = TextUtil.ONE_OR_MORE_ELEMENTS_ARE_NOT_CORRECT;
                }
                responses.add(new ResponseBoolean(Boolean.FALSE, e.getMessage(), e.getDetails(), e.getField()));
            } catch (Exception e) {
                logService.logStackTraceElements(LOGGER, Level.WARNING, e);
                if (response) {
                    response = false;
                    reason = TextUtil.ONE_OR_MORE_ELEMENTS_ARE_NOT_CORRECT;
                }
                responses.add(new ResponseBoolean(Boolean.FALSE, TextUtil.OPERATION_COULD_NOT_BE_PERFORMED));
            }
        }
        return new ResponseEntity<>(new ResponseBooleanList(responses, Boolean.valueOf(response), reason), Response.getHeaderJson(), HttpStatus.OK);
    }

    // ----------------------------------------------------------------------------------------------------

    @Override
    public List<NameElement> updateNames(List<NameElementCommandUpdate> nameElementCommands) {
        // validate authority - user & admin
        // convert
        // validate
        // update names

        try {
            List<NameElementCommand> commands = NameElementUtil.convertCommandUpdate2Command(nameElementCommands);
            namesService.validateNamesUpdate(commands);
            return namesService.updateNames(commands, TextUtil.TEST_WHO);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponseEntity<Resource> updateNames(MultipartFile file) {
        // validate authority - user & admin
        // convert from input
        // ( convert )
        // validate
        // update names
        // convert to output

        try {
            if (ExcelUtil.hasExcelFormat(file)) {
                List<NameElementCommand> commands = ExcelUtil.excelToNameElementCommands(file.getInputStream(), NameCommand.UPDATE);
                namesService.validateNamesUpdate(commands);
                List<NameElement> nameElements = namesService.updateNames(commands, TextUtil.TEST_WHO);
                InputStreamResource isr = new InputStreamResource(ExcelUtil.nameElementsToExcel(nameElements));
                return ResponseEntity.ok()
                        .header(HttpHeaders.CONTENT_DISPOSITION, ATTACHMENT_FILENAME_NAME_ELEMENT_XLSX)
                        .contentType(MediaType.parseMediaType(ExcelUtil.MIME_TYPE_EXCEL))
                        .body(isr);
            } else {
                throw new ServiceException(TextUtil.FILE_COULD_NOT_BE_PARSED, null, null, null);
            }
       } catch (ServiceException e) {
           logService.logServiceException(LOGGER, Level.WARNING, e);
           logService.logStackTraceElements(LOGGER, Level.WARNING, e);
           throw e;
       } catch (IOException e) {
           logService.logStackTraceElements(LOGGER, Level.WARNING, e);
           throw new ServiceException(TextUtil.FILE_COULD_NOT_BE_PARSED, e.getMessage(), null, e);
       } catch (Exception e) {
           logService.logStackTraceElements(LOGGER, Level.WARNING, e);
           throw e;
       }
    }

    // ----------------------------------------------------------------------------------------------------

    @Override
    public List<NameElement> deleteNames(List<NameElementCommandConfirm> nameElementCommands) {
        // validate authority - user & admin
        // convert
        // validate
        // delete names

        try {
            List<NameElementCommand> commands = NameElementUtil.convertCommandConfirm2Command(nameElementCommands);
            namesService.validateNamesDelete(commands);
            return namesService.deleteNames(commands, TextUtil.TEST_WHO);
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

    @Override
    public ResponseEntity<Resource> deleteNames(MultipartFile file) {
        // validate authority - user & admin
        // convert from input
        // ( convert )
        // validate
        // delete names
        // convert to output

        try {
            if (ExcelUtil.hasExcelFormat(file)) {
                List<NameElementCommand> commands = ExcelUtil.excelToNameElementCommands(file.getInputStream(), NameCommand.DELETE);
                namesService.validateNamesDelete(commands);
                List<NameElement> nameElements = namesService.deleteNames(commands, TextUtil.TEST_WHO);
                InputStreamResource isr = new InputStreamResource(ExcelUtil.nameElementsToExcel(nameElements));
                return ResponseEntity.ok()
                        .header(HttpHeaders.CONTENT_DISPOSITION, ATTACHMENT_FILENAME_NAME_ELEMENT_XLSX)
                        .contentType(MediaType.parseMediaType(ExcelUtil.MIME_TYPE_EXCEL))
                        .body(isr);
            } else {
                throw new ServiceException(TextUtil.FILE_COULD_NOT_BE_PARSED, null, null, null);
            }
        } catch (ServiceException e) {
            logService.logServiceException(LOGGER, Level.WARNING, e);
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        } catch (IOException e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw new ServiceException(TextUtil.FILE_COULD_NOT_BE_PARSED, e.getMessage(), null, e);
        } catch (Exception e) {
            logService.logStackTraceElements(LOGGER, Level.WARNING, e);
            throw e;
        }
    }

}
