/*
 * Copyright (C) 2023 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.service;

import java.text.MessageFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang3.StringUtils;
import org.openepics.names.repository.DeviceGroupRepository;
import org.openepics.names.repository.IDeviceGroupRepository;
import org.openepics.names.repository.IDeviceTypeRepository;
import org.openepics.names.repository.model.DeviceGroup;
import org.openepics.names.repository.model.DeviceType;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.util.EssNamingConvention;
import org.openepics.names.util.HolderSystemDeviceStructure;
import org.openepics.names.util.StructureChoice;
import org.openepics.names.util.StructureCommand;
import org.openepics.names.util.StructureElementUtil;
import org.openepics.names.util.StructureUtil;
import org.openepics.names.util.TextUtil;
import org.openepics.names.util.ValidateUtil;
import org.openepics.names.util.notification.NotificationUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

/**
 * This class provides structures services for device group.
 *
 * @author Lars Johansson
 */
@Service
public class DeviceGroupService {

    // note
    //     structurecommand
    //         cud - create update delete
    //         acr - approve cancel reject

    private static final Logger LOGGER = Logger.getLogger(DeviceGroupService.class.getName());

    private IDeviceGroupRepository iDeviceGroupRepository;
    private IDeviceTypeRepository iDeviceTypeRepository;
    private DeviceGroupRepository deviceGroupRepository;
    private DeviceTypeService deviceTypeService;

    @Autowired
    public DeviceGroupService(
            IDeviceGroupRepository iDeviceGroupRepository,
            IDeviceTypeRepository iDeviceTypeRepository,
            DeviceGroupRepository deviceGroupRepository,
            DeviceTypeService deviceTypeService) {
        this.iDeviceGroupRepository = iDeviceGroupRepository;
        this.iDeviceTypeRepository = iDeviceTypeRepository;
        this.deviceGroupRepository = deviceGroupRepository;
        this.deviceTypeService = deviceTypeService;
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification createStructure(StructureElementCommand structureElementCommand, Date when, String username, EssNamingConvention namingConvention, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     create structure to pending, not latest, not deleted, with data
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     type, parent, name, mnemonic, description, comment

        // note
        //     namingConvention.equivalenceClassRepresentative return null for null input
        //     rules for mnemonic

        String mnemonic = structureElementCommand.getMnemonic();
        mnemonic = StringUtils.isEmpty(mnemonic) ? null : mnemonic;
        String equivalenceClassRepresentative = namingConvention.equivalenceClassRepresentative(mnemonic);

        // create
        DeviceGroup deviceGroup = new DeviceGroup(UUID.randomUUID(), structureElementCommand.getParent(),
                structureElementCommand.getName(), mnemonic, equivalenceClassRepresentative,
                structureElementCommand.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                when, username, structureElementCommand.getComment());
        deviceGroupRepository.createDeviceGroup(deviceGroup);

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(deviceGroup),
                NotificationUtil.prepareNotification(Type.DEVICEGROUP, StructureCommand.CREATE, null, null, deviceGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification updateStructure(StructureElementCommand structureElementCommand, Date when, String username, EssNamingConvention namingConvention, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     create structure to pending, not latest, not deleted, with data
        //     find out previous
        //     return
        //         structure element for updated structure
        //         notification
        //
        // attributes
        //     uuid, type, parent, name, mnemonic, description, comment

        // note
        //     namingConvention.equivalenceClassRepresentative return null for null input
        //     rules for mnemonic

        String mnemonic = structureElementCommand.getMnemonic();
        mnemonic = StringUtils.isEmpty(mnemonic) ? null : mnemonic;
        String equivalenceClassRepresentative = namingConvention.equivalenceClassRepresentative(mnemonic);

        // create
        DeviceGroup deviceGroup = new DeviceGroup(structureElementCommand.getUuid(), structureElementCommand.getParent(),
                structureElementCommand.getName(), mnemonic, equivalenceClassRepresentative,
                structureElementCommand.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                when, username, structureElementCommand.getComment());
        deviceGroupRepository.createDeviceGroup(deviceGroup);

        // previous
        List<DeviceGroup> previouses = iDeviceGroupRepository.findPreviousByUuidAndId(deviceGroup.getUuid().toString(),  deviceGroup.getId());

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(deviceGroup),
                NotificationUtil.prepareNotification(Type.DEVICEGROUP, StructureCommand.UPDATE, null,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, deviceGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification deleteStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     create structure to pending, not latest, deleted, with data
        //     return
        //         structure element for deleted structure
        //         notification
        //
        // attributes
        //     uuid, comment

        // find
        List<DeviceGroup> deviceGroups = deviceGroupRepository.readDeviceGroups(Status.APPROVED, Boolean.FALSE, structureElementCommand.getUuid().toString(), null, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(deviceGroups, 1)) {
            return null;
        }
        DeviceGroup toBeDeleted = deviceGroups.get(0);

        // create
        DeviceGroup deviceGroup = new DeviceGroup(toBeDeleted.getUuid(), toBeDeleted.getParentUuid(),
                toBeDeleted.getName(), toBeDeleted.getMnemonic(), toBeDeleted.getMnemonicEquivalence(),
                toBeDeleted.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.TRUE,
                when, username, structureElementCommand.getComment());
        deviceGroupRepository.createDeviceGroup(deviceGroup);

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(deviceGroup),
                NotificationUtil.prepareNotification(Type.DEVICEGROUP, StructureCommand.DELETE, null, null, deviceGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification approveStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     update not latest
        //     find
        //     approve - update structure to status APPROVED, latest to true
        //     additional
        //         find out previous to find out what is approved - create update delete
        //         approve create
        //             automatically create name when system structure is approved
        //             condition on name and structure entry
        //                 system structure should exist (uuid), one entry that is not deleted
        //                 name should not exist (system structure mnemonic)
        //             within current transaction
        //         approve update
        //             update related names
        //             for different mnemonic
        //             not for device group
        //         approve delete
        //             keep related names - legacy names
        //             delete sub structures
        //     return
        //         structure element for approved structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<DeviceGroup> deviceGroups = deviceGroupRepository.readDeviceGroups(Status.APPROVED, Boolean.FALSE, uuid, null, null, null, null, null, null, null);
        DeviceGroup deviceGroup = null;
        if (ValidateUtil.isSize(deviceGroups, 1)) {
            deviceGroup = deviceGroups.get(0);

            // update not latest
            deviceGroup.setLatest(Boolean.FALSE);
            deviceGroupRepository.updateDeviceGroup(deviceGroup);
        }

        // find
        deviceGroups = deviceGroupRepository.readDeviceGroups(Status.PENDING, null, uuid, null, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(deviceGroups, 1)) {
            return null;
        }
        deviceGroup = deviceGroups.get(0);

        // approve
        deviceGroup.setAttributesStatusProcessed(Status.APPROVED, when, username, structureElementCommand.getComment());
        deviceGroup.setLatest(Boolean.TRUE);
        deviceGroupRepository.updateDeviceGroup(deviceGroup);

        // previous
        List<DeviceGroup> previouses = iDeviceGroupRepository.findPreviousByUuidAndId(uuid, deviceGroup.getId());
        StructureCommand structureCommandCUD = StructureUtil.getStructureCommandCUD(previouses, deviceGroup);
        DeviceGroup previous = previouses != null && !previouses.isEmpty() ? previouses.get(0) : null;

        // additional
        if (StructureCommand.DELETE.equals(structureCommandCUD)) {
            // not delete names - legacy names
            // delete sub structures - delete, approve
            List<StructureElementCommand> commands = Lists.newArrayList();
            List<DeviceType> deviceTypes = iDeviceTypeRepository.findLatestNotDeletedByParent(uuid);
            for (DeviceType deviceType : deviceTypes) {
                commands.add(new StructureElementCommand(deviceType.getUuid(), Type.DEVICETYPE, null, null, null, null, StructuresService.DELETE_AFTER_APPROVE_STRUCTURE_CHANGE));
            }
            for (StructureElementCommand command : commands) {
                deviceTypeService.deleteStructure(command, when, username, holderStructure);
            }
            for (StructureElementCommand command : commands) {
                deviceTypeService.approveStructure(command, when, username, holderStructure);
            }
        }

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.APPROVE_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(deviceGroup, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.DEVICEGROUP, structureCommandCUD, StructureCommand.APPROVE, previous, deviceGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification cancelStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     cancel - update structure to status CANCELLED
        //     find out previous
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<DeviceGroup> deviceGroups = deviceGroupRepository.readDeviceGroups(Status.PENDING, null, uuid, null, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(deviceGroups, 1)) {
            return null;
        }
        DeviceGroup deviceGroup = deviceGroups.get(0);

        // cancel
        deviceGroup.setAttributesStatusProcessed(Status.CANCELLED, when, username, structureElementCommand.getComment());
        deviceGroupRepository.updateDeviceGroup(deviceGroup);

        // previous
        List<DeviceGroup> previouses = iDeviceGroupRepository.findPreviousByUuidAndId(uuid, deviceGroup.getId());
        StructureCommand structureCommandCUD = NotificationUtil.getStructureCommandCUD(previouses, deviceGroup);

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.CANCEL_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(deviceGroup, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.DEVICEGROUP, structureCommandCUD, StructureCommand.CANCEL,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, deviceGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification rejectStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     reject - update structure to status REJECTED
        //     find out previous
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<DeviceGroup> deviceGroups = deviceGroupRepository.readDeviceGroups(Status.PENDING, null, uuid, null, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(deviceGroups, 1)) {
            return null;
        }
        DeviceGroup deviceGroup = deviceGroups.get(0);

        // reject
        deviceGroup.setAttributesStatusProcessed(Status.REJECTED, when, username, structureElementCommand.getComment());
        deviceGroupRepository.updateDeviceGroup(deviceGroup);

        // previous
        List<DeviceGroup> previouses = iDeviceGroupRepository.findPreviousByUuidAndId(uuid, deviceGroup.getId());
        StructureCommand structureCommandCUD = NotificationUtil.getStructureCommandCUD(previouses, deviceGroup);

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.REJECT_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(deviceGroup, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.DEVICEGROUP, structureCommandCUD, StructureCommand.REJECT,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, deviceGroup, holderStructure));
    }

}
