/*
 * Copyright (C) 2023 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.service;

import java.text.MessageFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang3.StringUtils;
import org.openepics.names.repository.DisciplineRepository;
import org.openepics.names.repository.IDeviceGroupRepository;
import org.openepics.names.repository.IDisciplineRepository;
import org.openepics.names.repository.model.DeviceGroup;
import org.openepics.names.repository.model.Discipline;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.util.EssNamingConvention;
import org.openepics.names.util.HolderSystemDeviceStructure;
import org.openepics.names.util.StructureChoice;
import org.openepics.names.util.StructureCommand;
import org.openepics.names.util.StructureElementUtil;
import org.openepics.names.util.StructureUtil;
import org.openepics.names.util.TextUtil;
import org.openepics.names.util.ValidateUtil;
import org.openepics.names.util.notification.NotificationUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

/**
 * This class provides structures services for discipline.
 *
 * @author Lars Johansson
 */
@Service
public class DisciplineService {

    // note
    //     structurecommand
    //         cud - create update delete
    //         acr - approve cancel reject

    private static final Logger LOGGER = Logger.getLogger(DisciplineService.class.getName());

    private IDisciplineRepository iDisciplineRepository;
    private IDeviceGroupRepository iDeviceGroupRepository;
    private DisciplineRepository disciplineRepository;
    private NamesService namesService;
    private DeviceGroupService deviceGroupService;

    @Autowired
    public DisciplineService(
            IDisciplineRepository iDisciplineRepository,
            IDeviceGroupRepository iDeviceGroupRepository,
            DisciplineRepository disciplineRepository,
            NamesService namesService,
            DeviceGroupService deviceGroupService) {
        this.iDisciplineRepository = iDisciplineRepository;
        this.iDeviceGroupRepository = iDeviceGroupRepository;
        this.disciplineRepository = disciplineRepository;
        this.namesService = namesService;
        this.deviceGroupService = deviceGroupService;
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification createStructure(StructureElementCommand structureElementCommand, Date when, String username, EssNamingConvention namingConvention, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     create structure to pending, not latest, not deleted, with data
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     type, parent, name, mnemonic, description, comment

        // note
        //     namingConvention.equivalenceClassRepresentative return null for null input
        //     rules for mnemonic

        String mnemonic = structureElementCommand.getMnemonic();
        mnemonic = StringUtils.isEmpty(mnemonic) ? null : mnemonic;
        String equivalenceClassRepresentative = namingConvention.equivalenceClassRepresentative(mnemonic);

        // create
        Discipline discipline = new Discipline(UUID.randomUUID(),
                structureElementCommand.getName(), mnemonic, equivalenceClassRepresentative,
                structureElementCommand.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                when, username, structureElementCommand.getComment());
        disciplineRepository.createDiscipline(discipline);

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(discipline),
                NotificationUtil.prepareNotification(Type.DISCIPLINE, StructureCommand.CREATE, null, null, discipline, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification updateStructure(StructureElementCommand structureElementCommand, Date when, String username, EssNamingConvention namingConvention, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     create structure to pending, not latest, not deleted, with data
        //     find out previous
        //     return
        //         structure element for updated structure
        //         notification
        //
        // attributes
        //     uuid, type, parent, name, mnemonic, description, comment

        // note
        //     namingConvention.equivalenceClassRepresentative return null for null input
        //     rules for mnemonic

        String mnemonic = structureElementCommand.getMnemonic();
        mnemonic = StringUtils.isEmpty(mnemonic) ? null : mnemonic;
        String equivalenceClassRepresentative = namingConvention.equivalenceClassRepresentative(mnemonic);

        // create
        Discipline discipline = new Discipline(structureElementCommand.getUuid(),
                structureElementCommand.getName(), mnemonic, equivalenceClassRepresentative,
                structureElementCommand.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                when, username, structureElementCommand.getComment());
        disciplineRepository.createDiscipline(discipline);

        // previous
        List<Discipline> previouses = iDisciplineRepository.findPreviousByUuidAndId(discipline.getUuid().toString(),  discipline.getId());

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(discipline),
                NotificationUtil.prepareNotification(Type.DISCIPLINE, StructureCommand.UPDATE, null,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, discipline, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification deleteStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     create structure to pending, not latest, deleted, with data
        //     return
        //         structure element for deleted structure
        //         notification
        //
        // attributes
        //     uuid, comment

        // find
        List<Discipline> disciplines = disciplineRepository.readDisciplines(Status.APPROVED, Boolean.FALSE, structureElementCommand.getUuid().toString(), null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(disciplines, 1)) {
            return null;
        }
        Discipline toBeDeleted = disciplines.get(0);

        // create
        Discipline discipline = new Discipline(toBeDeleted.getUuid(),
                toBeDeleted.getName(), toBeDeleted.getMnemonic(), toBeDeleted.getMnemonicEquivalence(),
                toBeDeleted.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.TRUE,
                when, username, structureElementCommand.getComment());
        disciplineRepository.createDiscipline(discipline);

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(discipline),
                NotificationUtil.prepareNotification(Type.DISCIPLINE, StructureCommand.DELETE, null, null, discipline, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification approveStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     update not latest
        //     find
        //     approve - update structure to status APPROVED, latest to true
        //     additional
        //         find out previous to find out what is approved - create update delete
        //         approve create
        //             automatically create name when system structure is approved
        //             condition on name and structure entry
        //                 system structure should exist (uuid), one entry that is not deleted
        //                 name should not exist (system structure mnemonic)
        //             within current transaction
        //         approve update
        //             update related names
        //             for different mnemonic
        //             not for device group
        //         approve delete
        //             keep related names - legacy names
        //             delete sub structures
        //     return
        //         structure element for approved structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        List<Discipline> disciplines = disciplineRepository.readDisciplines(Status.APPROVED, Boolean.FALSE, uuid, null, null, null, null, null, null);
        Discipline discipline = null;
        if (ValidateUtil.isSize(disciplines, 1)) {
            discipline = disciplines.get(0);

            // update not latest
            discipline.setLatest(Boolean.FALSE);
            disciplineRepository.updateDiscipline(discipline);
        }

        // find
        disciplines = disciplineRepository.readDisciplines(Status.PENDING, null, uuid, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(disciplines, 1)) {
            return null;
        }
        discipline = disciplines.get(0);

        // approve
        discipline.setAttributesStatusProcessed(Status.APPROVED, when, username, structureElementCommand.getComment());
        discipline.setLatest(Boolean.TRUE);
        disciplineRepository.updateDiscipline(discipline);

        // previous
        List<Discipline> previouses = iDisciplineRepository.findPreviousByUuidAndId(uuid, discipline.getId());
        StructureCommand structureCommandCUD = StructureUtil.getStructureCommandCUD(previouses, discipline);
        Discipline previous = previouses != null && !previouses.isEmpty() ? previouses.get(0) : null;

        // additional
        if (StructureCommand.UPDATE.equals(structureCommandCUD)) {
            namesService.updateNames(previous, discipline, username);
        } else if (StructureCommand.DELETE.equals(structureCommandCUD)) {
            // not delete names - legacy names
            // delete sub structures - delete, approve
            List<StructureElementCommand> commands = Lists.newArrayList();
            List<DeviceGroup> deviceGroups = iDeviceGroupRepository.findLatestNotDeletedByParent(uuid);
            for (DeviceGroup deviceGroup : deviceGroups) {
                commands.add(new StructureElementCommand(deviceGroup.getUuid(), Type.DEVICEGROUP, null, null, null, null, StructuresService.DELETE_AFTER_APPROVE_STRUCTURE_CHANGE));
            }
            for (StructureElementCommand command : commands) {
                deviceGroupService.deleteStructure(command, when, username, holderStructure);
            }
            for (StructureElementCommand command : commands) {
                deviceGroupService.approveStructure(command, when, username, holderStructure);
            }
        }

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.APPROVE_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(discipline, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.DISCIPLINE, structureCommandCUD, StructureCommand.APPROVE, previous, discipline, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification cancelStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     cancel - update structure to status CANCELLED
        //     find out previous
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<Discipline> disciplines = disciplineRepository.readDisciplines(Status.PENDING, null, uuid, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(disciplines, 1)) {
            return null;
        }
        Discipline discipline = disciplines.get(0);

        // cancel
        discipline.setAttributesStatusProcessed(Status.CANCELLED, when, username, structureElementCommand.getComment());
        disciplineRepository.updateDiscipline(discipline);

        // previous
        List<Discipline> previouses = iDisciplineRepository.findPreviousByUuidAndId(uuid, discipline.getId());
        StructureCommand structureCommandCUD = NotificationUtil.getStructureCommandCUD(previouses, discipline);

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.CANCEL_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(discipline, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.DISCIPLINE, structureCommandCUD, StructureCommand.CANCEL,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, discipline, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification rejectStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     reject - update structure to status REJECTED
        //     find out previous
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<Discipline> disciplines = disciplineRepository.readDisciplines(Status.PENDING, null, uuid, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(disciplines, 1)) {
            return null;
        }
        Discipline discipline = disciplines.get(0);

        // reject
        discipline.setAttributesStatusProcessed(Status.REJECTED, when, username, structureElementCommand.getComment());
        disciplineRepository.updateDiscipline(discipline);

        // previous
        List<Discipline> previouses = iDisciplineRepository.findPreviousByUuidAndId(uuid, discipline.getId());
        StructureCommand structureCommandCUD = NotificationUtil.getStructureCommandCUD(previouses, discipline);

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.REJECT_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(discipline, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.DISCIPLINE, structureCommandCUD, StructureCommand.REJECT,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, discipline, holderStructure));
    }

}
