/*
 * Copyright (C) 2023 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.service;

import java.text.MessageFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang3.StringUtils;
import org.openepics.names.repository.ISystemGroupRepository;
import org.openepics.names.repository.ISystemRepository;
import org.openepics.names.repository.SystemGroupRepository;
import org.openepics.names.repository.model.System;
import org.openepics.names.repository.model.SystemGroup;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.NameElementCommand;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.util.EssNamingConvention;
import org.openepics.names.util.HolderSystemDeviceStructure;
import org.openepics.names.util.StructureChoice;
import org.openepics.names.util.StructureCommand;
import org.openepics.names.util.StructureElementUtil;
import org.openepics.names.util.StructureUtil;
import org.openepics.names.util.TextUtil;
import org.openepics.names.util.ValidateUtil;
import org.openepics.names.util.notification.NotificationUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

/**
 * This class provides structures services for system group.
 *
 * @author Lars Johansson
 */
@Service
public class SystemGroupService {

    // note
    //     structurecommand
    //         cud - create update delete
    //         acr - approve cancel reject

    private static final Logger LOGGER = Logger.getLogger(SystemGroupService.class.getName());

    private ISystemGroupRepository iSystemGroupRepository;
    private ISystemRepository iSystemRepository;
    private SystemGroupRepository systemGroupRepository;
    private NamesService namesService;
    private SystemService systemService;

    @Autowired
    public SystemGroupService(
            ISystemGroupRepository iSystemGroupRepository,
            ISystemRepository iSystemRepository,
            SystemGroupRepository systemGroupRepository,
            NamesService namesService,
            SystemService systemService) {
        this.iSystemGroupRepository = iSystemGroupRepository;
        this.iSystemRepository = iSystemRepository;
        this.systemGroupRepository = systemGroupRepository;
        this.namesService = namesService;
        this.systemService = systemService;
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification createStructure(StructureElementCommand structureElementCommand, Date when, String username, EssNamingConvention namingConvention, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     create structure to pending, not latest, not deleted, with data
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     type, parent, name, mnemonic, description, comment

        // note
        //     namingConvention.equivalenceClassRepresentative return null for null input
        //     rules for mnemonic

        String mnemonic = structureElementCommand.getMnemonic();
        mnemonic = StringUtils.isEmpty(mnemonic) ? null : mnemonic;
        String equivalenceClassRepresentative = namingConvention.equivalenceClassRepresentative(mnemonic);

        // create
        SystemGroup systemGroup = new SystemGroup(UUID.randomUUID(),
                structureElementCommand.getName(), mnemonic, equivalenceClassRepresentative,
                structureElementCommand.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                when, username, structureElementCommand.getComment());
        systemGroupRepository.createSystemGroup(systemGroup);

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(systemGroup),
                NotificationUtil.prepareNotification(Type.SYSTEMGROUP, StructureCommand.CREATE, null, null, systemGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification updateStructure(StructureElementCommand structureElementCommand, Date when, String username, EssNamingConvention namingConvention, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     create structure to pending, not latest, not deleted, with data
        //     find out previous
        //     return
        //         structure element for updated structure
        //         notification
        //
        // attributes
        //     uuid, type, parent, name, mnemonic, description, comment

        // note
        //     namingConvention.equivalenceClassRepresentative return null for null input
        //     rules for mnemonic

        String mnemonic = structureElementCommand.getMnemonic();
        mnemonic = StringUtils.isEmpty(mnemonic) ? null : mnemonic;
        String equivalenceClassRepresentative = namingConvention.equivalenceClassRepresentative(mnemonic);

        // create
        SystemGroup systemGroup = new SystemGroup(structureElementCommand.getUuid(),
                structureElementCommand.getName(), mnemonic, equivalenceClassRepresentative,
                structureElementCommand.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                when, username, structureElementCommand.getComment());
        systemGroupRepository.createSystemGroup(systemGroup);

        // previous
        List<SystemGroup> previouses = iSystemGroupRepository.findPreviousByUuidAndId(systemGroup.getUuid().toString(),  systemGroup.getId());

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(systemGroup),
                NotificationUtil.prepareNotification(Type.SYSTEMGROUP, StructureCommand.UPDATE, null,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, systemGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification deleteStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     create structure to pending, not latest, deleted, with data
        //     return
        //         structure element for deleted structure
        //         notification
        //
        // attributes
        //     uuid, comment

        // find
        List<SystemGroup> systemGroups = systemGroupRepository.readSystemGroups(Status.APPROVED, Boolean.FALSE, structureElementCommand.getUuid().toString(), null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(systemGroups, 1)) {
            return null;
        }
        SystemGroup toBeDeleted = systemGroups.get(0);

        // create
        SystemGroup systemGroup = new SystemGroup(toBeDeleted.getUuid(),
                toBeDeleted.getName(), toBeDeleted.getMnemonic(), toBeDeleted.getMnemonicEquivalence(),
                toBeDeleted.getDescription(), Status.PENDING, Boolean.FALSE, Boolean.TRUE,
                when, username, structureElementCommand.getComment());
        systemGroupRepository.createSystemGroup(systemGroup);

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementRequested(systemGroup),
                NotificationUtil.prepareNotification(Type.SYSTEMGROUP, StructureCommand.DELETE, null, null, systemGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification approveStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     update not latest
        //     find
        //     approve - update structure to status APPROVED, latest to true
        //     additional
        //         find out previous to find out what is approved - create update delete
        //         approve create
        //             automatically create name when system structure is approved
        //             condition on name and structure entry
        //                 system structure should exist (uuid), one entry that is not deleted
        //                 name should not exist (system structure mnemonic)
        //             within current transaction
        //         approve update
        //             update related names
        //             for different mnemonic
        //             not for device group
        //         approve delete
        //             keep related names - legacy names
        //             delete sub structures
        //     return
        //         structure element for approved structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<SystemGroup> systemGroups = systemGroupRepository.readSystemGroups(Status.APPROVED, Boolean.FALSE, uuid, null, null, null, null, null, null);
        SystemGroup systemGroup = null;
        if (ValidateUtil.isSize(systemGroups, 1)) {
            systemGroup = systemGroups.get(0);

            // update not latest
            systemGroup.setLatest(Boolean.FALSE);
            systemGroupRepository.updateSystemGroup(systemGroup);
        }

        // find
        systemGroups = systemGroupRepository.readSystemGroups(Status.PENDING, null, uuid, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(systemGroups, 1)) {
            return null;
        }
        systemGroup = systemGroups.get(0);

        // approve
        systemGroup.setAttributesStatusProcessed(Status.APPROVED, when, username, structureElementCommand.getComment());
        systemGroup.setLatest(Boolean.TRUE);
        systemGroupRepository.updateSystemGroup(systemGroup);

        // previous
        List<SystemGroup> previouses = iSystemGroupRepository.findPreviousByUuidAndId(uuid, systemGroup.getId());
        StructureCommand structureCommandCUD = StructureUtil.getStructureCommandCUD(previouses, systemGroup);
        SystemGroup previous = previouses != null && !previouses.isEmpty() ? previouses.get(0) : null;

        // additional
        if (StructureCommand.CREATE.equals(structureCommandCUD)) {
            boolean hasMnemonic = !StringUtils.isEmpty(systemGroup.getMnemonic());
            boolean existsName = hasMnemonic && namesService.existsName(StructureUtil.getMnemonicPath(systemGroup, holderStructure));
            if (hasMnemonic && !existsName) {
                NameElementCommand nameElement = new NameElementCommand(null, systemGroup.getUuid(), null, null, StructuresService.SYSTEM_STRUCTURE_ONLY, null);
                namesService.createName(nameElement, when, username, holderStructure);
            }
        } else if (StructureCommand.UPDATE.equals(structureCommandCUD)) {
            namesService.updateNames(previous, systemGroup, username);
        } else if (StructureCommand.DELETE.equals(structureCommandCUD)) {
            // not delete names - legacy names
            // delete sub structures - delete, approve
            List<StructureElementCommand> commands = Lists.newArrayList();
            List<System> systems = iSystemRepository.findLatestNotDeletedByParent(uuid);
            for (System system : systems) {
                commands.add(new StructureElementCommand(system.getUuid(), Type.SYSTEM, null, null, null, null, StructuresService.DELETE_AFTER_APPROVE_STRUCTURE_CHANGE));
            }
            for (StructureElementCommand command : commands) {
                systemService.deleteStructure(command, when, username, holderStructure);
            }
            for (StructureElementCommand command : commands) {
                systemService.approveStructure(command, when, username, holderStructure);
            }
        }

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.APPROVE_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(systemGroup, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.SYSTEMGROUP, structureCommandCUD, StructureCommand.APPROVE, previous, systemGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification cancelStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     cancel - update structure to status CANCELLED
        //     find out previous
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<SystemGroup> systemGroups = systemGroupRepository.readSystemGroups(Status.PENDING, null, uuid, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(systemGroups, 1)) {
            return null;
        }
        SystemGroup systemGroup = systemGroups.get(0);

        // cancel
        systemGroup.setAttributesStatusProcessed(Status.CANCELLED, when, username, structureElementCommand.getComment());
        systemGroupRepository.updateSystemGroup(systemGroup);

        // previous
        List<SystemGroup> previouses = iSystemGroupRepository.findPreviousByUuidAndId(uuid, systemGroup.getId());
        StructureCommand structureCommandCUD = NotificationUtil.getStructureCommandCUD(previouses, systemGroup);

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.CANCEL_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(systemGroup, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.SYSTEMGROUP, structureCommandCUD, StructureCommand.CANCEL,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, systemGroup, holderStructure));
    }

    @Transactional(propagation = Propagation.MANDATORY)
    public StructureElementNotification rejectStructure(StructureElementCommand structureElementCommand, Date when, String username, HolderSystemDeviceStructure holderStructure) {
        // validation outside method
        // transaction
        //     support a current transaction, throw an exception if none exists
        //     attributes
        //     find
        //     reject - update structure to status REJECTED
        //     find out previous
        //     return
        //         structure element for created structure
        //         notification
        //
        // attributes
        //     uuid, comment

        String uuid = structureElementCommand.getUuid().toString();

        // find
        List<SystemGroup> systemGroups = systemGroupRepository.readSystemGroups(Status.PENDING, null, uuid, null, null, null, null, null, null);
        if (ValidateUtil.isNullOrNotSize(systemGroups, 1)) {
            return null;
        }
        SystemGroup systemGroup = systemGroups.get(0);

        // reject
        systemGroup.setAttributesStatusProcessed(Status.REJECTED, when, username, structureElementCommand.getComment());
        systemGroupRepository.updateSystemGroup(systemGroup);

        // previous
        List<SystemGroup> previouses = iSystemGroupRepository.findPreviousByUuidAndId(uuid, systemGroup.getId());
        StructureCommand structureCommandCUD = NotificationUtil.getStructureCommandCUD(previouses, systemGroup);

        LOGGER.log(Level.FINE, () -> MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.REJECT_STRUCTURE, TextUtil.COMMAND, structureCommandCUD));

        return new StructureElementNotification(
                StructureElementUtil.getStructureElementProcessed(systemGroup, holderStructure, StructureChoice.STRUCTURE),
                NotificationUtil.prepareNotification(Type.SYSTEMGROUP, structureCommandCUD, StructureCommand.REJECT,
                        previouses != null && !previouses.isEmpty() ? previouses.get(0) : null, systemGroup, holderStructure));
    }

}
