/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.openepics.names.repository.model.DeviceGroup;
import org.openepics.names.repository.model.DeviceType;
import org.openepics.names.repository.model.Discipline;
import org.openepics.names.repository.model.Subsystem;
import org.openepics.names.repository.model.System;
import org.openepics.names.repository.model.SystemGroup;

/**
 * Utility class and holder of system and device structure content that are used to populate REST API return elements.
 * <p>
 * Class is used for performance reasons to speed up preparation of what is to be returned.
 * </p>
 *
 * @author Lars Johansson
 */
public class HolderSystemDeviceStructure {

    private HashMap<UUID, SystemGroup> systemGroups;
    private HashMap<UUID, System>      systems;
    private HashMap<UUID, Subsystem>   subsystems;

    private HashMap<UUID, Discipline>  disciplines;
    private HashMap<UUID, DeviceGroup> deviceGroups;
    private HashMap<UUID, DeviceType>  deviceTypes;

    private boolean includeDeleted = true;

    /**
    * Public constructor to prepare containers for system and device structure content.
    *
    * @param holderIRepositories
    */
   public HolderSystemDeviceStructure (HolderIRepositories holderIRepositories) {
       this(holderIRepositories, true);
   }

    /**
     * Public constructor to prepare containers for system and device structure content.
     *
     * @param holderIRepositories holder of references to repositories
     * @param includeDeleted include deleted structure entries
     */
    public HolderSystemDeviceStructure (HolderIRepositories holderIRepositories, boolean includeDeleted) {
        List<SystemGroup> systemGroupsRead = null;
        List<System> systemsRead = null;
        List<Subsystem> subsystemsRead = null;
        List<Discipline> disciplinesRead = null;
        List<DeviceGroup> deviceGroupsRead = null;
        List<DeviceType> deviceTypesRead = null;

        if (includeDeleted) {
            systemGroupsRead = holderIRepositories.systemGroupRepository().findLatest();
            systemsRead = holderIRepositories.systemRepository().findLatest();
            subsystemsRead = holderIRepositories.subsystemRepository().findLatest();
            disciplinesRead = holderIRepositories.disciplineRepository().findLatest();
            deviceGroupsRead = holderIRepositories.deviceGroupRepository().findLatest();
            deviceTypesRead = holderIRepositories.deviceTypeRepository().findLatest();
        } else {
            systemGroupsRead = holderIRepositories.systemGroupRepository().findLatestNotDeleted();
            systemsRead = holderIRepositories.systemRepository().findLatestNotDeleted();
            subsystemsRead = holderIRepositories.subsystemRepository().findLatestNotDeleted();
            disciplinesRead = holderIRepositories.disciplineRepository().findLatestNotDeleted();
            deviceGroupsRead = holderIRepositories.deviceGroupRepository().findLatestNotDeleted();
            deviceTypesRead = holderIRepositories.deviceTypeRepository().findLatestNotDeleted();
        }

        // initial capacity
        //     default load factor 0.75
        //     set at proper size to avoid rehashing
        //     size/0.75+1 (may be rounded downwards so possibly +2 instead)
        float loadFactor = 0.75f;
        this.systemGroups = new HashMap<>((int)(systemGroupsRead.size()/loadFactor + 2), loadFactor);
        this.systems      = new HashMap<>((int)(systemsRead.size()/loadFactor + 2), loadFactor);
        this.subsystems   = new HashMap<>((int)(subsystemsRead.size()/loadFactor + 2), loadFactor);
        this.disciplines  = new HashMap<>((int)(disciplinesRead.size()/loadFactor + 2), loadFactor);
        this.deviceGroups = new HashMap<>((int)(deviceGroupsRead.size()/loadFactor + 2), loadFactor);
        this.deviceTypes  = new HashMap<>((int)(deviceTypesRead.size()/loadFactor + 2), loadFactor);

        for (SystemGroup systemGroup : systemGroupsRead) {
            this.systemGroups.put(systemGroup.getUuid(), systemGroup);
        }
        for (System system : systemsRead) {
            this.systems.put(system.getUuid(), system);
        }
        for (Subsystem subsystem : subsystemsRead) {
            this.subsystems.put(subsystem.getUuid(), subsystem);
        }
        for (Discipline discipline : disciplinesRead) {
            this.disciplines.put(discipline.getUuid(), discipline);
        }
        for (DeviceGroup deviceGroup : deviceGroupsRead) {
            this.deviceGroups.put(deviceGroup.getUuid(), deviceGroup);
        }
        for (DeviceType deviceType : deviceTypesRead) {
            this.deviceTypes.put(deviceType.getUuid(), deviceType);
        }
    }

    /**
     * Find system group name part by uuid.
     *
     * @param uuid uuid
     * @return system group name part
     */
    public SystemGroup findSystemGroupByUuid(UUID uuid) {
        return systemGroups.get(uuid);
    }
    /**
     * Find system name part by uuid.
     *
     * @param uuid uuid
     * @return system name part
     */
    public System findSystemByUuid(UUID uuid) {
        return systems.get(uuid);
    }
    /**
     * Find subsystem name part by uuid.
     *
     * @param uuid uuid
     * @return subsystem name part
     */
    public Subsystem findSubsystemByUuid(UUID uuid) {
        return subsystems.get(uuid);
    }

    /**
     * Find discipline name part by uuid.
     *
     * @param uuid uuid
     * @return discipline name part
     */
    public Discipline findDisciplineByUuid(UUID uuid) {
        return disciplines.get(uuid);
    }
    /**
     * Find device group name part by uuid.
     *
     * @param uuid uuid
     * @return device group name part
     */
    public DeviceGroup findDeviceGroupByUuid(UUID uuid) {
        return deviceGroups.get(uuid);
    }
    /**
     * Find device type name part by uuid.
     *
     * @param uuid uuid
     * @return device type name part
     */
    public DeviceType findDeviceTypeByUuid(UUID uuid) {
        return deviceTypes.get(uuid);
    }

    /**
     * Return if include deleted structure entries.
     *
     * @return if include deleted structure entries
     */
    public boolean getIncludeDeleted() {
        return includeDeleted;
    }

    /**
     * Return map with key-value pairs (uuid and system group).
     *
     * @return map with key-value pairs (uuid and system group)
     */
    public Map<UUID, SystemGroup> getUuidSystemGroups() {
        return systemGroups;
    }

    /**
     * Return map with key-value pairs (uuid and system).
     *
     * @return map with key-value pairs (uuid and system)
     */
    public Map<UUID, System> getUuidSystems() {
        return systems;
    }

    /**
     * Return map with key-value pairs (uuid and subsystem).
     *
     * @return map with key-value pairs (uuid and subsystem)
     */
    public Map<UUID, Subsystem> getUuidSubsystems() {
        return subsystems;
    }

    /**
     * Return map with key-value pairs (uuid and discipline).
     *
     * @return map with key-value pairs (uuid and discipline)
     */
    public Map<UUID, Discipline> getUuidDisciplines() {
        return disciplines;
    }

    /**
     * Return map with key-value pairs (uuid and device group).
     *
     * @return map with key-value pairs (uuid and device group)
     */
    public Map<UUID, DeviceGroup> getUuidDeviceGroups() {
        return deviceGroups;
    }

    /**
     * Return map with key-value pairs (uuid and device type).
     *
     * @return map with key-value pairs (uuid and device type)
     */
    public Map<UUID, DeviceType> getUuidDeviceTypes() {
        return deviceTypes;
    }

}
