/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.openepics.names.repository.model.DeviceGroup;
import org.openepics.names.repository.model.DeviceType;
import org.openepics.names.repository.model.Discipline;
import org.openepics.names.repository.model.Structure;
import org.openepics.names.repository.model.Subsystem;
import org.openepics.names.repository.model.System;
import org.openepics.names.repository.model.SystemGroup;

import com.google.common.collect.Lists;

/**
 * Utility class to assist in handling of structure (name part) content.
 *
 * @author Lars Johansson
 */
public class StructureUtil {

    /**
     * This class is not to be instantiated.
     */
    private StructureUtil() {
        throw new IllegalStateException("Utility class");
    }

    /**
     * Return mnemonic path for system group.
     *
     * @param systemGroup system group
     * @param holderStructure holder of system and device structure content
     * @return mnemonic path
     */
    public static String getMnemonicPath(SystemGroup systemGroup, HolderSystemDeviceStructure holderStructure) {
        if (ValidateUtil.isAnyNull(systemGroup, holderStructure)) {
            return null;
        }

        return !StringUtils.isEmpty(systemGroup.getMnemonic())
                ? systemGroup.getMnemonic()
                : null;
    }

    /**
     * Return mnemonic path for system.
     *
     * @param system system
     * @param holderStructure holder of system and device structure content
     * @return mnemonic path
     */
    public static String getMnemonicPath(System system, HolderSystemDeviceStructure holderStructure) {
        if (ValidateUtil.isAnyNull(system, holderStructure)) {
            return null;
        }

        return system.getMnemonic();
    }

    /**
     * Return mnemonic path for subsystem.
     *
     * @param subsystem subsystem
     * @param holderStructure holder of system and device structure content
     * @return mnemonic path
     */
    public static String getMnemonicPath(Subsystem subsystem, HolderSystemDeviceStructure holderStructure) {
        if (ValidateUtil.isAnyNull(subsystem, holderStructure)) {
            return null;
        }

        System system = holderStructure.findSystemByUuid(subsystem.getParentUuid());
        return system != null
                ? system.getMnemonic() + "-" + subsystem.getMnemonic()
                : null;
    }

    /**
     * Return mnemonic path for discipline.
     *
     * @param discipline discipline
     * @param holderStructure holder of system and device structure content
     * @return mnemonic path
     */
    public static String getMnemonicPath(Discipline discipline, HolderSystemDeviceStructure holderStructure) {
        if (ValidateUtil.isAnyNull(discipline, holderStructure)) {
            return null;
        }

        return discipline.getMnemonic();
    }

    /**
     * Return mnemonic path for device group.
     *
     * @param deviceGroup device group
     * @param holderStructure holder of system and device structure content
     * @return mnemonic path
     */
    public static String getMnemonicPath(DeviceGroup deviceGroup, HolderSystemDeviceStructure holderStructure) {
        if (ValidateUtil.isAnyNull(deviceGroup, holderStructure)) {
            return null;
        }

        Discipline discipline = holderStructure.findDisciplineByUuid(deviceGroup.getParentUuid());
        return discipline != null
                ? discipline.getMnemonic()
                : null;
    }

    /**
     * Return mnemonic path for device type.
     *
     * @param deviceType device type
     * @param holderStructure holder of system and device structure content
     * @return mnemonic path
     */
    public static String getMnemonicPath(DeviceType deviceType, HolderSystemDeviceStructure holderStructure) {
        if (ValidateUtil.isAnyNull(deviceType, holderStructure)) {
            return null;
        }

        DeviceGroup deviceGroup = holderStructure.findDeviceGroupByUuid(deviceType.getParentUuid());
        Discipline  discipline  = deviceGroup != null
                ? holderStructure.findDisciplineByUuid(deviceGroup.getParentUuid())
                : null;
        return discipline != null && deviceGroup != null
                ? discipline.getMnemonic() + "-" + deviceType.getMnemonic()
                : null;
    }

    // --------------------------------------------------

    /**
     * Return a list of mnemonic paths for system groups.
     *
     * @param holderStructure holder of system and device structure content
     * @param mnemonicEquivalence use mnemonic equivalence instead of mnemonic
     * @param namingConvention naming convention
     * @return a list of mnemonic paths for system groups
     */
    public static List<String> getMnemonicPathsSystemGroup(HolderSystemDeviceStructure holderStructure, boolean mnemonicEquivalence, EssNamingConvention namingConvention) {
        List<String> mnemonicPaths = new ArrayList<>();
        String value = null;
        for (Entry<UUID, SystemGroup> entry : holderStructure.getUuidSystemGroups().entrySet()) {
            if (!StringUtils.isEmpty(entry.getValue().getMnemonic())) {
                value = entry.getValue().getMnemonic();
            }

            if (mnemonicEquivalence) {
                mnemonicPaths.add(namingConvention.equivalenceClassRepresentative(value));
            } else {
                mnemonicPaths.add(value);
            }
        }
        return mnemonicPaths;
    }

    /**
     * Return a list of mnemonic paths for systems.
     *
     * @param holderStructure holder of system and device structure content
     * @param mnemonicEquivalence use mnemonic equivalence instead of mnemonic
     * @param namingConvention naming convention
     * @return a list of mnemonic paths for systems
     */
    public static List<String> getMnemonicPathsSystem(HolderSystemDeviceStructure holderStructure, boolean mnemonicEquivalence, EssNamingConvention namingConvention) {
        List<String> mnemonicPaths = new ArrayList<>();
        String value = null;
        for (Entry<UUID, System> entry : holderStructure.getUuidSystems().entrySet()) {
            value = entry.getValue().getMnemonic();

            if (mnemonicEquivalence) {
                mnemonicPaths.add(namingConvention.equivalenceClassRepresentative(value));
            } else {
                mnemonicPaths.add(value);
            }
        }
        return mnemonicPaths;
    }

    /**
     * Return a list of mnemonic paths for subsystems.
     *
     * @param holderStructure holder of system and device structure content
     * @param mnemonicEquivalence use mnemonic equivalence instead of mnemonic
     * @param namingConvention naming convention
     * @return a list of mnemonic paths for subsystems
     */
    public static List<String> getMnemonicPathsSubsystem(HolderSystemDeviceStructure holderStructure, boolean mnemonicEquivalence, EssNamingConvention namingConvention) {
        List<String> mnemonicPaths = new ArrayList<>();
        String value = null;
        for (Entry<UUID, Subsystem> entry : holderStructure.getUuidSubsystems().entrySet()) {
            System system = holderStructure.findSystemByUuid(entry.getValue().getParentUuid());
            value = system.getMnemonic() + "-" + entry.getValue().getMnemonic();

            if (mnemonicEquivalence) {
                mnemonicPaths.add(namingConvention.equivalenceClassRepresentative(value));
            } else {
                mnemonicPaths.add(value);
            }
        }
        return mnemonicPaths;
    }

    /**
     * Return a list of mnemonic paths for disciplines.
     *
     * @param holderStructure holder of system and device structure content
     * @param mnemonicEquivalence use mnemonic equivalence instead of mnemonic
     * @param namingConvention naming convention
     * @return a list of mnemonic paths for disciplines
     */
    public static List<String> getMnemonicPathsDiscipline(HolderSystemDeviceStructure holderStructure, boolean mnemonicEquivalence, EssNamingConvention namingConvention) {
        List<String> mnemonicPaths = new ArrayList<>();
        String value = null;
        for (Entry<UUID, Discipline> entry : holderStructure.getUuidDisciplines().entrySet()) {
            if (!StringUtils.isEmpty(entry.getValue().getMnemonic())) {
                value = entry.getValue().getMnemonic();
            }

            if (mnemonicEquivalence) {
                mnemonicPaths.add(namingConvention.equivalenceClassRepresentative(value));
            } else {
                mnemonicPaths.add(value);
            }
        }
        return mnemonicPaths;
    }

    /**
     * Return a list of mnemonic paths for device types.
     *
     * @param holderStructure holder of system and device structure content
     * @param mnemonicEquivalence use mnemonic equivalence instead of mnemonic
     * @param namingConvention naming convention
     * @return a list of mnemonic paths for device types
     */
    public static List<String> getMnemonicPathsDeviceType(HolderSystemDeviceStructure holderStructure, boolean mnemonicEquivalence, EssNamingConvention namingConvention) {
        List<String> mnemonicPaths = new ArrayList<>();
        String value = null;
        for (Entry<UUID, DeviceType> entry : holderStructure.getUuidDeviceTypes().entrySet()) {
            DeviceGroup deviceGroup = holderStructure.findDeviceGroupByUuid(entry.getValue().getParentUuid());
            Discipline discipline = holderStructure.findDisciplineByUuid(deviceGroup.getParentUuid());
            value = discipline.getMnemonic() + "-" + entry.getValue().getMnemonic();

            if (mnemonicEquivalence) {
                mnemonicPaths.add(namingConvention.equivalenceClassRepresentative(value));
            } else {
                mnemonicPaths.add(value);
            }
        }
        return mnemonicPaths;
    }

    // --------------------------------------------------

    /**
     * Highly specialized method to find out kind of structure command (create, update, delete).
     * Intended to be used for services purposes, including notification, in approve, cancel, reject operations.
     *
     * @param previous list of previous structures
     * @param structure structure
     * @return structure command
     */
    public static StructureCommand getStructureCommandCUD(List<?> previous, Structure structure) {
        if (previous == null || previous.isEmpty()) {
            return StructureCommand.CREATE;
        } else if (structure != null && Boolean.TRUE.equals(structure.isDeleted())) {
            return StructureCommand.DELETE;
        } else {
            return StructureCommand.UPDATE;
        }
    }

    protected static List<UUID> listSystemGroup2Uuid(List<SystemGroup> systemGroups) {
        final List<UUID> listUuid = Lists.newArrayList();
        for (SystemGroup systemGroup : systemGroups) {
            listUuid.add(systemGroup.getUuid());
        }
        return listUuid;
    }
    protected static List<UUID> listSystem2Uuid(List<System> systems) {
        final List<UUID> listUuid = Lists.newArrayList();
        for (System deviceType : systems) {
            listUuid.add(deviceType.getUuid());
        }
        return listUuid;
    }
    protected static List<UUID> listSubsystem2Uuid(List<Subsystem> subsystems) {
        final List<UUID> listUuid = Lists.newArrayList();
        for (Subsystem subsystem : subsystems) {
            listUuid.add(subsystem.getUuid());
        }
        return listUuid;
    }

    protected static List<UUID> listDiscipline2Uuid(List<Discipline> disciplines) {
        final List<UUID> listUuid = Lists.newArrayList();
        for (Discipline discipline : disciplines) {
            listUuid.add(discipline.getUuid());
        }
        return listUuid;
    }
    protected static List<UUID> listDeviceGroup2Uuid(List<DeviceGroup> deviceGroups) {
        final List<UUID> listUuid = Lists.newArrayList();
        for (DeviceGroup deviceGroup : deviceGroups) {
            listUuid.add(deviceGroup.getUuid());
        }
        return listUuid;
    }
    protected static List<UUID> listDeviceType2Uuid(List<DeviceType> deviceTypes) {
        final List<UUID> listUuid = Lists.newArrayList();
        for (DeviceType deviceType : deviceTypes) {
            listUuid.add(deviceType.getUuid());
        }
        return listUuid;
    }

}
