/*
 * Copyright (C) 2023 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.util.notification;

import java.text.MessageFormat;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.lang3.StringUtils;
import org.openepics.names.repository.model.DeviceGroup;
import org.openepics.names.repository.model.DeviceType;
import org.openepics.names.repository.model.Discipline;
import org.openepics.names.repository.model.Name;
import org.openepics.names.repository.model.Structure;
import org.openepics.names.repository.model.Subsystem;
import org.openepics.names.repository.model.System;
import org.openepics.names.repository.model.SystemGroup;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.util.HolderSystemDeviceStructure;
import org.openepics.names.util.NameCommand;
import org.openepics.names.util.StructureCommand;
import org.openepics.names.util.StructureUtil;
import org.openepics.names.util.TextUtil;
import org.openepics.names.util.ValidateUtil;

/**
 * Utility class to assist in handling of notifications.
 *
 * @author Lars Johansson
 */
public class NotificationUtil {

    private static final Logger LOGGER = Logger.getLogger(NotificationUtil.class.getName());

    private static final String EMPTY = "";

    /**
     * This class is not to be instantiated.
     */
    private NotificationUtil() {
        throw new IllegalStateException("Utility class");
    }

    /**
     * Highly specialized method to find out kind of structure command (create, update, delete) for notification purposes.
     * Intended to be used for notification purposes in approve, cancel, reject operations.
     *
     * @param previous list of previous structures
     * @param structure structure
     * @return structure command
     */
    public static StructureCommand getStructureCommandCUD(List<?> previous, Structure structure) {
        if (previous == null || previous.isEmpty()) {
            return StructureCommand.CREATE;
        } else if (structure != null && Boolean.TRUE.equals(structure.isDeleted())) {
            return StructureCommand.DELETE;
        } else {
            return StructureCommand.UPDATE;
        }
    }

    /**
     * Prepare notification and add to list of notifications if notification is available.
     *
     * @param notifications list of notifications
     * @param nameCommand name command
     * @param previous previous (name)
     * @param name name
     */
    public static void prepareAddNotification(List<NotificationName> notifications, NameCommand nameCommand, Name previous, Name name) {
        if (notifications == null) {
            return;
        }

        NotificationName notification = NotificationUtil.prepareNotification(nameCommand, previous, name);
        if (notification != null) {
            notifications.add(notification);
        }
    }

    /**
     * Prepare and return notification for name with information about change.
     * If information is not correct, <tt>null</tt> is returned.
     *
     * @param nameCommand name command
     * @param previous previous (name)
     * @param name name
     * @return notification
     */
    public static NotificationName prepareNotification(NameCommand nameCommand, Name previous, Name name) {
        if (LOGGER.isLoggable(Level.FINER)) {
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "nameCommand", nameCommand));
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "previous", previous));
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "name", name));
        }

        if (ValidateUtil.isAnyNull(nameCommand, name)) {
            return null;
        }
        if (ValidateUtil.isAnyEqual(nameCommand, NameCommand.UPDATE) && previous == null) {
            return null;
        }

        // notify
        //     combinations of commands and names
        //     about information going from previous to present
        //     to be able to notify about change
        //     ------------------------------------------------
        //     create                   name
        //     update        previous + name
        //     delete                   name
        // support fields
        //     name
        //     description

        String oldIndex = EMPTY;
        String oldName = EMPTY;
        String oldDescription = EMPTY;
        String newIndex = StringUtils.stripToEmpty(name.getInstanceIndex());
        String newName = StringUtils.stripToEmpty(name.getConventionName());
        String newDescription = StringUtils.stripToEmpty(name.getDescription());

        if (ValidateUtil.isAnyEqual(nameCommand, NameCommand.UPDATE)) {
            oldIndex = StringUtils.stripToEmpty(previous.getInstanceIndex());
            oldName = StringUtils.stripToEmpty(previous.getConventionName());
            oldDescription = StringUtils.stripToEmpty(previous.getDescription());
        }

        return new NotificationName(
                nameCommand,
                name.getUuid(),
                oldIndex,
                oldName,
                oldDescription,
                newIndex,
                newName,
                newDescription,
                name.getRequestedComment(),
                name.getRequested(),
                name.getRequestedBy());
    }

    /**
     * Prepare notification and add to list of notifications if notification is available.
     *
     * @param notifications list of notifications
     * @param type type
     * @param structureCommandCUD structure command (create update delete)
     * @param structureCommandACR structure command (approve cancel reject)
     * @param previous previous (structure)
     * @param structure structure
     * @param holderStructure holder of system and device structure content
     */
    public static void prepareAddNotification(List<NotificationStructure> notifications,
            Type type, StructureCommand structureCommandCUD, StructureCommand structureCommandACR,
            Structure previous, Structure structure, HolderSystemDeviceStructure holderStructure) {
        if (notifications == null) {
            return;
        }

        NotificationStructure notification = NotificationUtil.prepareNotification(type, structureCommandCUD, structureCommandACR, previous, structure, holderStructure);
        if (notification != null) {
            notifications.add(notification);
        }
    }

    /**
     * Prepare and return notification for system group with information about change.
     * If information is not correct, <tt>null</tt> is returned.
     *
     * @param type type
     * @param structureCommandCUD structure command (create update delete)
     * @param structureCommandACR structure command (approve cancel reject)
     * @param previous previous (structure)
     * @param structure structure
     * @param holderStructure holder of system and device structure content
     * @return notification
     */
    public static NotificationStructure prepareNotification(Type type, StructureCommand structureCommandCUD, StructureCommand structureCommandACR,
            Structure previous, Structure structure, HolderSystemDeviceStructure holderStructure) {
        if (LOGGER.isLoggable(Level.FINER)) {
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "type", type));
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "structureCommandCUD", structureCommandCUD));
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "structureCommandACR", structureCommandACR));
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "previous", previous));
            LOGGER.log(Level.FINER, MessageFormat.format(TextUtil.DESCRIPTION_NAME_VALUE, TextUtil.PREPARE_NOTIFICATION, "structure", structure));
        }

        if (ValidateUtil.isAnyNull(holderStructure, type, structureCommandCUD, structure)) {
            return null;
        }
        if (ValidateUtil.isAnyEqual(structureCommandCUD, StructureCommand.APPROVE, StructureCommand.CANCEL, StructureCommand.REJECT)) {
            return null;
        }
        if (ValidateUtil.isAnyEqual(structureCommandACR, StructureCommand.CREATE, StructureCommand.UPDATE, StructureCommand.DELETE)) {
            return null;
        }
        if (ValidateUtil.isAnyEqual(structureCommandCUD, StructureCommand.UPDATE) && previous == null) {
            return null;
        }

        // type
        //     system group
        //     system
        //     subsystem
        //     discipline
        //     device group
        //     device type
        // notify - structureCommandCUD + structureCommandACR
        //     combinations of commands and structures
        //     about information going from previous to present
        //     to be able to notify about change
        //     ------------------------------------------------
        //     create                             structure
        //     create + approve                   structure
        //     create + cancel                    structure
        //     create + reject                    structure
        //     update                  previous + structure
        //     update + approve        previous + structure
        //     update + cancel         previous + structure
        //     update + reject         previous + structure
        //     delete                             structure
        //     delete + approve                   structure
        //     delete + cancel                    structure
        //     delete + reject                    structure
        // support fields
        //     name
        //     mnemonic
        //     mnemonic path - to support hierarchy
        //     description

        String oldName = EMPTY;
        String oldMnemonic = EMPTY;
        String oldMnemonicpath = EMPTY;
        String oldDescription = EMPTY;
        String newName = EMPTY;
        String newMnemonic = EMPTY;
        String newMnemonicpath = EMPTY;
        String newDescription = EMPTY;

        String comment = EMPTY;
        Date when = null;
        String who = EMPTY;

        if (structureCommandACR == null) {
            comment = structure.getRequestedComment();
            when = structure.getRequested();
            who = structure.getRequestedBy();
        } else {
            comment = structure.getProcessedComment();
            when = structure.getProcessed();
            who = structure.getProcessedBy();
        }

        comment = StringUtils.stripToEmpty(comment);
        who = StringUtils.stripToEmpty(who);

        if (ValidateUtil.isAnyEqual(structureCommandCUD, StructureCommand.UPDATE)) {
            if (Type.SYSTEMGROUP.equals(type)) {
                oldMnemonicpath = StructureUtil.getMnemonicPath((SystemGroup) previous, holderStructure);
            } else if (Type.SYSTEM.equals(type)) {
                oldMnemonicpath = StructureUtil.getMnemonicPath((System) previous, holderStructure);
            } else if (Type.SUBSYSTEM.equals(type)) {
                oldMnemonicpath = StructureUtil.getMnemonicPath((Subsystem) previous, holderStructure);
            } else if (Type.DISCIPLINE.equals(type)) {
                oldMnemonicpath = StructureUtil.getMnemonicPath((Discipline) previous, holderStructure);
            } else if (Type.DEVICEGROUP.equals(type)) {
                oldMnemonicpath = StructureUtil.getMnemonicPath((DeviceGroup) previous, holderStructure);
            } else if (Type.DEVICETYPE.equals(type)) {
                oldMnemonicpath = StructureUtil.getMnemonicPath((DeviceType) previous, holderStructure);
            }

            oldName = StringUtils.stripToEmpty(previous.getName());
            oldMnemonic = StringUtils.stripToEmpty(previous.getMnemonic());
            oldMnemonicpath = StringUtils.stripToEmpty(oldMnemonicpath);
            oldDescription = StringUtils.stripToEmpty(previous.getDescription());
        }

        if (Type.SYSTEMGROUP.equals(type)) {
            newMnemonicpath = StructureUtil.getMnemonicPath((SystemGroup) structure, holderStructure);
        } else if (Type.SYSTEM.equals(type)) {
            newMnemonicpath = StructureUtil.getMnemonicPath((System) structure, holderStructure);
        } else if (Type.SUBSYSTEM.equals(type)) {
            newMnemonicpath = StructureUtil.getMnemonicPath((Subsystem) structure, holderStructure);
        } else if (Type.DISCIPLINE.equals(type)) {
            newMnemonicpath = StructureUtil.getMnemonicPath((Discipline) structure, holderStructure);
        } else if (Type.DEVICEGROUP.equals(type)) {
            newMnemonicpath = StructureUtil.getMnemonicPath((DeviceGroup) structure, holderStructure);
        } else if (Type.DEVICETYPE.equals(type)) {
            newMnemonicpath = StructureUtil.getMnemonicPath((DeviceType) structure, holderStructure);
        }
        newName = StringUtils.stripToEmpty(structure.getName());
        newMnemonic = StringUtils.stripToEmpty(structure.getMnemonic());
        newMnemonicpath = StringUtils.stripToEmpty(newMnemonicpath);
        newDescription = StringUtils.stripToEmpty(structure.getDescription());

        return new NotificationStructure(
                structureCommandCUD,
                structureCommandACR,
                type,
                structure.getUuid(),
                oldName,
                oldMnemonic,
                oldMnemonicpath,
                oldDescription,
                newName,
                newMnemonic,
                newMnemonicpath,
                newDescription,
                comment,
                when,
                who);
    }

    /**
     * Utility method to sort name notifications according to new name ascending.
     *
     * @param notifications name notifications
     */
    public static void sortByNewName(List<NotificationName> notifications) {
        if (notifications != null) {
            Collections.sort(notifications,
                    (NotificationName arg0, NotificationName arg1) -> arg0.getNewName().compareTo(arg1.getNewName()));
        }
    }

    /**
     * Utility method to sort structure notifications according to new mnemonic path ascending.
     *
     * @param notifications structure notifications
     */
    public static void sortByNewMnemonicpath(List<NotificationStructure> notifications) {
        if (notifications != null) {
            Collections.sort(notifications,
                    (NotificationStructure arg0, NotificationStructure arg1) -> arg0.getNewMnemonicpath().compareTo(arg1.getNewMnemonicpath()));
        }
    }

}
