/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.UUID;

import org.openepics.names.docker.ITUtil.AuthorizationChoice;
import org.openepics.names.docker.ITUtil.EndpointChoice;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.rest.beans.element.StructureElementCommandConfirm;
import org.openepics.names.rest.beans.element.StructureElementCommandCreate;
import org.openepics.names.rest.beans.element.StructureElementCommandUpdate;
import org.openepics.names.rest.beans.response.ResponseBoolean;
import org.openepics.names.rest.beans.response.ResponseBooleanList;
import org.openepics.names.rest.beans.response.ResponsePageStructureElements;
import org.openepics.names.util.StructureCommand;
import org.openepics.names.util.StructureElementUtil;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * Utility class to help (Docker) integration tests for Naming and PostgreSQL.
 *
 * @author Lars Johansson
 */
public class ITUtilStructureElement {

    static final ObjectMapper mapper = new ObjectMapper();

    /**
     * This class is not to be instantiated.
     */
    private ITUtilStructureElement() {
        throw new IllegalStateException("Utility class");
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Return string for structure element command array.
     *
     * @param value structure element command array
     * @return string for structure element command array
     */
    static String object2Json(StructureElementCommandCreate[] value) {
        try {
            return mapper.writeValueAsString(value);
        } catch (JsonProcessingException e) {
            fail();
        }
        return null;
    }

    /**
     * Return string for structure element command array.
     *
     * @param value structure element command array
     * @return string for structure element command array
     */
    static String object2Json(StructureElementCommandUpdate[] value) {
        try {
            return mapper.writeValueAsString(value);
        } catch (JsonProcessingException e) {
            fail();
        }
        return null;
    }

    /**
     * Return string for structure element command array.
     *
     * @param value structure element command array
     * @return string for structure element command array
     */
    static String object2Json(StructureElementCommandConfirm[] value) {
        try {
            return mapper.writeValueAsString(value);
        } catch (JsonProcessingException e) {
            fail();
        }
        return null;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Assert that array of structure elements has expected length.
     *
     * @param actual array of structure elements
     * @param expectedLength expected length of array
     */
    static void assertContentLength(StructureElement[] actual, int expectedLength) {
        assertNotNull(actual);
        assertEquals(expectedLength, actual.length);
    }

    /**
     * Assert that array of structure elements has length one and structure element is available.
     *
     * @param actual array of structure elements
     * @return structure element
     */
    static StructureElement assertContentLengthOne(StructureElement[] actual) {
        assertContentLength(actual, 1);
        assertNotNull(actual[0]);
        return actual[0];
    }

    /**
     * Assert that response page with list of structure elements has expected length.
     *
     * @param actual response page with list of structure elements
     * @param expectedLength expected length of list
     */
    static void assertContentLength(ResponsePageStructureElements actual, int expectedLength) {
        assertNotNull(actual);
        assertEquals(expectedLength, actual.getList().size());
        assertEquals(expectedLength, actual.getListSize());
    }

    /**
     * Assert that response page with list of structure elements has length one and structure element is available.
     *
     * @param actual response page with list of structure elements
     * @return structure element
     */
    static StructureElement assertContentLengthOne(ResponsePageStructureElements actual) {
        assertContentLength(actual, 1);
        assertNotNull(actual.getList().get(0));
        return actual.getList().get(0);
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Assert that arrays are equal with same length and same content in each array position.
     *
     * @param actual actual array of structure elements
     * @param expected expected arbitrary number of structure elements
     */
    static void assertEqualsStructureElements(StructureElement[] actual, StructureElement... expected) {
        if (expected != null) {
            assertNotNull(actual);
            assertEquals(expected.length, actual.length);
            for (int i=0; i<expected.length; i++) {
                assertTrue(expected[i].equals(actual[i]));
            }
        } else {
            assertNull(actual);
        }
    }

    /**
     * Assert content of structure element.
     *
     * @param actual structure element
     * @param expectedType expected type
     * @param expectedUuid expected uuid
     * @param expectedParent expected parent
     * @param expectedName expected name
     * @param expectedMnemonic expected mnemonic
     * @param expectedMnemonicPath expected mnemonic path
     * @param expectedLevel expected level
     * @param expectedDescription expected description
     * @param expectedStatus expected status
     * @param expectedIsLatest expected latest
     * @param expectedIsDeleted expected deleted
     * @param expectedWho expected who
     * @param expectedComment expected comment
     */
    static void assertContent(StructureElement actual,
            Type expectedType, UUID expectedUuid, UUID expectedParent,
            String expectedName, String expectedMnemonic, String expectedMnemonicPath, Integer expectedLevel,
            String expectedDescription, Status expectedStatus, Boolean expectedIsLatest, Boolean expectedIsDeleted,
            String expectedWho, String expectedComment) {
        assertNotNull(actual);
        assertEquals(expectedType, actual.getType());
        assertEquals(expectedUuid, actual.getUuid());
        assertEquals(expectedParent, actual.getParent());
        assertEquals(expectedName, actual.getName());
        assertEquals(expectedMnemonic, actual.getMnemonic());
        assertEquals(expectedMnemonicPath, actual.getMnemonicPath());
        assertEquals(expectedLevel, actual.getLevel());
        assertEquals(expectedDescription, actual.getDescription());
        assertEquals(expectedStatus, actual.getStatus());
        assertEquals(expectedIsLatest, actual.isLatest());
        assertEquals(expectedIsDeleted, actual.isDeleted());
        assertNotNull(actual.getWhen());
        assertEquals(expectedWho, actual.getWho());
        assertEquals(expectedComment, actual.getComment());
    }

    /**
     * Assert content of structure element.
     *
     * @param actual structure element
     * @param expected structure element command
     */
    static void assertContent(StructureElement actual, StructureElementCommand expected) {
        assertNotNull(actual);
        if (expected != null) {
            assertEquals(expected.getUuid(), actual.getUuid());
            assertEquals(expected.getType(), actual.getType());
            assertEquals(expected.getParent(), actual.getParent());
            assertEquals(expected.getName(), actual.getName());
            assertEquals(expected.getMnemonic(), actual.getMnemonic());
            assertEquals(expected.getDescription(), actual.getDescription());
            assertEquals(expected.getComment(), actual.getComment());
        }
    }


    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to read structure elements.
     *
     * @param queryString query string
     * @return response page structure elements
     */
    public static ResponsePageStructureElements assertRead(String queryString) {
        return assertRead(queryString, -1, -1);
    }
    /**
     * Utility method to read structure elements and assert number of items in list.
     *
     * @param queryString query string
     * @param expectedEqual (if non-negative number) equal to this number of items
     * @return response page structure elements
     */
    public static ResponsePageStructureElements assertRead(String queryString, int expectedEqual) {
        return assertRead(queryString, expectedEqual, expectedEqual);
    }
    /**
     * Utility method to read structure elements and assert number of items in list.
     *
     * @param queryString query string
     * @param expectedGreaterThanOrEqual (if non-negative number) greater than or equal to this number of items
     * @param expectedLessThanOrEqual (if non-negative number) less than or equal to this number of items
     * @return response page structure elements
     */
    public static ResponsePageStructureElements assertRead(String queryString, int expectedGreaterThanOrEqual, int expectedLessThanOrEqual) {
        try {
            String[] response = null;
            ResponsePageStructureElements responsePageStructureElements = null;

            response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_STRUCTURES + queryString);
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            responsePageStructureElements = mapper.readValue(response[1], ResponsePageStructureElements.class);

            // expected number of items in list
            //     (if non-negative number)
            //     expectedGreaterThanOrEqual <= nbr of items <= expectedLessThanOrEqual
            if (expectedGreaterThanOrEqual >= 0) {
                assertTrue(responsePageStructureElements.getListSize() >= expectedGreaterThanOrEqual);
            }
            if (expectedLessThanOrEqual >= 0) {
                assertTrue(responsePageStructureElements.getListSize() <= expectedLessThanOrEqual);
            }

            // expected value for latest for items in list
            assertLatest(responsePageStructureElements);

            return responsePageStructureElements;
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
        return null;
    }

    /**
     * Assert that response page with list of structure elements has maximum one entry that is latest
     * for any given uuid, which may not have status {@link Status#PENDING}.
     *
     * @param actual response page with list of structure elements
     */
    public static void assertLatest(ResponsePageStructureElements actual) {
        // latest
        //     status pending not with latest
        //     maximum one item for any given uuid in list may be latest

        assertNotNull(actual);
        Map<UUID, Long> mapUuidCountLatest = new TreeMap<>();
        for (StructureElement structureElement : actual.getList()) {
            if (structureElement.isLatest()) {
                ITUtil.addOne(structureElement.getUuid(), mapUuidCountLatest);
                assertNotEquals(Status.PENDING, structureElement.getStatus());
            }
        }
        for (Entry<UUID, Long> entry : mapUuidCountLatest.entrySet()) {
            assertTrue(entry.getValue() <= 1);
        }
    }

    /**
     * Utility method to get name equivalence of a mnemonic or name.
     *
     * @param name name
     * @param expected expected response
     */
    public static void assertEquivalence(String name, String expected) {
        try {
            String[] response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_STRUCTURES + "/equivalence/" + name);
            ITUtil.assertResponseLength2CodeOKContent(response, expected);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to check if structure exists.
     *
     * @param type type
     * @param mnemonicPath mnemonic path
     * @param expected expected response
     */
    public static void assertExists(Type type, String mnemonicPath, Boolean expected) {
        assertExists(type, mnemonicPath, expected, null);
    }
    /**
     * Utility method to check if structure exists.
     *
     * @param type type
     * @param mnemonicPath mnemonic path
     * @param expected expected response
     * @param expectedMessageEmpty expected message empty
     */
    public static void assertExists(Type type, String mnemonicPath, Boolean expected, Boolean expectedMessageEmpty) {
        try {
            String[] response = null;

            response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_STRUCTURES + "/exists/" + type.toString() + "/" + mnemonicPath);
            ITUtil.assertResponseLength2CodeOK(response);
            if (expectedMessageEmpty != null) {
                ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBoolean.class), expected, expectedMessageEmpty);
            } else {
                ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBoolean.class), expected);
            }
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to check if structure is valid to create.
     * Method will not produce a message.
     *
     * @param type type
     * @param mnemonicPath mnemonic path
     * @param expected expected response
     */
    public static void assertIsValidToCreate(Type type, String mnemonicPath, Boolean expected) {
        try {
            String[] response = null;

            response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_STRUCTURES + "/isValidToCreate/" + type.toString() + "/" + mnemonicPath);
            ITUtil.assertResponseLength2CodeOK(response);
            ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBoolean.class), expected);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to validate structure element command for create and assert expected response.
     *
     * @param structureElementCommand structure element command
     * @param structureCommand structure command
     * @param expected expected response
     */
    public static void assertValidate(StructureElementCommandCreate structureElementCommand, StructureCommand structureCommand, Boolean expected) {
        assertValidate(AuthorizationChoice.NONE, object2Json(new StructureElementCommandCreate[] {structureElementCommand}), structureCommand, HttpURLConnection.HTTP_OK, expected);
    }
    /**
     * Utility method to validate structure element command for update and assert expected response.
     *
     * @param structureElementCommand structure element command
     * @param structureCommand structure command
     * @param expected expected response
     */
    public static void assertValidate(StructureElementCommandUpdate structureElementCommand, StructureCommand structureCommand, Boolean expected) {
        assertValidate(AuthorizationChoice.NONE, object2Json(new StructureElementCommandUpdate[] {structureElementCommand}), structureCommand, HttpURLConnection.HTTP_OK, expected);
    }
    /**
     * Utility method to validate structure element command for confirm and assert expected response.
     *
     * @param structureElementCommand structure element command
     * @param structureCommand structure command
     * @param expected expected response
     */
    public static void assertValidate(StructureElementCommandConfirm structureElementCommand, StructureCommand structureCommand, Boolean expected) {
        assertValidate(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), structureCommand, HttpURLConnection.HTTP_OK, expected);
    }
    /**
     * Utility method to validate structure element commands for create and assert expected response.
     *
     * @param structureElementCommands structure element commands
     * @param structureCommand structure command
     * @param expected expected response
     */
    public static void assertValidate(StructureElementCommandCreate[] structureElementCommands, StructureCommand structureCommand, Boolean expected) {
        assertValidate(AuthorizationChoice.NONE, object2Json(structureElementCommands), structureCommand, HttpURLConnection.HTTP_OK, expected);
    }
    /**
     * Utility method to validate structure element commands for update and assert expected response.
     *
     * @param structureElementCommands structure element commands
     * @param structureCommand structure command
     * @param expected expected response
     */
    public static void assertValidate(StructureElementCommandUpdate[] structureElementCommands, StructureCommand structureCommand, Boolean expected) {
        assertValidate(AuthorizationChoice.NONE, object2Json(structureElementCommands), structureCommand, HttpURLConnection.HTTP_OK, expected);
    }
    /**
     * Utility method to validate structure element commands for confirm and assert expected response.
     *
     * @param structureElementCommands structure element commands
     * @param structureCommand structure command
     * @param expected expected response
     */
    public static void assertValidate(StructureElementCommandConfirm[] structureElementCommands, StructureCommand structureCommand, Boolean expected) {
        assertValidate(AuthorizationChoice.NONE, object2Json(structureElementCommands), structureCommand, HttpURLConnection.HTTP_OK, expected);
    }
    /**
     * Utility method to validate structure elements and assert expected response.
     *
     * @param json json
     * @param structureCommand structure command
     * @param responseCode response code
     */
    public static void assertValidate(String json, StructureCommand structureCommand, int responseCode) {
        assertValidate(AuthorizationChoice.NONE, json, structureCommand, responseCode, null);
    }
    /**
     * Utility method to validate structure elements and assert expected response.
     *
     * @param authorizationChoice authorization choice (none, user, admin)
     * @param json json
     * @param structureCommand structure command
     * @param responseCode response code
     * @param expected expected response
     */
    public static void assertValidate(AuthorizationChoice authorizationChoice, String json, StructureCommand structureCommand, int responseCode, Boolean expected) {
        String path = getValidatePath(structureCommand);

        try {
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlGetPathJson(authorizationChoice, EndpointChoice.STRUCTURES, path, json));
            ITUtil.assertResponseLength2Code(response, responseCode);
            if (expected != null) {
                ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBooleanList.class), expected);
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to return validate path for structure command.
     *
     * @param structureCommand structure command
     * @return validate path
     */
    private static String getValidatePath(StructureCommand structureCommand) {
        String validatePath = "";
        switch (structureCommand) {
            case CREATE:
                validatePath = "/validateCreate";
                break;
            case UPDATE:
                validatePath = "/validateUpdate";
                break;
            case DELETE:
                validatePath = "/validateDelete";
                break;
            case APPROVE:
                validatePath = "/validateApprove";
                break;
            case CANCEL:
                validatePath = "/validateCancel";
                break;
            case REJECT:
                validatePath = "/validateReject";
                break;
            default:
                break;
        }
        return validatePath;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to create a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return created structure element
     */
    public static StructureElement assertCreate(StructureElementCommandCreate structureElementCommand) {
        StructureElement[] createdStructureElements = assertCreate(AuthorizationChoice.NONE, object2Json(new StructureElementCommandCreate[] {structureElementCommand}), HttpURLConnection.HTTP_CREATED);
        return createdStructureElements != null
                ? createdStructureElements[0]
                : null;
    }
    /**
     * Utility method to create a structure element and assert result and response code.
     *
     * @param structureElementCommand structure element command
     * @param responseCode response code
     * @return created structure element
     */
    public static StructureElement assertCreate(StructureElementCommandCreate structureElementCommand, int responseCode) {
        StructureElement[] createdStructureElements = assertCreate(AuthorizationChoice.NONE, object2Json(new StructureElementCommandCreate[] {structureElementCommand}), responseCode);
        return createdStructureElements != null
                ? createdStructureElements[0]
                : null;
    }
    /**
     * Utility method to create structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return created structure element
     */
    public static StructureElement[] assertCreate(StructureElementCommandCreate[] structureElementCommands) {
        return assertCreate(AuthorizationChoice.NONE, object2Json(structureElementCommands), HttpURLConnection.HTTP_CREATED);
    }
    /**
     * Utility method to create json (structure elements) and assert result and response code.
     *
     * @param json json
     * @param responseCode response code
     * @return created structure element
     */
    public static StructureElement[] assertCreate(String json, int responseCode) {
        return assertCreate(AuthorizationChoice.NONE, json, responseCode);
    }
    /**
     * Utility method to create json (structure elements) and assert result and response code.
     *
     * @param authorizationChoice authorization choice (none, user, admin)
     * @param json json
     * @param responseCode response code
     * @return created structure element
     */
    public static StructureElement[] assertCreate(AuthorizationChoice authorizationChoice, String json, int responseCode) {
        StructureElement[] createdStructureElements  = null;

        try {
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlPostPathJson(authorizationChoice, EndpointChoice.STRUCTURES, "", json));
            ITUtil.assertResponseLength2Code(response, responseCode);

            if (HttpURLConnection.HTTP_CREATED == responseCode) {
                createdStructureElements = mapper.readValue(response[1], StructureElement[].class);

                for (StructureElement createdStructureElement : createdStructureElements) {
                    assertNotNull(createdStructureElement.getUuid());
                    assertEquals(Status.PENDING, createdStructureElement.getStatus());
                    assertEquals(Boolean.FALSE, createdStructureElement.isLatest());
                    assertEquals(Boolean.FALSE, createdStructureElement.isDeleted());
                    assertNotNull(createdStructureElement.getWhen());
                }

                // not set uuid for input parameter(s)
                //     convenience, easy when array length 1, otherwise not easy
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return createdStructureElements;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to update a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return updated structure element
     */
    public static StructureElement assertUpdate(StructureElementCommandUpdate structureElementCommand) {
        StructureElement[] updatedStructureElements = assertUpdate(AuthorizationChoice.NONE, object2Json(new StructureElementCommandUpdate[] {structureElementCommand}), HttpURLConnection.HTTP_OK);
        return updatedStructureElements != null
                ? updatedStructureElements[0]
                : null;
    }
    /**
     * Utility method to update a structure element and assert result and response code.
     *
     * @param structureElementCommand structure element command
     * @param responseCode response code
     * @return updated structure element
     */
    public static StructureElement assertUpdate(StructureElementCommandUpdate structureElementCommand, int responseCode) {
        StructureElement[] updatedStructureElements = assertUpdate(AuthorizationChoice.NONE, object2Json(new StructureElementCommandUpdate[] {structureElementCommand}), responseCode);
        return updatedStructureElements != null
                ? updatedStructureElements[0]
                : null;
    }
    /**
     * Utility method to update structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return updated structure element
     */
    public static StructureElement[] assertUpdate(StructureElementCommandUpdate[] structureElementCommands) {
        return assertUpdate(AuthorizationChoice.NONE, object2Json(structureElementCommands), HttpURLConnection.HTTP_OK);
    }
    /**
     * Utility method to update json (structure elements) and assert result and response code.
     *
     * @param authorizationChoice authorization choice (none, user, admin)
     * @param json json
     * @param responseCode response code
     * @return updated structure element
     */
    public static StructureElement[] assertUpdate(AuthorizationChoice authorizationChoice, String json, int responseCode) {
        StructureElement[] updatedStructureElements  = null;

        try {
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlPutPathJson(authorizationChoice, EndpointChoice.STRUCTURES, "", json));
            ITUtil.assertResponseLength2Code(response, responseCode);

            if (HttpURLConnection.HTTP_OK == responseCode) {
                updatedStructureElements = mapper.readValue(response[1], StructureElement[].class);

                for (StructureElement updatedStructureElement : updatedStructureElements) {
                    assertNotNull(updatedStructureElement.getUuid());
                    assertEquals(Status.PENDING, updatedStructureElement.getStatus());
                    assertEquals(Boolean.FALSE, updatedStructureElement.isLatest());
                    assertEquals(Boolean.FALSE, updatedStructureElement.isDeleted());
                    assertNotNull(updatedStructureElement.getWhen());
                }
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return updatedStructureElements;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to delete a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return deleted structure element
     */
    public static StructureElement assertDelete(StructureElementCommandConfirm structureElementCommand) {
        StructureElement[] deletedStructureElements = assertDelete(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), HttpURLConnection.HTTP_OK);
        return deletedStructureElements != null
                ? deletedStructureElements[0]
                : null;
    }
    /**
     * Utility method to delete a structure element and assert result and response code.
     *
     * @param structureElementCommand structure element command
     * @param responseCode response code
     * @return deleted structure element
     */
    public static StructureElement assertDelete(StructureElementCommandConfirm structureElementCommand, int responseCode) {
        StructureElement[] deletedStructureElements = assertDelete(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), responseCode);
        return deletedStructureElements != null
                ? deletedStructureElements[0]
                : null;
    }
    /**
     * Utility method to delete structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return deleted structure element
     */
    public static StructureElement[] assertDelete(StructureElementCommandConfirm[] structureElementCommands) {
        return assertDelete(AuthorizationChoice.NONE, object2Json(structureElementCommands), HttpURLConnection.HTTP_OK);
    }
    /**
     * Utility method to delete json (structure elements) and assert result and response code.
     *
     * @param authorizationChoice authorization choice (none, user, admin)
     * @param json json
     * @param responseCode response code
     * @return deleted structure element
     */
    public static StructureElement[] assertDelete(AuthorizationChoice authorizationChoice, String json, int responseCode) {
        StructureElement[] deletedStructureElements  = null;

        try {
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlDeletePathJson(authorizationChoice, EndpointChoice.STRUCTURES, "", json));
            ITUtil.assertResponseLength2Code(response, responseCode);

            if (HttpURLConnection.HTTP_OK == responseCode) {
                deletedStructureElements = mapper.readValue(response[1], StructureElement[].class);

                for (StructureElement deletedStructureElement : deletedStructureElements) {
                    assertNotNull(deletedStructureElement.getUuid());
                    assertEquals(Status.PENDING, deletedStructureElement.getStatus());
                    assertEquals(Boolean.FALSE, deletedStructureElement.isLatest());
                    assertEquals(Boolean.TRUE, deletedStructureElement.isDeleted());
                    assertNotNull(deletedStructureElement.getWhen());
                }
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return deletedStructureElements;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to approve a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return approved structure element
     */
    public static StructureElement assertApprove(StructureElementCommandConfirm structureElementCommand) {
        StructureElement[] approvedStructureElements = assertApprove(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), HttpURLConnection.HTTP_OK);
        return approvedStructureElements != null
                ? approvedStructureElements[0]
                : null;
    }
    /**
     * Utility method to approve a structure element and assert result and response code.
     *
     * @param structureElementCommand structure element command
     * @param responseCode response code
     * @return approved structure element
     */
    public static StructureElement assertApprove(StructureElementCommandConfirm structureElementCommand, int responseCode) {
        StructureElement[] approvedStructureElements = assertApprove(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), responseCode);
        return approvedStructureElements != null
                ? approvedStructureElements[0]
                : null;
    }
    /**
     * Utility method to approve structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return approved structure element
     */
    public static StructureElement[] assertApprove(StructureElementCommandConfirm[] structureElementCommands) {
        return assertApprove(AuthorizationChoice.NONE, object2Json(structureElementCommands), HttpURLConnection.HTTP_OK);
    }
    /**
     * Utility method to approve json (structure elements) and assert result and response code.
     *
     * @param authorizationChoice authorization choice (none, user, admin)
     * @param json json
     * @param responseCode response code
     * @return approved structure element
     */
    public static StructureElement[] assertApprove(AuthorizationChoice authorizationChoice, String json, int responseCode) {
        StructureElement[] approvedStructureElements = null;

        try {
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlPatchPathJson(authorizationChoice, EndpointChoice.STRUCTURES, "/approve", json));
            ITUtil.assertResponseLength2Code(response, responseCode);

            if (HttpURLConnection.HTTP_OK == responseCode) {
                approvedStructureElements = mapper.readValue(response[1], StructureElement[].class);

                for (StructureElement approvedStructureElement : approvedStructureElements) {
                    assertNotNull(approvedStructureElement.getUuid());
                    assertEquals(Status.APPROVED, approvedStructureElement.getStatus());
                    assertEquals(Boolean.TRUE, approvedStructureElement.isLatest());
                    // not known if deleted
                    assertNotNull(approvedStructureElement.getWhen());
                }
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return approvedStructureElements;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to cancel a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return cancelled structure element
     */
    public static StructureElement assertCancel(StructureElementCommandConfirm structureElementCommand) {
        StructureElement[] cancelledStructureElements = assertCancel(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), HttpURLConnection.HTTP_OK);
        return cancelledStructureElements != null
                ? cancelledStructureElements[0]
                : null;
    }
    /**
     * Utility method to cancel a structure element and assert result and response code.
     *
     * @param structureElementCommand structure element command
     * @param responseCode response code
     * @return cancelled structure element
     */
    public static StructureElement assertCancel(StructureElementCommandConfirm structureElementCommand, int responseCode) {
        StructureElement[] cancelledStructureElements = assertCancel(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), responseCode);
        return cancelledStructureElements != null
                ? cancelledStructureElements[0]
                : null;
    }
    /**
     * Utility method to cancel structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return cancelled structure element
     */
    public static StructureElement[] assertCancel(StructureElementCommandConfirm[] structureElementCommands) {
        return assertCancel(AuthorizationChoice.NONE, object2Json(structureElementCommands), HttpURLConnection.HTTP_OK);
    }
    /**
     * Utility method to cancel json (structure elements) and assert result and response code.
     *
     * @param authorizationChoice authorization choice (none, user, admin)
     * @param json json
     * @param responseCode response code
     * @return cancelled structure element
     */
    public static StructureElement[] assertCancel(AuthorizationChoice authorizationChoice, String json, int responseCode) {
        StructureElement[] cancelledStructureElements = null;

        try {
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlPatchPathJson(authorizationChoice, EndpointChoice.STRUCTURES, "/cancel", json));
            ITUtil.assertResponseLength2Code(response, responseCode);

            if (HttpURLConnection.HTTP_OK == responseCode) {
                cancelledStructureElements = mapper.readValue(response[1], StructureElement[].class);

                for (StructureElement cancelledStructureElement : cancelledStructureElements) {
                    assertNotNull(cancelledStructureElement.getUuid());
                    assertEquals(Status.CANCELLED, cancelledStructureElement.getStatus());
                    assertEquals(Boolean.FALSE, cancelledStructureElement.isLatest());
                    // not known if deleted
                    assertNotNull(cancelledStructureElement.getWhen());
                }
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return cancelledStructureElements;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to reject a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return rejected structure element
     */
    public static StructureElement assertReject(StructureElementCommandConfirm structureElementCommand) {
        StructureElement[] rejectedStructureElements = assertReject(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), HttpURLConnection.HTTP_OK);
        return rejectedStructureElements != null
                ? rejectedStructureElements[0]
                : null;
    }
    /**
     * Utility method to reject a structure element and assert result and response code.
     *
     * @param structureElementCommand structure element command
     * @param responseCode response code
     * @return rejected structure element
     */
    public static StructureElement assertReject(StructureElementCommandConfirm structureElementCommand, int responseCode) {
        StructureElement[] rejectedStructureElements = assertReject(AuthorizationChoice.NONE, object2Json(new StructureElementCommandConfirm[] {structureElementCommand}), responseCode);
        return rejectedStructureElements != null
                ? rejectedStructureElements[0]
                : null;
    }
    /**
     * Utility method to reject structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return rejected structure element
     */
    public static StructureElement[] assertReject(StructureElementCommandConfirm[] structureElementCommands) {
        return assertReject(AuthorizationChoice.NONE, object2Json(structureElementCommands), HttpURLConnection.HTTP_OK);
    }
    /**
     * Utility method to reject json (structure elements) and assert result and response code.
     *
     * @param authorizationChoice authorization choice (none, user, admin)
     * @param json json
     * @param responseCode response code
     * @return rejected structure element
     */
    public static StructureElement[] assertReject(AuthorizationChoice authorizationChoice, String json, int responseCode) {
        StructureElement[] rejectedStructureElements = null;

        try {
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlPatchPathJson(authorizationChoice, EndpointChoice.STRUCTURES, "/reject", json));
            ITUtil.assertResponseLength2Code(response, responseCode);

            if (HttpURLConnection.HTTP_OK == responseCode) {
                rejectedStructureElements = mapper.readValue(response[1], StructureElement[].class);

                for (StructureElement rejectedStructureElement : rejectedStructureElements) {
                    assertNotNull(rejectedStructureElement.getUuid());
                    assertEquals(Status.REJECTED, rejectedStructureElement.getStatus());
                    assertEquals(Boolean.FALSE, rejectedStructureElement.isLatest());
                    // not known if deleted
                    assertNotNull(rejectedStructureElement.getWhen());
                }
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return rejectedStructureElements;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to create, approve a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return approved structure element (after create)
     */
    public static StructureElement assertCreateApprove(StructureElementCommandCreate structureElementCommand) {
        return assertApprove(StructureElementUtil.convertElement2CommandConfirm(assertCreate(structureElementCommand)));
    }
    /**
     * Utility method to create, approve structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return approved structure elements (after create)
     */
    public static StructureElement[] assertCreateApprove(StructureElementCommandCreate[] structureElementCommands) {
        return assertApprove(StructureElementUtil.convertElement2CommandConfirm(assertCreate(structureElementCommands)));
    }

    /**
     * Utility method to create, cancel a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return cancelled structure element (after create)
     */
    public static StructureElement assertCreateCancel(StructureElementCommandCreate structureElementCommand) {
        return assertCancel(StructureElementUtil.convertElement2CommandConfirm(assertCreate(structureElementCommand)));
    }
    /**
     * Utility method to create, cancel structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return cancelled structure elements (after create)
     */
    public static StructureElement[] assertCreateCancel(StructureElementCommandCreate[] structureElementCommands) {
        return assertCancel(StructureElementUtil.convertElement2CommandConfirm(assertCreate(structureElementCommands)));
    }

    /**
     * Utility method to create, reject a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return rejected structure element (after create)
     */
    public static StructureElement assertCreateReject(StructureElementCommandCreate structureElementCommand) {
        return assertReject(StructureElementUtil.convertElement2CommandConfirm(assertCreate(structureElementCommand)));
    }
    /**
     * Utility method to create, reject structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return rejected structure elements (after create)
     */
    public static StructureElement[] assertCreateReject(StructureElementCommandCreate[] structureElementCommands) {
        return assertReject(StructureElementUtil.convertElement2CommandConfirm(assertCreate(structureElementCommands)));
    }

    /**
     * Utility method to update, approve a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return approved structure element (after update)
     */
    public static StructureElement assertUpdateApprove(StructureElementCommandUpdate structureElementCommand) {
        return assertApprove(StructureElementUtil.convertElement2CommandConfirm(assertUpdate(structureElementCommand)));
    }
    /**
     * Utility method to update, approve structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return approved structure elements (after update)
     */
    public static StructureElement[] assertUpdateApprove(StructureElementCommandUpdate[] structureElementCommands) {
        return assertApprove(StructureElementUtil.convertElement2CommandConfirm(assertUpdate(structureElementCommands)));
    }

    /**
     * Utility method to update, cancel a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return cancelled structure element (after update)
     */
    public static StructureElement assertUpdateCancel(StructureElementCommandUpdate structureElementCommand) {
        return assertCancel(StructureElementUtil.convertElement2CommandConfirm(assertUpdate(structureElementCommand)));
    }
    /**
     * Utility method to update, cancel structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return cancelled structure elements (after update)
     */
    public static StructureElement[] assertUpdateCancel(StructureElementCommandUpdate[] structureElementCommands) {
        return assertCancel(StructureElementUtil.convertElement2CommandConfirm(assertUpdate(structureElementCommands)));
    }

    /**
     * Utility method to update, reject a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return rejected structure element (after update)
     */
    public static StructureElement assertUpdateReject(StructureElementCommandUpdate structureElementCommand) {
        return assertReject(StructureElementUtil.convertElement2CommandConfirm(assertUpdate(structureElementCommand)));
    }
    /**
     * Utility method to update, reject structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return rejected structure elements (after update)
     */
    public static StructureElement[] assertUpdateReject(StructureElementCommandUpdate[] structureElementCommands) {
        return assertReject(StructureElementUtil.convertElement2CommandConfirm(assertUpdate(structureElementCommands)));
    }

    /**
     * Utility method to delete, approve a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return approved structure element (after delete)
     */
    public static StructureElement assertDeleteApprove(StructureElementCommandConfirm structureElementCommand) {
        return assertApprove(StructureElementUtil.convertElement2CommandConfirm(assertDelete(structureElementCommand)));
    }
    /**
     * Utility method to delete, approve structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return approved structure elements (after delete)
     */
    public static StructureElement[] assertDeleteApprove(StructureElementCommandConfirm[] structureElementCommands) {
        return assertApprove(StructureElementUtil.convertElement2CommandConfirm(assertDelete(structureElementCommands)));
    }

    /**
     * Utility method to delete, cancel a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return cancelled structure element (after delete)
     */
    public static StructureElement assertDeleteCancel(StructureElementCommandConfirm structureElementCommand) {
        return assertCancel(StructureElementUtil.convertElement2CommandConfirm(assertDelete(structureElementCommand)));
    }
    /**
     * Utility method to delete, cancel structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return cancelled structure elements (after delete)
     */
    public static StructureElement[] assertDeleteCancel(StructureElementCommandConfirm[] structureElementCommands) {
        return assertCancel(StructureElementUtil.convertElement2CommandConfirm(assertDelete(structureElementCommands)));
    }

    /**
     * Utility method to delete, reject a structure element and assert result.
     *
     * @param structureElementCommand structure element command
     * @return rejected structure element (after delete)
     */
    public static StructureElement assertDeleteReject(StructureElementCommandConfirm structureElementCommand) {
        return assertReject(StructureElementUtil.convertElement2CommandConfirm(assertDelete(structureElementCommand)));
    }
    /**
     * Utility method to delete, reject structure elements and assert result.
     *
     * @param structureElementCommands structure element commands
     * @return rejected structure elements (after delete)
     */
    public static StructureElement[] assertDeleteReject(StructureElementCommandConfirm[] structureElementCommands) {
        return assertReject(StructureElementUtil.convertElement2CommandConfirm(assertDelete(structureElementCommands)));
    }

}
