/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.File;
import java.net.HttpURLConnection;
import java.util.UUID;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.rest.beans.element.StructureElementCommandConfirm;
import org.openepics.names.rest.beans.element.StructureElementCommandCreate;
import org.openepics.names.rest.beans.element.StructureElementCommandUpdate;
import org.openepics.names.util.StructureCommand;
import org.openepics.names.util.StructureElementUtil;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is structures endpoint and device group.
 * </p>
 *
 * @author Lars Johansson
 */
@Testcontainers
class StructuresDeviceGroupIT {

    // note
    //     StructureElement - when, who, comment - depend on requested or processed
    //         requested, requested by, requested comment
    //         processed, processed by, processed comment
    //     if less than a second between requested and processed, then considered one entry with processed
    //     history
    //         mnemonic path does not make same sense for history
    //         (very) tricky to find mnemonic path for uuid at proper time (history)
    //         therefore empty mnemonic path for history for structure
    //         one history entry if less than one second between requested and processed, otherwise two history entries
    //     attributes for entry for operations - create, update, delete, approve, cancel, reject
    //         some set client side, others set server side
    //         client side
    //             type, uuid, parent uuid, name, mnemonic, description, comment
    //         may be set client side for test purposes
    //     order of methods in test not known
    //         therefore mnemonic values chosen to not interfere with mnemonic values in other methods
    //
    //     StructureElement extends StructureElementCommand
    //     -->
    //     for such operations that send Command objects to backend, it is also possible to send StructureElement
    //
    //     StructureElementCommand
    //	       create  -       type, parent, name, mnemonic, description, comment
    //	       update  - uuid, type, parent, name, mnemonic, description, comment
    //	       delete  - uuid, type,                                      comment
    //	       approve - uuid, type,                                      comment
    //	       cancel  - uuid, type,                                      comment
    //	       reject  - uuid, type,                                      comment
    //
    //     response status codes

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-integrationtest.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    private static UUID disciplineUuid  = null;
    private static UUID discipline2Uuid = null;

    @BeforeAll
    public static void initAll() {
        // init discipline

        StructureElementCommandCreate structureElementCommandCreate = null;
        StructureElement approvedStructureElement = null;

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "name", "Di",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineUuid = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "name", "Di2",
                "description", "comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        discipline2Uuid = approvedStructureElement.getUuid();
    }

    @Test
    void equivalenceMnemonic() {
        // purpose
        //     test mnemonic equivalence
        //
        // what
        //     read        equivalence mnemonic

        ITUtilStructureElement.assertEquivalence("Dg", "DG");
    }

    @Test
    void checkCreate() {
        // purpose
        //     test conditions for create device group
        //         not create itself
        //
        // what - combination of
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //
        // validate create
        //     type
        //     parent
        //     name
        //     mnemonic
        //     description
        //     comment

        try {
            StructureElementCommandCreate structureElementCommandCreate = new StructureElementCommandCreate();

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Cc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "CC", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Cc", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "CC", Boolean.FALSE);

            ITUtilStructureElement.assertValidate("[{asdf]", StructureCommand.CREATE, HttpURLConnection.HTTP_BAD_REQUEST);
            ITUtilStructureElement.assertCreate("[{asdf]", HttpURLConnection.HTTP_BAD_REQUEST);

            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setType(Type.DEVICEGROUP);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setParent(disciplineUuid);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setName("name");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.TRUE);

            structureElementCommandCreate.setType(null);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setType(Type.DEVICEGROUP);
            structureElementCommandCreate.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void checkCreateMnemonic() {
        // purpose
        //     test conditions for create device group
        //         not create itself
        //
        // what - combination of
        //     read        validate create
        //
        // note
        //     mnemonic
        try {
            StructureElementCommandCreate structureElementCommandCreate = new StructureElementCommandCreate();

            structureElementCommandCreate.setType(Type.DEVICEGROUP);
            structureElementCommandCreate.setParent(disciplineUuid);
            structureElementCommandCreate.setName("name");
            structureElementCommandCreate.setDescription("description");
            structureElementCommandCreate.setComment("comment");

            // mnemonic rules

            structureElementCommandCreate.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.TRUE);

            structureElementCommandCreate.setMnemonic("");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.TRUE);

            structureElementCommandCreate.setMnemonic("C");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Ccc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Cccc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Ccccc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Cccccc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Ccccccc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Cccccccc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Ccccccccc");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            // mnemonic rules (2)

            structureElementCommandCreate.setMnemonic(" ");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Grp ");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Grp");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("000");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Grp0");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic(":");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Grp:");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("Grp:   ");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("1");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("12");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("123");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("1234");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("12345");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("123456");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("1234567");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("12345678");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandCreate.setMnemonic("123456789");
            ITUtilStructureElement.assertValidate(structureElementCommandCreate, StructureCommand.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElementCommandCreate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void createApprove() {
        // purpose
        //     test create and approve system
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create in order to approve

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElementCommand structureElementCommand = null;
            StructureElement createdStructureElement = null;

            structureElementCommand = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            structureElementCommandCreate = StructureElementUtil.convertCommand2CommandCreate(structureElementCommand);
            structureElementCommandConfirm = StructureElementUtil.convertCommand2CommandConfirm(structureElementCommand);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
            UUID uuid = structureElementCommandConfirm.getUuid();

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.TRUE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.FALSE);
            structureElementCommandConfirm.setUuid(uuid);

            // approve
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(createdStructureElement.getUuid());
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void createCancel() {
        // purpose
        //     test create and cancel system
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate cancel
        //     patch       cancel structures
        //
        // note
        //     create in order to cancel

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElementCommand structureElementCommand = null;
            StructureElement createdStructureElement = null;

            structureElementCommand = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            structureElementCommandCreate = StructureElementUtil.convertCommand2CommandCreate(structureElementCommand);
            structureElementCommandConfirm = StructureElementUtil.convertCommand2CommandConfirm(structureElementCommand);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
            UUID uuid = structureElementCommandConfirm.getUuid();

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.TRUE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.FALSE);
            structureElementCommandConfirm.setUuid(uuid);

            // cancel
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(uuid);
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void createReject() {
        // purpose
        //     test create and reject system
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate reject
        //     patch       reject structures
        //
        // note
        //     create in order to reject

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElementCommand structureElementCommand = null;
            StructureElement createdStructureElement = null;

            structureElementCommand = new StructureElementCommand(
                    null, Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            structureElementCommandCreate = StructureElementUtil.convertCommand2CommandCreate(structureElementCommand);
            structureElementCommandConfirm = StructureElementUtil.convertCommand2CommandConfirm(structureElementCommand);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
            UUID uuid = structureElementCommandConfirm.getUuid();

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.TRUE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.FALSE);
            structureElementCommandConfirm.setUuid(uuid);

            // reject
            ITUtilStructureElement.assertReject(structureElementCommandConfirm);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DEVICEGROUP, "Di", Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandCreate,  StructureCommand.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(uuid);
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void checkUpdate() {
        // purpose
        //     test conditions for update device group
        //         not update
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update
        //
        // validate update
        //     uuid
        //     type
        //     parent
        //     name
        //     mnemonic
        //     description
        //     comment

        try {
            UUID uuid = null;

            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandUpdate structureElementCommandUpdate = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(approvedStructureElement);
            uuid = approvedStructureElement.getUuid();

            // validate update

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);

            structureElementCommandUpdate.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, HttpURLConnection.HTTP_NOT_FOUND);

            structureElementCommandUpdate.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);

            structureElementCommandUpdate.setType(null);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setType(Type.SYSTEM);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setType(Type.DEVICEGROUP);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);

            structureElementCommandUpdate.setParent(null);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setParent(disciplineUuid);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);

            structureElementCommandUpdate.setName(null);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setName("name");
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);

            structureElementCommandUpdate.setMnemonic("Cu");
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);

            structureElementCommandUpdate.setDescription(null);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);

            structureElementCommandUpdate.setComment(null);
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandUpdate.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElementCommandUpdate, StructureCommand.UPDATE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void updateApprove() {
        // purpose
        //     test update and approve device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     update      update structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update, approve

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandUpdate structureElementCommandUpdate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(approvedStructureElement);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(approvedStructureElement);

            structureElementCommandUpdate.setComment("comment update approve check");

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.FALSE);

            // update
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate);

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, HttpURLConnection.HTTP_CONFLICT);

            // approve
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(approvedStructureElement.getUuid());
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void updateCancel() {
        // purpose
        //     test update and cancel device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate cancel
        //     update      update structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to update, cancel

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandUpdate structureElementCommandUpdate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(approvedStructureElement);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(approvedStructureElement);

            structureElementCommandUpdate.setComment("comment update cancel check");

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.FALSE);

            // update
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate);

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.TRUE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, HttpURLConnection.HTTP_CONFLICT);

            // cancel
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(approvedStructureElement.getUuid());
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void updateReject() {
        // purpose
        //     test update and reject device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate reject
        //     update      update structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to update, reject

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandUpdate structureElementCommandUpdate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(approvedStructureElement);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(approvedStructureElement);

            structureElementCommandUpdate.setComment("comment update reject check");

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.FALSE);

            // update
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate);

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.TRUE);
            ITUtilStructureElement.assertUpdate(structureElementCommandUpdate, HttpURLConnection.HTTP_CONFLICT);

            // reject
            ITUtilStructureElement.assertReject(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandUpdate,  StructureCommand.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(approvedStructureElement.getUuid());
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void checkDelete() {
        // purpose
        //     test conditions for delete device group
        //         not delete
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete
        //
        // validate delete
        //     uuid
        //     type
        //     comment

        try {
            UUID uuid = null;

            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(approvedStructureElement);
            uuid = approvedStructureElement.getUuid();

            // validate delete

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);

            structureElementCommandConfirm.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);

            structureElementCommandConfirm.setType(null);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandConfirm.setType(Type.SYSTEM);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);

            structureElementCommandConfirm.setType(Type.DEVICEGROUP);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);

            structureElementCommandConfirm.setComment(null);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);

            structureElementCommandConfirm.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void deleteApprove() {
        // purpose
        //     test delete and approve device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     delete      delete structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete, approve

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(approvedStructureElement);

            structureElementCommandConfirm.setComment("comment delete approve check");

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.FALSE);

            // delete
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.TRUE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);

            // approve
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.APPROVE, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(approvedStructureElement.getUuid());
            ITUtilStructureElement.assertApprove(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void deleteCancel() {
        // purpose
        //     test delete and cancel device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate cancel
        //     update      delete structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to delete, cancel

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(approvedStructureElement);

            structureElementCommandConfirm.setComment("comment update delete check");

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.FALSE);

            // delete
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.TRUE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);

            // cancel
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.CANCEL, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(approvedStructureElement.getUuid());
            ITUtilStructureElement.assertCancel(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void deleteReject() {
        // purpose
        //     test delete and reject device group
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate reject
        //     update      delete structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to delete, reject

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(approvedStructureElement);

            structureElementCommandConfirm.setComment("comment delete reject check");

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.FALSE);

            // delete
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.TRUE);
            ITUtilStructureElement.assertDelete(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);

            // reject
            ITUtilStructureElement.assertReject(structureElementCommandConfirm);

            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElementCommandConfirm, StructureCommand.REJECT, Boolean.FALSE);

            structureElementCommandConfirm.setUuid(null);
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, ITUtil.HTTP_UNPROCESSABLE_ENTITY);
            structureElementCommandConfirm.setUuid(UUID.randomUUID());
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, HttpURLConnection.HTTP_NOT_FOUND);
            structureElementCommandConfirm.setUuid(approvedStructureElement.getUuid());
            ITUtilStructureElement.assertReject(structureElementCommandConfirm, HttpURLConnection.HTTP_CONFLICT);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void readSearchHistoryApprove() {
        // purpose
        //     test read device group in various ways for create, approve
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement createdStructureElement  = null;
            StructureElement approvedStructureElement = null;
            int length = 0;

            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);

            // read (1)
            length = ITUtilStructureElement.assertRead("/DEVICEGROUP", 1, -1).getListSize();

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   1, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", 0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&mnemonic=Rsha",   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&mnemonic=Rsha",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&mnemonic=Rsha", 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&mnemonic=Rsha",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + createdStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",           1);
            ITUtilStructureElement.assertRead("/mnemonic/Rsha",         0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rsha",      0);

            ITUtilStructureElement.assertRead("/mnemonicPath/Di",       1, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Rsha",     0);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di-Rsha",  0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElementCommandConfirm);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DEVICEGROUP", 1, -1).getListSize());

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  1, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", 0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&mnemonic=Rsha",   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&mnemonic=Rsha",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&mnemonic=Rsha", 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&mnemonic=Rsha",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + approvedStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",          1);
            ITUtilStructureElement.assertRead("/mnemonic/Rsha",        0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rsha",     0);

            ITUtilStructureElement.assertRead("/mnemonicPath/Di",      1, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Rsha",    0);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di-Rsha", 0);

            ITUtilStructureElement.assertRead("/history/" + approvedStructureElement.getUuid().toString(), 2);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void readSearchHistoryCancel() {
        // purpose
        //     test read device group in various ways for create, cancel
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement createdStructureElement   = null;
            StructureElement cancelledStructureElement = null;
            int length = 0;

            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);

            // read (1)
            length = ITUtilStructureElement.assertRead("/DEVICEGROUP", 1, -1).getListSize();

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   1, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", 0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&mnemonic=Rshc",   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&mnemonic=Rshc",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&mnemonic=Rshc", 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&mnemonic=Rshc",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + createdStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",          1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshc",        0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshc",     0);

            ITUtilStructureElement.assertRead("/mnemonicPath/Di",      1, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Rshc",    0);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di-Rshc", 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElementCommandConfirm);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DEVICEGROUP", 1, -1).getListSize());

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", 1, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&mnemonic=Rshc",   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&mnemonic=Rshc",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&mnemonic=Rshc", 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&mnemonic=Rshc",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + cancelledStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",          1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshc",        0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshc",     0);

            ITUtilStructureElement.assertRead("/mnemonicPath/Di",      1, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Rshc",    0);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di-Rshc", 0);

            ITUtilStructureElement.assertRead("/history/" + cancelledStructureElement.getUuid().toString(), 2);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void readSearchHistoryReject() {
        // purpose
        //     test read device group in various ways for create, reject
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElementCommandCreate structureElementCommandCreate = null;
            StructureElementCommandConfirm structureElementCommandConfirm = null;
            StructureElement createdStructureElement  = null;
            StructureElement rejectedStructureElement = null;
            int length = 0;

            structureElementCommandCreate = new StructureElementCommandCreate(
                    Type.DEVICEGROUP, disciplineUuid,
                    "name", null,
                    "description", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElementCommandCreate);
            structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);

            // read (1)
            length = ITUtilStructureElement.assertRead("/DEVICEGROUP", 1, -1).getListSize();

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   1, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", 0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&mnemonic=Rshr",   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&mnemonic=Rshr",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&mnemonic=Rshr", 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&mnemonic=Rshr",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + createdStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",          1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshr",        0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshr",     0);

            ITUtilStructureElement.assertRead("/mnemonicPath/Di",      1, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Rshr",    0);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di-Rshr", 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElementCommandConfirm);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DEVICEGROUP", 1, -1).getListSize());

            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING",   0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED",  0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED", 0, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED",  1, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&mnemonic=Rshr",   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&mnemonic=Rshr",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&mnemonic=Rshr", 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&mnemonic=Rshr",  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + rejectedStructureElement.getUuid().toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/Di",          1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshr",        0);
            ITUtilStructureElement.assertRead("/mnemonic/Di-Rshr",     0);

            ITUtilStructureElement.assertRead("/mnemonicPath/Di",      1, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Rshr",    0);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di-Rshr", 0);

            ITUtilStructureElement.assertRead("/history/" + rejectedStructureElement.getUuid().toString(), 2);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void readSearchHistoryStatusLatest1() {
        // purpose
        //     test read device type in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         1 entry for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElementCommandCreate structureElementCommandCreate = null;
        StructureElement createdStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreate(structureElementCommandCreate);
        uuid = createdStructureElement.getUuid();

        // a number of entries for which the last entry has status CANCELLED
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateCancel(structureElementCommandCreate);
        uuid2 = createdStructureElement.getUuid();

        // a number of entries for which the last entry has status REJECTED
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateReject(structureElementCommandCreate);
        uuid3 = createdStructureElement.getUuid();

        // a number of entries for which the last entry has status APPROVED
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        uuid4 = createdStructureElement.getUuid();

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid.toString(),  1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid2.toString(), 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid3.toString(), 1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid4.toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/P1", 0);
            ITUtilStructureElement.assertRead("/mnemonic/C1", 0);
            ITUtilStructureElement.assertRead("/mnemonic/R1", 0);
            ITUtilStructureElement.assertRead("/mnemonic/A1", 0);

            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  1);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 2);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 2);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 2);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-P1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-C1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-R1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-A1", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void readSearchHistoryStatusLatest9() {
        // purpose
        //     test read device type in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         9 entries for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElementCommandCreate structureElementCommandCreate = null;
        StructureElementCommandUpdate structureElementCommandUpdate = null;
        StructureElement responseStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responseStructureElement);
        uuid = responseStructureElement.getUuid();

        structureElementCommandUpdate.setComment("comment 2");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 3");
        ITUtilStructureElement.assertUpdateCancel(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 4");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 5");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 6");
        ITUtilStructureElement.assertUpdateReject(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 7");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 8");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 9");
        ITUtilStructureElement.assertUpdate(structureElementCommandUpdate);

        // a number of entries for which the last entry has status CANCELLED
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responseStructureElement);
        uuid2 = responseStructureElement.getUuid();

        structureElementCommandUpdate.setComment("comment 2");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 3");
        ITUtilStructureElement.assertUpdateCancel(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 4");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 5");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 6");
        ITUtilStructureElement.assertUpdateReject(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 7");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 8");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 9");
        ITUtilStructureElement.assertUpdateCancel(structureElementCommandUpdate);

        // a number of entries for which the last entry has status REJECTED
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responseStructureElement);
        uuid3 = responseStructureElement.getUuid();

        structureElementCommandUpdate.setComment("comment 2");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 3");
        ITUtilStructureElement.assertUpdateCancel(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 4");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 5");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 6");
        ITUtilStructureElement.assertUpdateReject(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 7");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 8");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 9");
        ITUtilStructureElement.assertUpdateReject(structureElementCommandUpdate);

        // a number of entries for which the last entry has status APPROVED
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineUuid,
                "name", null,
                "description", "comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responseStructureElement);
        uuid4 = responseStructureElement.getUuid();

        structureElementCommandUpdate.setComment("comment 2");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 3");
        ITUtilStructureElement.assertUpdateCancel(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 4");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 5");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 6");
        ITUtilStructureElement.assertUpdateReject(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 7");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 8");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate.setComment("comment 9");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid.toString(),  2);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid2.toString(), 2);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid3.toString(), 2);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid4.toString(), 1);

            ITUtilStructureElement.assertRead("/mnemonic/P9", 0);
            ITUtilStructureElement.assertRead("/mnemonic/C9", 0);
            ITUtilStructureElement.assertRead("/mnemonic/R9", 0);
            ITUtilStructureElement.assertRead("/mnemonic/A9", 0);

            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  17);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 18);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 18);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 18);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-P9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-C9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-R9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di-A9", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void readSearchStatusDeletedChildren() {
        // purpose
        //     test read device group in various ways
        //         status
        //       ( latest )
        //         deleted
        //         children
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     querying for Status.APPROVED means latest approved

        StructureElementCommandCreate structureElementCommandCreate = null;
        StructureElementCommandUpdate structureElementCommandUpdate = null;
        StructureElementCommandConfirm structureElementCommandConfirm = null;
        StructureElement createdStructureElement = null;
        UUID uuid = null;
        UUID uuid2 = null;

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        uuid = createdStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateCancel(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateCancel(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateCancel(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateCancel(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateCancel(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateReject(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateReject(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateReject(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateReject(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreateReject(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreate(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreate(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreate(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreate(structureElementCommandCreate);
        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        ITUtilStructureElement.assertCreate(structureElementCommandCreate);

        String description2 = "some other description";
        String comment2 = "some other comment";
        String description3 = "more description";
        String comment3 = "more comment";

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        uuid2 = createdStructureElement.getUuid();
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description2);
        structureElementCommandUpdate.setComment(comment2);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description3);
        structureElementCommandUpdate.setComment(comment3);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteReject(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description2);
        structureElementCommandUpdate.setComment(comment2);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description3);
        structureElementCommandUpdate.setComment(comment3);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteReject(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description2);
        structureElementCommandUpdate.setComment(comment2);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description3);
        structureElementCommandUpdate.setComment(comment3);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteReject(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description2);
        structureElementCommandUpdate.setComment(comment2);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description3);
        structureElementCommandUpdate.setComment(comment3);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteReject(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description2);
        structureElementCommandUpdate.setComment(comment2);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(createdStructureElement);
        structureElementCommandUpdate.setDescription(description3);
        structureElementCommandUpdate.setComment(comment3);
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteReject(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteApprove(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteApprove(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteApprove(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteApprove(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDeleteApprove(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, discipline2Uuid,
                "name", null,
                "description", "comment");
        createdStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        structureElementCommandConfirm = StructureElementUtil.convertElement2CommandConfirm(createdStructureElement);
        ITUtilStructureElement.assertDelete(structureElementCommandConfirm);

        // 60 device group entries

        try {
            // from first structure element
            assertNotNull(uuid);

            // read & search
            //     /{type}
            //     type, statuses, deleted, uuid, parent, name, mnemonic, mnemonicEquivalence, mnemonicPath, description
            //     combination
            ITUtilStructureElement.assertRead("/DEVICEGROUP?mnemonicPath=Di2",                                                  30);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&mnemonicPath=Di2",                                  5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=APPROVED&mnemonicPath=Di2",                                20);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=CANCELLED&mnemonicPath=Di2",                                0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=REJECTED&mnemonicPath=Di2",                                 5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?statuses=PENDING&statuses=APPROVED&mnemonicPath=Di2",               25);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&mnemonicPath=Di2",                                    15);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=PENDING&mnemonicPath=Di2",                    0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=APPROVED&mnemonicPath=Di2",                  15);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=CANCELLED&mnemonicPath=Di2",                  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=REJECTED&mnemonicPath=Di2",                   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=false&statuses=PENDING&statuses=APPROVED&mnemonicPath=Di2", 15);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&mnemonicPath=Di2",                                     15);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=PENDING&mnemonicPath=Di2",                     5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=APPROVED&mnemonicPath=Di2",                    5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=CANCELLED&mnemonicPath=Di2",                   0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=REJECTED&mnemonicPath=Di2",                    5);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?deleted=true&statuses=PENDING&statuses=APPROVED&mnemonicPath=Di2",  10);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + discipline2Uuid.toString(),    0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid.toString(),               1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + uuid2.toString(),              2);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?parent=" + discipline2Uuid.toString(), 45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?parent=" + uuid.toString(),             0);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?name=na",                  0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?name=na_",                 0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?name=na__",               45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?name=na___",               0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?name=na%",                45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?name=name",               45, -1);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?mnemonicEquivalence=A__",  0);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?mnemonic=A__",             0);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?description=desc",         0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?description=desc%",       35, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?description=sc",           0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?description=sc%",          0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?description=%sc",          0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?description=%sc%",        45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?description=description", 35, -1);

            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=test who",         45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=test",              0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=who",               0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=test%",            45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=%who",             45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=%est%",            45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=%wh%",             45, -1);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=wh%",               0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=asdf",              0);
            ITUtilStructureElement.assertRead("/DEVICEGROUP?who=%asdf%",            0);

            // children
            //     /children/{uuid}
            //     uuid, type, statuses, deleted, name, mnemonic, mnemonicEquivalence, mnemonicPath, description
            ITUtilStructureElement.assertRead("/children/" + uuid.toString(),                                   0);
            ITUtilStructureElement.assertRead("/children/" + discipline2Uuid.toString(),                       15, -1);
            ITUtilStructureElement.assertRead("/children/" + uuid.toString()            + "?type=DEVICEGROUP",  0);
            ITUtilStructureElement.assertRead("/children/" + discipline2Uuid.toString() + "?type=DEVICEGROUP",  0);
            ITUtilStructureElement.assertRead("/children/" + uuid.toString()            + "?type=DISCIPLINE",   0);
            ITUtilStructureElement.assertRead("/children/" + discipline2Uuid.toString() + "?type=DISCIPLINE",  15, -1);

            // mnemonic
            //     /mnemonic/{mnemonic}
            ITUtilStructureElement.assertRead("/mnemonic/A",        0);
            ITUtilStructureElement.assertRead("/mnemonic/A__",      0);
            ITUtilStructureElement.assertRead("/mnemonic/AG_",      0);
            ITUtilStructureElement.assertRead("/mnemonic/AG1",      0);
            ITUtilStructureElement.assertRead("/mnemonic/Di2",      1);

            // mnemonic path
            //     /mnemonicPath/{mnemonicPath}
            ITUtilStructureElement.assertRead("/mnemonicPath/D",    0);
            ITUtilStructureElement.assertRead("/mnemonicPath/D%",  16, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/D__", 16, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di_", 16, -1);
            ITUtilStructureElement.assertRead("/mnemonicPath/Di2", 16);

            // history
            //     /history/{uuid}
            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),            2);
            ITUtilStructureElement.assertRead("/history/" + discipline2Uuid.toString(), 2);

            //     /history?parameter=value
            //         parameter must be available
            ITUtilStructureElement.assertRead("/history?type=DEVICEGROUP&uuid=" + uuid.toString(),       2);
            ITUtilStructureElement.assertRead("/history?type=DEVICEGROUP&uuid=" + uuid2.toString(),      8);
            ITUtilStructureElement.assertRead("/history?type=DEVICEGROUP&uuid=" + discipline2Uuid.toString(), 0);
            ITUtilStructureElement.assertRead("/history?uuid=" + uuid.toString(),       2);
            ITUtilStructureElement.assertRead("/history?uuid=" + uuid2.toString(),      8);
            ITUtilStructureElement.assertRead("/history?uuid=" + disciplineUuid.toString(), 2);
            ITUtilStructureElement.assertRead("/history?parent=" + uuid.toString(),  0);
            ITUtilStructureElement.assertRead("/history?parent=" + disciplineUuid.toString(),  32);
            ITUtilStructureElement.assertRead("/history?name=na%",                  120, -1);
            ITUtilStructureElement.assertRead("/history?mnemonic=AF_",              0);
            ITUtilStructureElement.assertRead("/history?mnemonicEquivalence=_A_",   0);
            ITUtilStructureElement.assertRead("/history?description=%other%",       40);
            ITUtilStructureElement.assertRead("/history?who=%wh%",                  120, -1);

            ITUtilStructureElement.assertRead("/history?type=DEVICEGROUP&uuid=" + uuid2.toString() + "&description=more%", 8);

            ITUtilStructureElement.assertExists(Type.DEVICEGROUP, "Di2", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

}
