/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker.complex;

import java.io.File;
import java.util.UUID;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.openepics.names.docker.ITUtil;
import org.openepics.names.docker.ITUtilNameElement;
import org.openepics.names.docker.ITUtilStructureElement;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.NameElementCommand;
import org.openepics.names.rest.beans.element.NameElementCommandCreate;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.rest.beans.element.StructureElementCommandCreate;
import org.openepics.names.util.NameCommand;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is names endpoint and rules for disciplines P&ID and Scientific in particular.
 * In practice, this class contains tests for instance index, en masse.
 * </p>
 *
 * @author Lars Johansson
 *
 * @see NamingConventionUtil
 */
@SuppressWarnings("unused")
@Testcontainers
class NamesInstanceIndexIT {

    // note
    //     disciplines
    //         P&ID
    //         Scientific
    //     mnemonic path P&ID numeric
    //     see NamingConventionUtil
    //
    //     NameElement extends NameElementCommand, StructureElement extends StructureElementCommand
    //     -->
    //     for such operations that send Command objects to backend, it is also possible to send
    //     NameElement, StructureElement

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-integrationtest.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    private static UUID systemGroupAcc  = null;
    private static UUID systemRFQ       = null;
    private static UUID subsystem010PRL = null;
    private static UUID subsystem010    = null;
    private static UUID subsystemN1U1   = null;

    private static UUID disciplineCryo  = null;
    private static UUID disciplineEMR   = null;
    private static UUID disciplineHVAC  = null;
    private static UUID disciplineProc  = null;
    private static UUID disciplineSC    = null;
    private static UUID disciplineVac   = null;
    private static UUID disciplineWtrC  = null;
    private static UUID disciplineBMD   = null;

    private static UUID deviceGroupCryo = null;
    private static UUID deviceGroupEMR  = null;
    private static UUID deviceGroupHVAC = null;
    private static UUID deviceGroupProc = null;
    private static UUID deviceGroupSC   = null;
    private static UUID deviceGroupVac  = null;
    private static UUID deviceGroupWtrC = null;
    private static UUID deviceGroupBMD  = null;

    private static UUID deviceType_Cryo_FS  = null;
    private static UUID deviceType_Cryo_IOC = null;
    private static UUID deviceType_Cryo_RFA = null;
    private static UUID deviceType_Cryo_TT  = null;

    private static UUID deviceType_EMR_FS   = null;
    private static UUID deviceType_EMR_IOC  = null;
    private static UUID deviceType_EMR_RFA  = null;
    private static UUID deviceType_EMR_TT   = null;

    private static UUID deviceType_HVAC_FS  = null;
    private static UUID deviceType_HVAC_IOC = null;
    private static UUID deviceType_HVAC_RFA = null;
    private static UUID deviceType_HVAC_TT  = null;

    private static UUID deviceType_Proc_FS  = null;
    private static UUID deviceType_Proc_IOC = null;
    private static UUID deviceType_Proc_RFA = null;
    private static UUID deviceType_Proc_TT  = null;

    private static UUID deviceType_SC_FS    = null;
    private static UUID deviceType_SC_IOC   = null;
    private static UUID deviceType_SC_RFA   = null;
    private static UUID deviceType_SC_TT    = null;

    private static UUID deviceType_Vac_FS   = null;
    private static UUID deviceType_Vac_IOC  = null;
    private static UUID deviceType_Vac_RFA  = null;
    private static UUID deviceType_Vac_TT   = null;

    private static UUID deviceType_WtrC_FS  = null;
    private static UUID deviceType_WtrC_IOC = null;
    private static UUID deviceType_WtrC_RFA = null;
    private static UUID deviceType_WtrC_TT  = null;

    private static UUID deviceType_BMD_FS   = null;
    private static UUID deviceType_BMD_IOC  = null;
    private static UUID deviceType_BMD_RFA  = null;
    private static UUID deviceType_BMD_TT   = null;

    @BeforeAll
    public static void initAll() {
        // init system group, system, subsystem, discipline, device group, device type

        StructureElementCommandCreate structureElementCommandCreate = null;
        StructureElement approvedStructureElement = null;

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SYSTEMGROUP, null,
                "Accelerator", "Acc",
                "The ESS Linear Accelerator", "approved by alfio");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        systemGroupAcc = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SYSTEM, systemGroupAcc,
                "Radio Frequency Quadrupole", "RFQ",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        systemRFQ = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SUBSYSTEM, systemRFQ,
                "01 Phase Reference Line", "010PRL",
                "empty", "Approved by Daniel Piso");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        subsystem010PRL = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SUBSYSTEM, systemRFQ,
                "RFQ-010", "010",
                "empty", "Approved by Daniel Piso");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        subsystem010 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SUBSYSTEM, systemRFQ,
                "Power switch board 01", "N1U1",
                "Electrical power cabinets", "Approved by Daniel Piso");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        subsystemN1U1 = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Cryogenics", "Cryo",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineCryo = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Electromagnetic Resonators", "EMR",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineEMR = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Heating, Cooling and Air Conditioning", "HVAC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineHVAC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "General Process Control", "Proc",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineProc = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Software Controllers", "SC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineSC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Vacuum", "Vac",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineVac = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Water Cooling", "WtrC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineWtrC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Beam Magnets and Deflectors", "BMD",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineBMD = approvedStructureElement.getUuid();


        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineCryo,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupCryo = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineEMR,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupEMR = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineHVAC,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupHVAC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineProc,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupProc = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineSC,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupSC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineVac,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupVac = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineWtrC,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupWtrC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineBMD,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupBMD = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupCryo,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Cryo_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupCryo,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Cryo_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupCryo,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Cryo_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupCryo,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Cryo_TT = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupEMR,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_EMR_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupEMR,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_EMR_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupEMR,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_EMR_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupEMR,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_EMR_TT = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupHVAC,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_HVAC_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupHVAC,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_HVAC_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupHVAC,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_HVAC_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupHVAC,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_HVAC_TT = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupProc,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Proc_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupProc,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Proc_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupProc,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Proc_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupProc,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Proc_TT = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupSC,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_SC_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupSC,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_SC_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupSC,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_SC_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupSC,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_SC_TT = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupVac,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Vac_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupVac,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Vac_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupVac,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Vac_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupVac,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Vac_TT = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupWtrC,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_WtrC_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupWtrC,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_WtrC_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupWtrC,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_WtrC_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupWtrC,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_WtrC_TT = approvedStructureElement.getUuid();

        // ----------------------------------------------------------------------------------------------------

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupBMD,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_BMD_FS = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupBMD,
                "Input Output Controller", "IOC",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_BMD_IOC = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupBMD,
                "RF Antenna", "RFA",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_BMD_RFA = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupBMD,
                "Temperature Transmitter", "TT",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_BMD_TT = approvedStructureElement.getUuid();
    }

  @Test
  void checkCreateInstanceIndex_Cryo() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline Cryo
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_Cryo_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_Cryo_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

  @Test
  void checkCreateInstanceIndex_EMR() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline EMR
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_EMR_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_EMR_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

  @Test
  void checkCreateInstanceIndex_HVAC() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline HVAC
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_HVAC_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_HVAC_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

  @Test
  void checkCreateInstanceIndex_Proc() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline Proc
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_Proc_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_Proc_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

  @Test
  void checkCreateInstanceIndex_SC() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline SC
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_SC_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_SC_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

  @Test
  void checkCreateInstanceIndex_Vac() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline Vac
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_Vac_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_Vac_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

  @Test
  void checkCreateInstanceIndex_WtrC() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline WtrC
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_WtrC_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_WtrC_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

  @Test
  void checkCreateInstanceIndex_BMD() {
      // purpose
      //     test conditions for create name
      //         not create itself
      //
      // what - combination of
      //     read        validate create
      //
      // note
      //     discipline BMD
      //     instance index

      NameElementCommandCreate nameElementCommandCreate = new NameElementCommandCreate();
      nameElementCommandCreate.setParentSystemStructure(subsystem010PRL);
      nameElementCommandCreate.setDescription("description");
      nameElementCommandCreate.setComment("comment");

      // ----------------------------------------------------------------------------------------------------

      nameElementCommandCreate.setParentDeviceStructure(deviceType_BMD_IOC);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);

      nameElementCommandCreate.setParentDeviceStructure(deviceType_BMD_RFA);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "051",       Boolean.TRUE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00",        Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0000000",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "00000000",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000000000", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12",        Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1234567",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "12345678",  Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123456789", Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000a",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ab",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abc",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "000ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123a",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ab",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abc",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123abcd",   Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123A",      Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123AB",     Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABC",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "123ABCD",   Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1",         Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "01",        Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "001",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0001",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "0110",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10",        Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "100",       Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "1000",      Boolean.TRUE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "10001",     Boolean.FALSE);

      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "",          Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, " ",         Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "Idx",       Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abcdef",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "abc123",    Boolean.FALSE);
      ITUtilNameElement.assertValidate(nameElementCommandCreate, NameCommand.CREATE, "a!",        Boolean.FALSE);
  }

}
