/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker.complex;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.File;
import java.util.UUID;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.openepics.names.docker.ITUtil;
import org.openepics.names.docker.ITUtilNameElement;
import org.openepics.names.docker.ITUtilStructureElement;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.NameElement;
import org.openepics.names.rest.beans.element.NameElementCommand;
import org.openepics.names.rest.beans.element.NameElementCommandConfirm;
import org.openepics.names.rest.beans.element.NameElementCommandCreate;
import org.openepics.names.rest.beans.element.NameElementCommandUpdate;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommandCreate;
import org.openepics.names.util.NameCommand;
import org.openepics.names.util.NameElementUtil;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is names endpoint.
 * In particular, purpose is to test arrays of names.
 * </p>
 *
 * @author Lars Johansson
 */
@Testcontainers
class NamesMultipleIT {

    // names for each of
    //     system group                       + description + comment
    //     system group + device type + index + description + comment
    //     system                             + description + comment
    //     system       + device type + index + description + comment
    //     subsystem                          + description + comment
    //     subsystem    + device type + index + description + comment

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-integrationtest.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    private static UUID systemGroupAcc     = null;
    private static UUID systemRFQ          = null;
    private static UUID subsystem010PRL    = null;

    private static UUID disciplineCryo     = null;
    private static UUID deviceGroupCryo    = null;
    private static UUID deviceType_Cryo_FS = null;

    @BeforeAll
    public static void initAll() {
        // init system group, system, subsystem, discipline, device group, device type

        StructureElementCommandCreate structureElementCommandCreate = null;
        StructureElement approvedStructureElement = null;

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SYSTEMGROUP, null,
                "Accelerator", "Acc",
                "The ESS Linear Accelerator", "approved by alfio");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        systemGroupAcc = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SYSTEM, systemGroupAcc,
                "Radio Frequency Quadrupole", "RFQ",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        systemRFQ = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.SUBSYSTEM, systemRFQ,
                "01 Phase Reference Line", "010PRL",
                "empty", "Approved by Daniel Piso");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        subsystem010PRL = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DISCIPLINE, null,
                "Cryogenics", "Cryo",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        disciplineCryo = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICEGROUP, disciplineCryo,
                "empty", null,
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceGroupCryo = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(
                Type.DEVICETYPE, deviceGroupCryo,
                "Flow Switch", "FS",
                "empty", "empty");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        deviceType_Cryo_FS = approvedStructureElement.getUuid();
    }

    @Test
    void checkCreate() {
        // purpose
        //     test conditions for create name
        //         not create itself
        //
        // what - combination of
        //     read        exists name
        //     read        is legacy name
        //     read        is valid to create
        //     read        validate create
        //
        // note
        //     with and without index

        try {
            NameElementCommandCreate[] nameElementCommandsCreate = null;
            NameElementCommandUpdate[] nameElementCommandsUpdate = null;
            NameElementCommandConfirm[] nameElementCommandsConfirm = null;
            NameElementCommand[] nameElementCommands = null;
            UUID uuid = null;
            String value = null;

            nameElementCommands = new NameElementCommand[] {
                    new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "011", "checkCreate 1.3.011", "checkCreate 1.3.011"),
                    new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "012", "checkCreate 1.3.012", "checkCreate 1.3.012"),
                    new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "013", "checkCreate 1.3.013", "checkCreate 1.3.013"),
                    new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "011", "checkCreate 2.3.011", "checkCreate 2.3.011"),
                    new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "012", "checkCreate 2.3.012", "checkCreate 2.3.012"),
                    new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "013", "checkCreate 2.3.013", "checkCreate 2.3.013"),
                    new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "011", "checkCreate 3.3.011", "checkCreate 3.3.011"),
                    new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "012", "checkCreate 3.3.012", "checkCreate 3.3.012"),
                    new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "013", "checkCreate 3.3.013", "checkCreate 3.3.013")
            };
            nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
            nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
            nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

            ITUtilNameElement.assertExists("RFQ:Cryo-FS-011", Boolean.FALSE);
            ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-011", Boolean.FALSE);
            ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-011", Boolean.TRUE);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
            ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
            ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

            uuid = nameElementCommandsCreate[3].getParentSystemStructure();
            nameElementCommandsCreate[3].setParentSystemStructure(null);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
            nameElementCommandsCreate[3].setParentSystemStructure(uuid);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);

            uuid = nameElementCommandsCreate[3].getParentDeviceStructure();
            nameElementCommandsCreate[3].setParentDeviceStructure(null);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
            nameElementCommandsCreate[3].setParentDeviceStructure(uuid);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);

            value = nameElementCommandsCreate[3].getIndex();
            nameElementCommandsCreate[3].setIndex(null);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
            nameElementCommandsCreate[3].setIndex(value);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);

            value = nameElementCommandsCreate[3].getDescription();
            nameElementCommandsCreate[3].setDescription(null);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
            nameElementCommandsCreate[3].setDescription(value);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);

            value = nameElementCommandsCreate[3].getComment();
            nameElementCommandsCreate[3].setComment(null);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
            nameElementCommandsCreate[3].setComment(value);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void create() {
        // purpose
        //     test create name
        //
        // what - combination of
        //     create      create names
        //     read        exists name
        //     read        is legacy name
        //     read        is valid to create
        //     read        validate create
        //
        // note
        //     with and without index

        try {
            NameElementCommandCreate[] nameElementCommandsCreate = null;
            NameElementCommandUpdate[] nameElementCommandsUpdate = null;
            NameElementCommandConfirm[] nameElementCommandsConfirm = null;
            NameElementCommand[] nameElementCommands = null;
            NameElement[] createdNameElements = null;
            int nbrNames = -1;

            nameElementCommands = new NameElementCommand[] {
                    new NameElementCommand(null, systemGroupAcc,  null,               null,  "create 1.0.0",   "create 1.0.0"),
                    new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "021", "create 1.3.021", "create 1.3.021"),
                    new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "022", "create 1.3.022", "create 1.3.022"),
                    new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "023", "create 1.3.023", "create 1.3.023"),
                    new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "021", "create 2.3.021", "create 2.3.021"),
                    new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "022", "create 2.3.022", "create 2.3.022"),
                    new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "023", "create 2.3.023", "create 2.3.023"),
                    new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "021", "create 3.3.021", "create 3.3.021"),
                    new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "022", "create 3.3.022", "create 3.3.022"),
                    new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "023", "create 3.3.023", "create 3.3.023")
            };
            nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
            nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
            nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
            ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
            ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

            nameElementCommandsCreate = new NameElementCommandCreate[] {
                    nameElementCommandsCreate[1],
                    nameElementCommandsCreate[2],
                    nameElementCommandsCreate[3],
                    nameElementCommandsCreate[4],
                    nameElementCommandsCreate[5],
                    nameElementCommandsCreate[6],
                    nameElementCommandsCreate[7],
                    nameElementCommandsCreate[8],
                    nameElementCommandsCreate[9]
            };

            ITUtilNameElement.assertExists("Acc:Cryo-FS-023", Boolean.FALSE);
            ITUtilNameElement.assertIsLegacy("Acc:Cryo-FS-023", Boolean.FALSE);
            ITUtilNameElement.assertIsValidToCreate("Acc:Cryo-FS-023", Boolean.TRUE);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
            ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
            ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

            nbrNames = ITUtilNameElement.assertRead("?deleted=false").getListSize();

            createdNameElements = ITUtilNameElement.assertCreate(nameElementCommandsCreate);
            assertNotNull(createdNameElements);
            assertEquals(9, createdNameElements.length);

            assertEquals(nbrNames + 9, ITUtilNameElement.assertRead("?deleted=false").getListSize());

            nameElementCommandsUpdate = NameElementUtil.convertElement2CommandUpdate(createdNameElements);
            nameElementCommandsConfirm = NameElementUtil.convertElement2CommandConfirm(createdNameElements);

            ITUtilNameElement.assertExists("Acc:Cryo-FS-023", Boolean.TRUE);
            ITUtilNameElement.assertIsLegacy("Acc:Cryo-FS-023", Boolean.FALSE);
            ITUtilNameElement.assertIsValidToCreate("Acc:Cryo-FS-023", Boolean.FALSE);
            ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
            ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
            ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    void checkUpdate() {
        // purpose
        //     test conditions for update name
        //         not update
        //
        // what - combination of
        //     create      create names
        //     read        exists name
        //     read        is legacy name
        //     read        is valid to create
        //     read        validate create
        //     read        validate update
        //
        // note
        //     create in order to update
        //     validate update

        NameElementCommandCreate[] nameElementCommandsCreate = null;
        NameElementCommandUpdate[] nameElementCommandsUpdate = null;
        NameElementCommandConfirm[] nameElementCommandsConfirm = null;
        NameElementCommand[] nameElementCommands = null;
        NameElement[] createdNameElements = null;
        int nbrNames = -1;
        UUID uuid = null;
        String value = null;

        nameElementCommands = new NameElementCommand[] {
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "031", "checkUpdate 1.3.031", "checkUpdate 1.3.031"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "032", "checkUpdate 1.3.032", "checkUpdate 1.3.032"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "033", "checkUpdate 1.3.033", "checkUpdate 1.3.033"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "031", "checkUpdate 2.3.031", "checkUpdate 2.3.031"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "032", "checkUpdate 2.3.032", "checkUpdate 2.3.032"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "033", "checkUpdate 2.3.033", "checkUpdate 2.3.033"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "031", "checkUpdate 3.3.031", "checkUpdate 3.3.031"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "032", "checkUpdate 3.3.032", "checkUpdate 3.3.032"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "033", "checkUpdate 3.3.033", "checkUpdate 3.3.033")
        };
        nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
        nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
        nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

        ITUtilNameElement.assertExists("RFQ:Cryo-FS-031", Boolean.FALSE);
        ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-031", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-031", Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

        nbrNames = ITUtilNameElement.assertRead("?deleted=false").getListSize();

        createdNameElements = ITUtilNameElement.assertCreate(nameElementCommandsCreate);
        assertNotNull(createdNameElements);
        assertEquals(9, createdNameElements.length);

        assertEquals(nbrNames + 9, ITUtilNameElement.assertRead("?deleted=false").getListSize());

        nameElementCommandsUpdate = NameElementUtil.convertElement2CommandUpdate(createdNameElements);
        nameElementCommandsConfirm = NameElementUtil.convertElement2CommandConfirm(createdNameElements);

        ITUtilNameElement.assertExists("RFQ:Cryo-FS-031", Boolean.TRUE);
        ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-031", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-031", Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);

        uuid = nameElementCommandsUpdate[3].getUuid();
        nameElementCommandsUpdate[3].setUuid(null);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        nameElementCommandsUpdate[3].setUuid(uuid);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);

        uuid = nameElementCommandsUpdate[3].getParentSystemStructure();
        nameElementCommandsUpdate[3].setParentSystemStructure(null);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        nameElementCommandsUpdate[3].setParentSystemStructure(uuid);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);

        uuid = nameElementCommandsUpdate[3].getParentDeviceStructure();
        nameElementCommandsUpdate[3].setParentDeviceStructure(null);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        nameElementCommandsUpdate[3].setParentDeviceStructure(uuid);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);

        value = nameElementCommandsUpdate[3].getIndex();
        nameElementCommandsUpdate[3].setIndex(null);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        nameElementCommandsUpdate[3].setIndex(value);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);

        value = nameElementCommandsUpdate[3].getDescription();
        nameElementCommandsUpdate[3].setDescription(null);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        nameElementCommandsUpdate[3].setDescription(value);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);

        value = nameElementCommandsUpdate[3].getComment();
        nameElementCommandsUpdate[3].setComment(null);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        nameElementCommandsUpdate[3].setComment(value);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
    }

    @Test
    void update() {
        // purpose
        //     test update name
        //
        // what - combination of
        //     create      create names
        //     read        exists name
        //     read        is legacy name
        //     read        is valid to create
        //     read        validate create
        //     read        validate update
        //     update      update names
        //
        // note
        //     create in order to update

        NameElementCommandCreate[] nameElementCommandsCreate = null;
        NameElementCommandUpdate[] nameElementCommandsUpdate = null;
        NameElementCommandConfirm[] nameElementCommandsConfirm = null;
        NameElementCommand[] nameElementCommands = null;
        NameElement[] createdNameElements = null;
        NameElement[] updatedNameElements = null;
        int nbrNames = -1;

        nameElementCommands = new NameElementCommand[] {
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "041", "update 1.3.041", "update 1.3.041"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "042", "update 1.3.042", "update 1.3.042"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "043", "update 1.3.043", "update 1.3.043"),
                new NameElementCommand(null, systemRFQ,       null,               null,  "update 2.0.0",   "update 2.0.0"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "041", "update 2.3.041", "update 2.3.041"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "042", "update 2.3.042", "update 2.3.042"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "043", "update 2.3.043", "update 2.3.043"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "041", "update 3.3.041", "update 3.3.041"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "042", "update 3.3.042", "update 3.3.042"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "043", "update 3.3.043", "update 3.3.043")
        };
        nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
        nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
        nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

        nameElementCommands = new NameElementCommand[] {
                nameElementCommands[0],
                nameElementCommands[1],
                nameElementCommands[2],
                nameElementCommands[4],
                nameElementCommands[5],
                nameElementCommands[6],
                nameElementCommands[7],
                nameElementCommands[8],
                nameElementCommands[9]
        };
        nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
        nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
        nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

        ITUtilNameElement.assertExists("RFQ", Boolean.TRUE);
        ITUtilNameElement.assertIsLegacy("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

        nbrNames = ITUtilNameElement.assertRead("?deleted=false").getListSize();

        createdNameElements = ITUtilNameElement.assertCreate(nameElementCommandsCreate);
        assertNotNull(createdNameElements);
        assertEquals(9, createdNameElements.length);

        assertEquals(nbrNames + 9, ITUtilNameElement.assertRead("?deleted=false").getListSize());

        nameElementCommandsUpdate = NameElementUtil.convertElement2CommandUpdate(createdNameElements);
        nameElementCommandsConfirm = NameElementUtil.convertElement2CommandConfirm(createdNameElements);

        ITUtilNameElement.assertExists("RFQ", Boolean.TRUE);
        ITUtilNameElement.assertIsLegacy("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);

        nameElementCommandsUpdate[0].setDescription("0");
        nameElementCommandsUpdate[1].setDescription("1");
        nameElementCommandsUpdate[2].setDescription("2");
        nameElementCommandsUpdate[3].setDescription("3");
        nameElementCommandsUpdate[4].setDescription("4");
        nameElementCommandsUpdate[5].setDescription("5");
        nameElementCommandsUpdate[6].setDescription("6");
        nameElementCommandsUpdate[7].setDescription("7");
        nameElementCommandsUpdate[8].setDescription("8");

        ITUtilNameElement.assertExists("RFQ", Boolean.TRUE);
        ITUtilNameElement.assertIsLegacy("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);

        updatedNameElements = ITUtilNameElement.assertUpdate(nameElementCommandsUpdate);
        assertNotNull(updatedNameElements);
        assertEquals(9, updatedNameElements.length);

        assertEquals(nbrNames + 9, ITUtilNameElement.assertRead("?deleted=false").getListSize());

        ITUtilNameElement.assertExists("RFQ", Boolean.TRUE);
        ITUtilNameElement.assertIsLegacy("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ", Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);
    }

    @Test
    void checkDelete() {
        // purpose
        //     test conditions for delete name
        //         not delete
        //
        // what - combination of
        //     create      create names
        //     read        exists name
        //     read        is legacy name
        //     read        is valid to create
        //     read        validate create
        //     read        validate delete
        //
        // note
        //     create in order to delete
        //     validate delete
        //         uuid
        //         comment

        NameElementCommandCreate[] nameElementCommandsCreate = null;
        NameElementCommandUpdate[] nameElementCommandsUpdate = null;
        NameElementCommandConfirm[] nameElementCommandsConfirm = null;
        NameElementCommand[] nameElementCommands = null;
        NameElement[] createdNameElements = null;
        int nbrNames = -1;
        UUID uuid = null;
        String value = null;

        nameElementCommands = new NameElementCommand[] {
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "051", "checkDelete 1.3.051", "checkDelete 1.3.051"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "052", "checkDelete 1.3.052", "checkDelete 1.3.052"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "053", "checkDelete 1.3.053", "checkDelete 1.3.053"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "051", "checkDelete 2.3.051", "checkDelete 2.3.051"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "052", "checkDelete 2.3.052", "checkDelete 2.3.052"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "053", "checkDelete 2.3.053", "checkDelete 2.3.053"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "051", "checkDelete 3.3.051", "checkDelete 3.3.051"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "052", "checkDelete 3.3.052", "checkDelete 3.3.052"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "053", "checkDelete 3.3.053", "checkDelete 3.3.053")
        };
        nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
        nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
        nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

        ITUtilNameElement.assertExists("RFQ:Cryo-FS-051", Boolean.FALSE);
        ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-051", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-051", Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

        nbrNames = ITUtilNameElement.assertRead("?deleted=false").getListSize();

        createdNameElements = ITUtilNameElement.assertCreate(nameElementCommandsCreate);
        assertNotNull(createdNameElements);
        assertEquals(9, createdNameElements.length);

        assertEquals(nbrNames + 9, ITUtilNameElement.assertRead("?deleted=false").getListSize());

        nameElementCommandsCreate = NameElementUtil.convertElement2CommandCreate(createdNameElements);
        nameElementCommandsUpdate = NameElementUtil.convertElement2CommandUpdate(createdNameElements);
        nameElementCommandsConfirm = NameElementUtil.convertElement2CommandConfirm(createdNameElements);

        ITUtilNameElement.assertExists("RFQ:Cryo-FS-051", Boolean.TRUE);
        ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-051", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-051", Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);

        uuid = nameElementCommandsConfirm[3].getUuid();
        nameElementCommandsConfirm[3].setUuid(null);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);
        nameElementCommandsConfirm[3].setUuid(uuid);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);

        value = nameElementCommandsConfirm[3].getComment();
        nameElementCommandsConfirm[3].setComment(null);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);
        nameElementCommandsConfirm[3].setComment(value);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);
    }

    @Test
    void delete() {
        // purpose
        //     test delete name
        //
        // what - combination of
        //     create      create names
        //     read        exists name
        //     read        is legacy name
        //     read        is valid to create
        //     read        validate create
        //     read        validate update
        //     delete      delete names
        //
        // note
        //     create in order to delete
        //
        // names for each of
        //     system group                       + description + comment
        //     system group + device type + index + description + comment
        //     system                             + description + comment
        //     system       + device type + index + description + comment
        //     subsystem                          + description + comment
        //     subsystem    + device type + index + description + comment

        NameElementCommandCreate[] nameElementCommandsCreate = null;
        NameElementCommandUpdate[] nameElementCommandsUpdate = null;
        NameElementCommandConfirm[] nameElementCommandsConfirm = null;
        NameElementCommand[] nameElementCommands = null;
        NameElement[] createdNameElements = null;
        NameElement[] deletedNameElements = null;
        int nbrNames = -1;

        nameElementCommands = new NameElementCommand[] {
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "061", "delete 1.3.061", "delete 1.3.061"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "062", "delete 1.3.062", "delete 1.3.062"),
                new NameElementCommand(null, systemGroupAcc,  deviceType_Cryo_FS, "063", "delete 1.3.063", "delete 1.3.063"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "061", "delete 2.3.061", "delete 2.3.061"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "062", "delete 2.3.062", "delete 2.3.062"),
                new NameElementCommand(null, systemRFQ,       deviceType_Cryo_FS, "063", "delete 2.3.063", "delete 2.3.063"),
                new NameElementCommand(null, subsystem010PRL, null,               null,  "delete 3.0.0",   "delete 3.0.0"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "061", "delete 3.3.061", "delete 3.3.061"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "062", "delete 3.3.062", "delete 3.3.062"),
                new NameElementCommand(null, subsystem010PRL, deviceType_Cryo_FS, "063", "delete 3.3.063", "delete 3.3.063")
        };
        nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
        nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
        nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

        ITUtilNameElement.assertValidate(nameElementCommandsCreate, NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate, NameCommand.UPDATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

        nameElementCommands = new NameElementCommand[] {
                nameElementCommands[0],
                nameElementCommands[1],
                nameElementCommands[2],
                nameElementCommands[3],
                nameElementCommands[4],
                nameElementCommands[5],
                nameElementCommands[7],
                nameElementCommands[8],
                nameElementCommands[9]
        };
        nameElementCommandsCreate = NameElementUtil.convertCommand2CommandCreate(nameElementCommands);
        nameElementCommandsUpdate = NameElementUtil.convertCommand2CommandUpdate(nameElementCommands);
        nameElementCommandsConfirm = NameElementUtil.convertCommand2CommandConfirm(nameElementCommands);

        ITUtilNameElement.assertExists("RFQ:Cryo-FS-061", Boolean.FALSE);
        ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-061", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-061", Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);

        nbrNames = ITUtilNameElement.assertRead("?deleted=false").getListSize();

        createdNameElements = ITUtilNameElement.assertCreate(nameElementCommandsCreate);
        assertNotNull(createdNameElements);
        assertEquals(9, createdNameElements.length);

        assertEquals(nbrNames + 9, ITUtilNameElement.assertRead("?deleted=false").getListSize());

        nameElementCommandsCreate = NameElementUtil.convertElement2CommandCreate(createdNameElements);
        nameElementCommandsUpdate = NameElementUtil.convertElement2CommandUpdate(createdNameElements);
        nameElementCommandsConfirm = NameElementUtil.convertElement2CommandConfirm(createdNameElements);

        ITUtilNameElement.assertExists("RFQ:Cryo-FS-061", Boolean.TRUE);
        ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-061", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-061", Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.TRUE);

        deletedNameElements = ITUtilNameElement.assertDelete(nameElementCommandsConfirm);
        assertNotNull(deletedNameElements);
        assertEquals(9, deletedNameElements.length);

        assertEquals(nbrNames, ITUtilNameElement.assertRead("?deleted=false").getListSize());

        ITUtilNameElement.assertExists("RFQ:Cryo-FS-061", Boolean.FALSE);
        ITUtilNameElement.assertIsLegacy("RFQ:Cryo-FS-061", Boolean.FALSE);
        ITUtilNameElement.assertIsValidToCreate("RFQ:Cryo-FS-061", Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsCreate,  NameCommand.CREATE, Boolean.TRUE);
        ITUtilNameElement.assertValidate(nameElementCommandsUpdate,  NameCommand.UPDATE, Boolean.FALSE);
        ITUtilNameElement.assertValidate(nameElementCommandsConfirm, NameCommand.DELETE, Boolean.FALSE);
    }

}
