/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker.complex;

import java.io.File;
import java.util.UUID;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.openepics.names.docker.ITUtil;
import org.openepics.names.docker.ITUtilNameElement;
import org.openepics.names.docker.ITUtilStructureElement;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.NameElementCommandCreate;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommandCreate;
import org.openepics.names.rest.beans.element.StructureElementCommandUpdate;
import org.openepics.names.rest.beans.response.ResponsePageStructureElements;
import org.openepics.names.util.StructureElementUtil;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is names endpoint and rules for legacy name.
 * In practice, this class contains tests for structures and names, before and after update of structures.
 * </p>
 *
 * @author Lars Johansson
 */
@Testcontainers
class StructuresCUDUpdateIT {

    // test
    //     create update delete structures and ensure name is as expected
    //
    // setup
    //     system structure
    //         Sg
    //         Sys
    //         Sys-Sub
    //     system structure + device structure + index
    //         Sg:Di-Dt-Idx
    //         Sys:Di-Dt-Idx
    //         Sys-Sub:Di-Dt-Idx
    //
    // note
    //     namecommand
    //         cud - create update delete
    //     structurecommand
    //         cud - create update delete
    //         acr - approve cancel reject

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-integrationtest.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    // system group
    //     sg0
    private static UUID sg1 = null;
    private static UUID sg2 = null;

    // system
    private static UUID sys11, sys12, sys13 = null;
    private static UUID sys21, sys22, sys23 = null;

    // subsystem
    //     sub111, sub112, sub113
    //     sub121, sub122, sub123
    //     sub131, sub132, sub133
    //     sub211, sub212, sub213
    //     sub221, sub222, sub223
    //     sub231, sub232, sub233
    private static UUID sub132 = null;
    private static UUID sub231, sub232, sub233 = null;

    // discipline
    private static UUID di2 = null;

    // device group
    private static UUID dg21, dg22, dg23 = null;

    // device type
    private static UUID dt211, dt212, dt213 = null;
    private static UUID dt221, dt222, dt223 = null;
    private static UUID dt231, dt232, dt233 = null;

    // name
    //     n211, n212, n213
    //     n221, n222, n223
    //     n231, n232, n233

    @BeforeAll
    public static void initAll() {
        // init system group, system, subsystem, discipline, device group, device type
        //
        // test create structure and ensure name is as expected

        NameElementCommandCreate nameElementCommandCreate = null;
        StructureElementCommandCreate structureElementCommandCreate = null;
        StructureElement approvedStructureElement = null;

        String name = "name";
        String description = "description";
        String comment = "comment";

        // sg0
        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEMGROUP, null, name, null, description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        ITUtilStructureElement.assertRead("/SYSTEMGROUP", 1);
        ITUtilStructureElement.assertRead("/SYSTEM",      0);
        ITUtilStructureElement.assertRead("/SUBSYSTEM",   0);
        ITUtilNameElement.assertRead("", 0);

        // at this point
        //     system group
        //         sg0 - no mnemonic

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEMGROUP, null, name, "Sg1", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sg1 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEM, sg1, name, "Sys11", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sys11 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEM, sg1, name, "Sys12", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sys12 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEM, sg1, name, "Sys13", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sys13 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys11, name, "Sub111", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys11, name, "Sub112", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys11, name, "Sub113", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys12, name, "Sub121", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys12, name, "Sub122", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys12, name, "Sub123", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys13, name, "Sub131", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys13, name, "Sub132", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sub132 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys13, name, "Sub133", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        ITUtilStructureElement.assertRead("/SYSTEMGROUP", 2);
        ITUtilStructureElement.assertRead("/SYSTEM",      3);
        ITUtilStructureElement.assertRead("/SUBSYSTEM",   9);
        ITUtilNameElement.assertRead("", 13);


        // at this point
        //     above
        //     +
        //     system group
        //         Sg1
        //     system
        //         Sys11, Sys12, Sys13
        //     subsystem
        //         Sub111, Sub112, Sub113
        //         Sub121, Sub122, Sub123
        //         Sub131, Sub132, Sub133
        //     name (system structure only)
        //         Sg1
        //         Sys11, Sys12, Sys13
        //         Sys11-Sub111, Sys11-Sub112, Sys11-Sub113
        //         Sys12-Sub121, Sys12-Sub122, Sys12-Sub123
        //         Sys13-Sub131, Sys13-Sub132, Sys13-Sub133

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEMGROUP, null, name, "Sg2", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sg2 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEM, sg2, name, "Sys21", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sys21 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEM, sg2, name, "Sys22", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sys22 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SYSTEM, sg2, name, "Sys23", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sys23 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys21, name, "Sub211", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys21, name, "Sub212", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys21, name, "Sub213", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys22, name, "Sub221", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys22, name, "Sub222", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys22, name, "Sub223", description, comment);
        ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys23, name, "Sub231", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sub231 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys23, name, "Sub232", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sub232 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.SUBSYSTEM, sys23, name, "Sub233", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        sub233 = approvedStructureElement.getUuid();

        ITUtilStructureElement.assertRead("/SYSTEMGROUP",  3);
        ITUtilStructureElement.assertRead("/SYSTEM",       6);
        ITUtilStructureElement.assertRead("/SUBSYSTEM",   18);
        ITUtilNameElement.assertRead("", 26);

        // at this point
        //     above
        //     +
        //     system group
        //         Sg2
        //     system
        //         Sys21, Sys22, Sys23
        //     subsystem
        //         Sub211, Sub212, Sub213
        //         Sub221, Sub222, Sub223
        //         Sub231, Sub232, Sub233
        //     name (system structure only)
        //         Sg2
        //         Sys21, Sys22, Sys23
        //         Sys21-Sub211, Sys21-Sub212, Sys21-Sub213
        //         Sys22-Sub221, Sys22-Sub222, Sys22-Sub223
        //         Sys23-Sub231, Sys23-Sub232, Sys23-Sub233

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DISCIPLINE, null, name, "Di2", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        di2 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICEGROUP, di2, name, "", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dg21 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICEGROUP, di2, name, "", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dg22 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICEGROUP, di2, name, "", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dg23 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg21, name, "Dt211", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt211 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg21, name, "Dt212", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt212 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg21, name, "Dt213", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt213 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg22, name, "Dt221", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt221 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg22, name, "Dt222", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt222 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg22, name, "Dt223", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt223 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg23, name, "Dt231", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt231 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg23, name, "Dt232", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt232 = approvedStructureElement.getUuid();

        structureElementCommandCreate = new StructureElementCommandCreate(Type.DEVICETYPE, dg23, name, "Dt233", description, comment);
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElementCommandCreate);
        dt233 = approvedStructureElement.getUuid();

        ITUtilStructureElement.assertRead("/SYSTEMGROUP",  3);
        ITUtilStructureElement.assertRead("/SYSTEM",       6);
        ITUtilStructureElement.assertRead("/SUBSYSTEM",   18);
        ITUtilStructureElement.assertRead("/DISCIPLINE",   1);
        ITUtilStructureElement.assertRead("/DEVICEGROUP",  3);
        ITUtilStructureElement.assertRead("/DEVICETYPE",   9);
        ITUtilNameElement.assertRead("", 26);

        // at this point
        //     above
        //     +
        //     discipline
        //         Di2
        //     device group
        //         -, -, -
        //     device type
        //         Dt211, Dt212, Dt213
        //         Dt221, Dt222, Dt223
        //         Dt231, Dt232, Dt233

        nameElementCommandCreate = new NameElementCommandCreate(sg2, dt211, "211", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sg2, dt212, "212", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sg2, dt213, "213", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sys22, dt221, "221", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sys22, dt222, "222", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sys22, dt223, "223", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sub231, dt231, "231", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sub232, dt232, "232", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        nameElementCommandCreate = new NameElementCommandCreate(sub233, dt233, "233", description, comment);
        ITUtilNameElement.assertCreate(nameElementCommandCreate);

        ITUtilStructureElement.assertRead("/SYSTEMGROUP",  3);
        ITUtilStructureElement.assertRead("/SYSTEM",       6);
        ITUtilStructureElement.assertRead("/SUBSYSTEM",   18);
        ITUtilStructureElement.assertRead("/DISCIPLINE",   1);
        ITUtilStructureElement.assertRead("/DEVICEGROUP",  3);
        ITUtilStructureElement.assertRead("/DEVICETYPE",   9);
        ITUtilNameElement.assertRead("", 35);

        // at this point
        //     above
        //     +
        //     name (system structure + device structure + index)
        //         Sg2:Di2-Dt211-211
        //         Sg2:Di2-Dt212-212
        //         Sg2:Di2-Dt213-213
        //         Sys22:Di2-Dt221-221
        //         Sys22:Di2-Dt222-222
        //         Sys22:Di2-Dt223-223
        //         Sys23-Sub231:Di2-Dt231-231
        //         Sys23-Sub232:Di2-Dt232-232
        //         Sys23-Sub233:Di2-Dt233-233

        ITUtilNameElement.assertExists("Sg1",   Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys11", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys12", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys13", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys11-Sub111", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys11-Sub112", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys11-Sub113", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys12-Sub121", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys12-Sub122", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys12-Sub123", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys13-Sub131", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys13-Sub132", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys13-Sub133", Boolean.TRUE);

        ITUtilNameElement.assertExists("Sg2",   Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys21", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys21-Sub211", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys21-Sub212", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys21-Sub213", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22-Sub221", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22-Sub222", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22-Sub223", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub231", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub232", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub233", Boolean.TRUE);

        ITUtilNameElement.assertExists("Sg2:Di2-Dt211-211", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt212-212", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt213-213", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt221-221", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt222-222", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt223-223", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub231:Di2-Dt231-231", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-Dt232-232", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub233:Di2-Dt233-233", Boolean.TRUE);
    }

    @Test
    void updateSystemStructure() {
        // purpose
        //     test update system structure and ensure name is as expected

        ResponsePageStructureElements responsePage = null;
        StructureElementCommandUpdate structureElementCommandUpdate = null;

        responsePage = ITUtilStructureElement.assertRead("/SUBSYSTEM?uuid=" + sub132, 1);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responsePage.getList().get(0));
        structureElementCommandUpdate.setDescription("asdf");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sys13-Sub132", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys13-Su132",  Boolean.FALSE);

        structureElementCommandUpdate.setMnemonic("Su132");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sys13-Sub132", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys13-Su132",  Boolean.TRUE);

        // ----------------------------------------

        responsePage = ITUtilStructureElement.assertRead("/SYSTEM?uuid=" + sys12, 1);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responsePage.getList().get(0));
        structureElementCommandUpdate.setDescription("asdf");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sys12", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys12-Sub121", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys12-Sub122", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys12-Sub123", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sy12",  Boolean.FALSE);
        ITUtilNameElement.assertExists("Sy12-Sub121",  Boolean.FALSE);
        ITUtilNameElement.assertExists("Sy12-Sub122",  Boolean.FALSE);
        ITUtilNameElement.assertExists("Sy12-Sub123",  Boolean.FALSE);

        structureElementCommandUpdate.setMnemonic("Sy12");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sys12", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys12-Sub121", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys12-Sub122", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys12-Sub123", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sy12",  Boolean.TRUE);
        ITUtilNameElement.assertExists("Sy12-Sub121",  Boolean.TRUE);
        ITUtilNameElement.assertExists("Sy12-Sub122",  Boolean.TRUE);
        ITUtilNameElement.assertExists("Sy12-Sub123",  Boolean.TRUE);

        // ----------------------------------------

        responsePage = ITUtilStructureElement.assertRead("/SYSTEMGROUP?uuid=" + sg1, 1);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responsePage.getList().get(0));
        structureElementCommandUpdate.setDescription("asdf");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sg1", Boolean.TRUE);
        ITUtilNameElement.assertExists("S1",  Boolean.FALSE);

        structureElementCommandUpdate.setMnemonic("S1");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sg1", Boolean.FALSE);
        ITUtilNameElement.assertExists("S1",  Boolean.TRUE);
    }

    @Test
    void updateDeviceStructure() {
        // purpose
        //     test update system structure and ensure name is as expected

        ResponsePageStructureElements responsePage = null;
        StructureElementCommandUpdate structureElementCommandUpdate = null;

        responsePage = ITUtilStructureElement.assertRead("/DEVICETYPE?uuid=" + dt232, 1);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responsePage.getList().get(0));
        structureElementCommandUpdate.setDescription("asdf");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-Dt232-232", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-D232-232",  Boolean.FALSE);

        structureElementCommandUpdate.setMnemonic("D232");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-Dt232-232", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-D232-232",  Boolean.TRUE);

        // ----------------------------------------

        responsePage = ITUtilStructureElement.assertRead("/DEVICEGROUP?uuid=" + dg22, 1);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responsePage.getList().get(0));
        structureElementCommandUpdate.setDescription("asdf");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sg2:Di2-Dt211-211", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt212-212", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt213-213", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt221-221", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt222-222", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt223-223", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub231:Di2-Dt231-231", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-D232-232",  Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub233:Di2-Dt233-233", Boolean.TRUE);

        ITUtilNameElement.assertExists("Sg2:D2-Dt211-211", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sg2:D2-Dt212-212", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sg2:D2-Dt213-213", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt221-221", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt222-222", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt223-223", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub231:D2-Dt231-231", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub232:D2-D232-232",  Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub233:D2-Dt233-233", Boolean.FALSE);

        // ----------------------------------------

        responsePage = ITUtilStructureElement.assertRead("/DISCIPLINE?uuid=" + di2, 1);
        structureElementCommandUpdate = StructureElementUtil.convertElement2CommandUpdate(responsePage.getList().get(0));
        structureElementCommandUpdate.setDescription("asdf");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sg2:Di2-Dt211-211", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt212-212", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt213-213", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt221-221", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt222-222", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt223-223", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub231:Di2-Dt231-231", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-D232-232",  Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub233:Di2-Dt233-233", Boolean.TRUE);

        ITUtilNameElement.assertExists("Sg2:D2-Dt211-211", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sg2:D2-Dt212-212", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sg2:D2-Dt213-213", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt221-221", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt222-222", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt223-223", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub231:D2-Dt231-231", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub232:D2-D232-232",  Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub233:D2-Dt233-233", Boolean.FALSE);

        structureElementCommandUpdate.setMnemonic("D2");
        ITUtilStructureElement.assertUpdateApprove(structureElementCommandUpdate);

        ITUtilNameElement.assertExists("Sg2:Di2-Dt211-211", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt212-212", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sg2:Di2-Dt213-213", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt221-221", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt222-222", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys22:Di2-Dt223-223", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub231:Di2-Dt231-231", Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub232:Di2-D232-232",  Boolean.FALSE);
        ITUtilNameElement.assertExists("Sys23-Sub233:Di2-Dt233-233", Boolean.FALSE);

        ITUtilNameElement.assertExists("Sg2:D2-Dt211-211", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:D2-Dt212-212", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sg2:D2-Dt213-213", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt221-221", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt222-222", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys22:D2-Dt223-223", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub231:D2-Dt231-231", Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub232:D2-D232-232",  Boolean.TRUE);
        ITUtilNameElement.assertExists("Sys23-Sub233:D2-Dt233-233", Boolean.TRUE);
    }

}
