/*
 * Copyright (C) 2021 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.util;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.Date;
import java.util.UUID;

import org.junit.jupiter.api.Test;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.rest.beans.element.StructureElement;
import org.openepics.names.rest.beans.element.StructureElementCommand;
import org.openepics.names.rest.beans.element.StructureElementCommandConfirm;
import org.openepics.names.rest.beans.element.StructureElementCommandCreate;
import org.openepics.names.rest.beans.element.StructureElementCommandUpdate;

/**
 * Unit tests for StructureElementUtil class.
 *
 * @author Lars Johansson
 *
 * @see StructureElementUtil
 */
class StructureElementUtilTest {

    /**
     * Test of get structure element for content.
     */
    @Test
    void getStructureElementContent() {
        UUID uuid = UUID.randomUUID();
        Type type = Type.DEVICETYPE;
        UUID parent = UUID.randomUUID();
        String name = "gh";
        String mnemonic = "jk";
        String mnemonicPath = "zx";
        Integer level = 3;
        String description = "cv";
        Status status = Status.PENDING;
        Boolean latest = Boolean.FALSE;
        Boolean deleted = Boolean.FALSE;
        Date when = new Date();
        String who = "bn";
        String comment = "ml";

        StructureElement structureElement = StructureElementUtil.getStructureElement(
                uuid,
                type,
                parent,
                name, mnemonic, mnemonicPath, level,
                description, status, latest, deleted,
                when, who, comment);

        assertNotNull(structureElement);
        assertEquals(uuid, structureElement.getUuid());
        assertEquals(type, structureElement.getType());
        assertEquals(parent, structureElement.getParent());
        assertEquals(name, structureElement.getName());
        assertEquals(mnemonic, structureElement.getMnemonic());
        assertEquals(mnemonicPath, structureElement.getMnemonicPath());
        assertEquals(level, structureElement.getLevel());
        assertEquals(description, structureElement.getDescription());
        assertEquals(status, structureElement.getStatus());
        assertEquals(latest, structureElement.isLatest());
        assertEquals(deleted, structureElement.isDeleted());
        assertEquals(when, structureElement.getWhen());
        assertEquals(who, structureElement.getWho());
        assertEquals(comment, structureElement.getComment());
    }

    // ----------------------------------------------------------------------------------------------------

    @Test
    void equals() {
        StructureElement se1 = new StructureElement();
        StructureElement se2 = new StructureElement();

        //        Field[] fields = se1.getClass().getFields();
        //        Field[] declaredFields = se1.getClass().getDeclaredFields();
        //
        //        Field[] superclass_fields = se1.getClass().getSuperclass().getFields();
        //        Field[] superclass_declaredFields = se1.getClass().getSuperclass().getDeclaredFields();
        //
        //        for (Field field : fields) {
        //            System.out.println("fields.field.getName:                    " + field.getName());
        //        }
        //        System.out.println("-----------------------------------------");
        //        for (Field field : declaredFields) {
        //            System.out.println("declaredFields.field.getName:            " + field.getName());
        //        }
        //        System.out.println("-----------------------------------------");
        //        for (Field field : superclass_fields) {
        //            System.out.println("superclass_fields.field.getName:         " + field.getName());
        //        }
        //        System.out.println("-----------------------------------------");
        //        for (Field field : superclass_declaredFields) {
        //            System.out.println("superclass_declaredFields.field.getName: " + field.getName());
        //        }

        assertTrue(se1.equals(se2));
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Test of convert structure element command for create to structure element command.
     */
    @Test
    void convertCommandCreate2Command() {
        Type type = Type.SUBSYSTEM;
        UUID uuid2 = UUID.randomUUID();
        String str1 = "qwer";
        String str2 = "asdf";
        String str3 = "zxcv";
        String str4 = "tyui";

        StructureElementCommandCreate commandCreate = new StructureElementCommandCreate(type, uuid2, str1, str2, str3, str4);
        StructureElementCommand command = StructureElementUtil.convertCommandCreate2Command(commandCreate);

        assertNull(command.getUuid());
        assertEquals(type, command.getType());
        assertEquals(uuid2, command.getParent());
        assertEquals(str1, command.getName());
        assertEquals(str2, command.getMnemonic());
        assertEquals(str3, command.getDescription());
        assertEquals(str4, command.getComment());
    }

    /**
     * Test of convert structure element command for update to structure element command.
     */
    @Test
    void convertCommandUpdate2Command() {
        UUID uuid1 = UUID.randomUUID();
        Type type = Type.SUBSYSTEM;
        UUID uuid2 = UUID.randomUUID();
        String str1 = "qwer";
        String str2 = "asdf";
        String str3 = "zxcv";
        String str4 = "tyui";

        StructureElementCommandUpdate commandUpdate = new StructureElementCommandUpdate(uuid1, type, uuid2, str1, str2, str3, str4);
        StructureElementCommand command = StructureElementUtil.convertCommandUpdate2Command(commandUpdate);

        assertEquals(uuid1, command.getUuid());
        assertEquals(type, command.getType());
        assertEquals(uuid2, command.getParent());
        assertEquals(str1, command.getName());
        assertEquals(str2, command.getMnemonic());
        assertEquals(str3, command.getDescription());
        assertEquals(str4, command.getComment());
    }

    /**
     * Test of convert structure element command for confirm to structure element command.
     */
    @Test
    void convertCommandConfirm2Command() {
        UUID uuid1 = UUID.randomUUID();
        Type type = Type.SUBSYSTEM;
        String str4 = "tyui";

        StructureElementCommandConfirm commandConfirm = new StructureElementCommandConfirm(uuid1, type, str4);
        StructureElementCommand command = StructureElementUtil.convertCommandConfirm2Command(commandConfirm);

        assertEquals(uuid1, command.getUuid());
        assertEquals(type, command.getType());
        assertNull(command.getParent());
        assertNull(command.getName());
        assertNull(command.getMnemonic());
        assertNull(command.getDescription());
        assertEquals(str4,  command.getComment());
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Test of convert structure element command to structure element command for create.
     */
    @Test
    void convertCommand2CommandCreate() {
        UUID uuid1 = UUID.randomUUID();
        Type type = Type.SUBSYSTEM;
        UUID uuid2 = UUID.randomUUID();
        String str1 = "qwer";
        String str2 = "asdf";
        String str3 = "zxcv";
        String str4 = "tyui";

        StructureElementCommand command = new StructureElementCommand(uuid1, type, uuid2, str1, str2, str3, str4);
        StructureElementCommandCreate commandCreate = StructureElementUtil.convertCommand2CommandCreate(command);

        assertNotNull(commandCreate);
        assertEquals(type, commandCreate.getType());
        assertEquals(uuid2, commandCreate.getParent());
        assertEquals(str1, commandCreate.getName());
        assertEquals(str2, commandCreate.getMnemonic());
        assertEquals(str3, commandCreate.getDescription());
        assertEquals(str4, commandCreate.getComment());
    }

    /**
     * Test of convert structure element command to structure element command for update.
     */
    @Test
    void convertCommand2CommandUpdate() {
        UUID uuid1 = UUID.randomUUID();
        Type type = Type.SUBSYSTEM;
        UUID uuid2 = UUID.randomUUID();
        String str1 = "qwer";
        String str2 = "asdf";
        String str3 = "zxcv";
        String str4 = "tyui";

        StructureElementCommand command = new StructureElementCommand(uuid1, type, uuid2, str1, str2, str3, str4);
        StructureElementCommandUpdate commandUpdate = StructureElementUtil.convertCommand2CommandUpdate(command);

        assertNotNull(commandUpdate);
        assertEquals(uuid1, commandUpdate.getUuid());
        assertEquals(type, commandUpdate.getType());
        assertEquals(uuid2, commandUpdate.getParent());
        assertEquals(str1, commandUpdate.getName());
        assertEquals(str2, commandUpdate.getMnemonic());
        assertEquals(str3, commandUpdate.getDescription());
        assertEquals(str4, commandUpdate.getComment());
    }

    /**
     * Test of convert structure element command to structure element command for confirm.
     */
    @Test
    void convertCommand2CommandConfirm() {
        UUID uuid1 = UUID.randomUUID();
        Type type = Type.SUBSYSTEM;
        UUID uuid2 = UUID.randomUUID();
        String str1 = "qwer";
        String str2 = "asdf";
        String str3 = "zxcv";
        String str4 = "tyui";

        StructureElementCommand command = new StructureElementCommand(uuid1, type, uuid2, str1, str2, str3, str4);
        StructureElementCommandConfirm commandConfirm = StructureElementUtil.convertCommand2CommandConfirm(command);

        assertNotNull(commandConfirm);
        assertEquals(uuid1, commandConfirm.getUuid());
        assertEquals(type, commandConfirm.getType());
        assertEquals(str4, commandConfirm.getComment());
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Test of convert structure element to structure element command for create.
     */
    @Test
    void convertElement2CommandCreate() {
        UUID uuid = UUID.randomUUID();
        Type type = Type.DEVICETYPE;
        UUID parent = UUID.randomUUID();
        String name = "gh";
        String mnemonic = "jk";
        String mnemonicPath = "zx";
        Integer level = 3;
        String description = "cv";
        Status status = Status.PENDING;
        Boolean latest = Boolean.FALSE;
        Boolean deleted = Boolean.FALSE;
        Date when = new Date();
        String who = "bn";
        String comment = "ml";

        StructureElement element = StructureElementUtil.getStructureElement(
                uuid,
                type,
                parent,
                name, mnemonic, mnemonicPath, level,
                description, status, latest, deleted,
                when, who, comment);

        StructureElementCommandCreate commandCreate = StructureElementUtil.convertElement2CommandCreate(element);

        assertNotNull(commandCreate);
        assertEquals(type, commandCreate.getType());
        assertEquals(parent, commandCreate.getParent());
        assertEquals(name, commandCreate.getName());
        assertEquals(mnemonic, commandCreate.getMnemonic());
        assertEquals(description, commandCreate.getDescription());
        assertEquals(comment, commandCreate.getComment());
    }

    /**
     * Test of convert structure element to structure element command for update.
     */
    @Test
    void convertElement2CommandUpdate() {
        UUID uuid = UUID.randomUUID();
        Type type = Type.DEVICETYPE;
        UUID parent = UUID.randomUUID();
        String name = "gh";
        String mnemonic = "jk";
        String mnemonicPath = "zx";
        Integer level = 3;
        String description = "cv";
        Status status = Status.PENDING;
        Boolean latest = Boolean.FALSE;
        Boolean deleted = Boolean.FALSE;
        Date when = new Date();
        String who = "bn";
        String comment = "ml";

        StructureElement element = StructureElementUtil.getStructureElement(
                uuid,
                type,
                parent,
                name, mnemonic, mnemonicPath, level,
                description, status, latest, deleted,
                when, who, comment);

        StructureElementCommandUpdate commandUpdate = StructureElementUtil.convertElement2CommandUpdate(element);

        assertNotNull(commandUpdate);
        assertEquals(uuid, commandUpdate.getUuid());
        assertEquals(type, commandUpdate.getType());
        assertEquals(parent, commandUpdate.getParent());
        assertEquals(name, commandUpdate.getName());
        assertEquals(mnemonic, commandUpdate.getMnemonic());
        assertEquals(description, commandUpdate.getDescription());
        assertEquals(comment, commandUpdate.getComment());
    }

    /**
     * Test of convert structure element to structure element command for confirm.
     */
    @Test
    void convertElement2CommandConfirm() {
        UUID uuid = UUID.randomUUID();
        Type type = Type.DEVICETYPE;
        UUID parent = UUID.randomUUID();
        String name = "gh";
        String mnemonic = "jk";
        String mnemonicPath = "zx";
        Integer level = 3;
        String description = "cv";
        Status status = Status.PENDING;
        Boolean latest = Boolean.FALSE;
        Boolean deleted = Boolean.FALSE;
        Date when = new Date();
        String who = "bn";
        String comment = "ml";

        StructureElement element = StructureElementUtil.getStructureElement(
                uuid,
                type,
                parent,
                name, mnemonic, mnemonicPath, level,
                description, status, latest, deleted,
                when, who, comment);

        StructureElementCommandConfirm commandConfirm = StructureElementUtil.convertElement2CommandConfirm(element);

        assertNotNull(commandConfirm);
        assertEquals(uuid, commandConfirm.getUuid());
        assertEquals(type, commandConfirm.getType());
        assertEquals(comment, commandConfirm.getComment());
    }

}
